@@ -41,7 +41,7 @@
 
 #include "lyr_freetype.h"
 #endif
-
+#include <synfig/cairo_renddesc.h>
 #include <pango/pangocairo.h>
 
 using namespace std;
@@ -93,27 +93,34 @@ Layer_Freetype::Layer_Freetype()
 {
 	face=0;
 
-	size=Vector(0.25,0.25);
-	text=_("Text Layer");
-	color=Color::black();
-	origin=Vector(0,0);
-	orient=Vector(0.5,0.5);
-	compress=1.0;
-	vcompress=1.0;
-	weight=WEIGHT_NORMAL;
-	style=PANGO_STYLE_NORMAL;
-	family="Sans Serif";
-	use_kerning=true;
-	grid_fit=false;
+	param_size=ValueBase(Vector(0.25,0.25));
+	param_text=ValueBase(_("Text Layer"));
+	param_color=ValueBase(Color::black());
+	param_origin=ValueBase(Vector(0,0));
+	param_orient=ValueBase(Vector(0.5,0.5));
+	param_compress=ValueBase(Real(1.0));
+	param_vcompress=ValueBase(Real(1.0));
+	param_weight=ValueBase(WEIGHT_NORMAL);
+	param_style=ValueBase(PANGO_STYLE_NORMAL);
+	param_family=ValueBase("Sans Serif");
+	param_use_kerning=ValueBase(true);
+	param_grid_fit=ValueBase(false);
+	param_invert=ValueBase(false);
+	param_font=ValueBase("");
+
 	old_version=false;
+
 	set_blend_method(Color::BLEND_COMPOSITE);
 	needs_sync_=true;
 
+	synfig::String family=param_family.get(synfig::String());
+	int style=param_style.get(int());
+	int weight=param_weight.get(int());
+
 	new_font(family,style,weight);
 
-	invert=false;
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 Layer_Freetype::~Layer_Freetype()
@@ -335,6 +342,7 @@ void fss2path(char *path, FSSpec *fss)
 bool
 Layer_Freetype::new_face(const String &newfont)
 {
+	synfig::String font=param_font.get(synfig::String());
 	int error;
 	FT_Long face_index=0;
 
@@ -463,23 +471,67 @@ Layer_Freetype::set_param(const String & param, const ValueBase &value)
 		return true;
 	}
 */
-	IMPORT_PLUS(family,new_font(family,style,weight));
-	IMPORT_PLUS(weight,new_font(family,style,weight));
-	IMPORT_PLUS(style,new_font(family,style,weight));
-	IMPORT_PLUS(size, if(old_version){size/=2.0;} needs_sync_=true );
-	IMPORT_PLUS(text,needs_sync_=true);
-	IMPORT_PLUS(origin,needs_sync_=true);
-	IMPORT_PLUS(color, { if (color.get_a() == 0) { if (converted_blend_) {
+	IMPORT_VALUE_PLUS(param_family,
+		{
+			synfig::String family=param_family.get(synfig::String());
+			int style=param_style.get(int());
+			int weight=param_weight.get(int());
+			new_font(family,style,weight);
+		}
+		);
+		
+	IMPORT_VALUE_PLUS(param_weight,
+		{
+			synfig::String family=param_family.get(synfig::String());
+			int style=param_style.get(int());
+			int weight=param_weight.get(int());
+			new_font(family,style,weight);
+		}
+		);
+	IMPORT_VALUE_PLUS(param_style,
+		{
+			synfig::String family=param_family.get(synfig::String());
+			int style=param_style.get(int());
+			int weight=param_weight.get(int());
+			new_font(family,style,weight);
+		}
+		);
+	IMPORT_VALUE_PLUS(param_size,
+		{
+			if(old_version)
+			{
+				synfig::Vector size=param_size.get(synfig::Vector());
+				size/=2.0;
+				param_size.set(size);
+			}
+			needs_sync_=true;
+		}
+		);
+	IMPORT_VALUE_PLUS(param_text,needs_sync_=true);
+	IMPORT_VALUE_PLUS(param_origin,needs_sync_=true);
+	IMPORT_VALUE_PLUS(param_color,
+		{
+			Color color=param_color.get(Color());
+			if (color.get_a() == 0)
+			{
+				if (converted_blend_)
+				{
 					set_blend_method(Color::BLEND_ALPHA_OVER);
-					color.set_a(1); } else transparent_color_ = true; } });
-	IMPORT(invert);
-	IMPORT_PLUS(orient,needs_sync_=true);
-	IMPORT_PLUS(compress,needs_sync_=true);
-	IMPORT_PLUS(vcompress,needs_sync_=true);
-	IMPORT_PLUS(use_kerning,needs_sync_=true);
-	IMPORT_PLUS(grid_fit,needs_sync_=true);
-
-	IMPORT_AS(origin,"pos");
+					color.set_a(1);
+					param_color.set(color);
+				} else transparent_color_ = true;
+			}
+		}
+		);
+	IMPORT_VALUE(param_invert);
+	IMPORT_VALUE_PLUS(param_orient,needs_sync_=true);
+	IMPORT_VALUE_PLUS(param_compress,needs_sync_=true);
+	IMPORT_VALUE_PLUS(param_vcompress,needs_sync_=true);
+	IMPORT_VALUE_PLUS(param_use_kerning,needs_sync_=true);
+	IMPORT_VALUE_PLUS(param_grid_fit,needs_sync_=true);
+	
+	if(param=="pos")
+		return set_param("origin", value);
 
 	return Layer_Composite::set_param(param,value);
 }
@@ -487,20 +539,20 @@ Layer_Freetype::set_param(const String & param, const ValueBase &value)
 ValueBase
 Layer_Freetype::get_param(const String& param)const
 {
-	EXPORT(font);
-	EXPORT(family);
-	EXPORT(style);
-	EXPORT(weight);
-	EXPORT(size);
-	EXPORT(text);
-	EXPORT(color);
-	EXPORT(origin);
-	EXPORT(orient);
-	EXPORT(compress);
-	EXPORT(vcompress);
-	EXPORT(use_kerning);
-	EXPORT(grid_fit);
-	EXPORT(invert);
+	EXPORT_VALUE(param_font);
+	EXPORT_VALUE(param_family);
+	EXPORT_VALUE(param_style);
+	EXPORT_VALUE(param_weight);
+	EXPORT_VALUE(param_size);
+	EXPORT_VALUE(param_text);
+	EXPORT_VALUE(param_color);
+	EXPORT_VALUE(param_origin);
+	EXPORT_VALUE(param_orient);
+	EXPORT_VALUE(param_compress);
+	EXPORT_VALUE(param_vcompress);
+	EXPORT_VALUE(param_use_kerning);
+	EXPORT_VALUE(param_grid_fit);
+	EXPORT_VALUE(param_invert);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -608,8 +660,9 @@ Layer_Freetype::sync()
 inline Color
 Layer_Freetype::color_func(const Point &point_ __attribute__ ((unused)), int quality __attribute__ ((unused)), float supersample __attribute__ ((unused)))const
 {
+	bool invert=param_invert.get(bool());
 	if (invert)
-		return color;
+		return param_color.get(Color());
 	else
 		return Color::alpha();
 }
@@ -634,18 +687,25 @@ Layer_Freetype::get_color(Context context, const synfig::Point &pos)const
 bool
 Layer_Freetype::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	bool use_kerning=param_use_kerning.get(bool());
+	bool grid_fit=param_grid_fit.get(bool());
+	bool invert=param_invert.get(bool());
+	Color color=param_color.get(Color());
+	synfig::Point origin=param_origin.get(Point());
+	synfig::Vector orient=param_orient.get(Vector());
+	
 	static synfig::RecMutex freetype_mutex;
 
 	if(needs_sync_)
 		const_cast<Layer_Freetype*>(this)->sync();
 
 	int error;
-	Vector size(Layer_Freetype::size*2);
+	Vector size(Layer_Freetype::param_size.get(synfig::Vector())*2);
 
 	if(!context.accelerated_render(surface,quality,renddesc,cb))
 		return false;
 
-	if(is_disabled() || text.empty())
+	if(is_disabled() || param_text.get(synfig::String()).empty())
 		return true;
 
 	// If there is no font loaded, just bail
@@ -655,7 +715,7 @@ Layer_Freetype::accelerated_render(Context context,Surface *surface,int quality,
 		return true;
 	}
 
-	String text(Layer_Freetype::text);
+	String text(Layer_Freetype::param_text.get(synfig::String()));
 	if(text=="@_FILENAME_@" && get_canvas() && !get_canvas()->get_file_name().empty())
 	{
 		text=basename(get_canvas()->get_file_name());
@@ -696,8 +756,8 @@ Layer_Freetype::accelerated_render(Context context,Surface *surface,int quality,
 	const float xerror(abs(size[0]*pw)/(float)face->size->metrics.x_ppem/1.13f/0.996);
 	const float yerror(abs(size[1]*ph)/(float)face->size->metrics.y_ppem/1.13f/0.996);
 	//synfig::info("xerror=%f, yerror=%f",xerror,yerror);
-	const float compress(Layer_Freetype::compress*xerror);
-	const float vcompress(Layer_Freetype::vcompress*yerror);
+	const float compress(Layer_Freetype::param_compress.get(Real())*xerror);
+	const float vcompress(Layer_Freetype::param_vcompress.get(Real())*yerror);
 
 	if(error)
 	{
@@ -921,47 +981,62 @@ Layer_Freetype::accelerated_render(Context context,Surface *surface,int quality,
 
 ////
 bool
-Layer_Freetype::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Layer_Freetype::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
-	const Point	tl(renddesc.get_tl());
-	const Point br(renddesc.get_br());
+	int style=param_style.get(int());
+	int weight=param_weight.get(int());
+	synfig::Vector size=param_size.get(Vector());
+	synfig::Real compress=param_compress.get(synfig::Real());
+	synfig::Real vcompress=param_vcompress.get(synfig::Real());
+	bool invert=param_invert.get(bool());
+	Color color=param_color.get(Color());
+	synfig::Point origin=param_origin.get(Point());
+	synfig::Vector orient=param_orient.get(Vector());
+	synfig::String font=param_font.get(synfig::String());
+	synfig::String text=param_text.get(synfig::String());
 	
-	const int	w(renddesc.get_w());
-	const int	h(renddesc.get_h());
-	
-	// Width and Height of a pixel
-	const Real pw = (br[0] - tl[0]) / w;
-	const Real ph = (br[1] - tl[1]) / h;
-	
-	const double sx(1/pw);
-	const double sy(1/ph);
-	const double tx((-tl[0]+origin[0])*sx);
-	const double ty((-tl[1]+origin[1])*sy);
-
 	if(!is_solid_color())
 	{
 		// Initially render what's behind us
-		if(!context.accelerated_cairorender(surface,quality,renddesc,cb))
+		if(!context.accelerated_cairorender(cr,quality,renddesc,cb))
 		{
 			if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
 			return false;
 		}
 	}
+	
+	RendDesc workdesc(renddesc);
+	
+	// Untransform the render desc
+	if(!cairo_renddesc_untransform(cr, workdesc))
+		return false;
 
+	// New expanded workdesc values
+	const int ww=workdesc.get_w();
+	const int wh=workdesc.get_h();
+	const double wtlx=workdesc.get_tl()[0];
+	const double wtly=workdesc.get_tl()[1];
+	const double wpw=workdesc.get_pw();
+	const double wph=workdesc.get_ph();
+	const double wsx=1/wpw;
+	const double wsy=1/wph;
+	const double wtx=(-wtlx+origin[0])*wsx;
+	const double wty=(-wtly+origin[1])*wsy;
+	
 	// Cairo context
 	cairo_surface_t* subimage;
 	cairo_surface_t* inverted;
-	subimage=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, w, h);
+	subimage=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, ww, wh);
 	cairo_t* subcr=cairo_create(subimage);
 	cairo_t* invertcr;
 	if(invert)
 	{
-		inverted=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, w, h);
+		inverted=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, ww, wh);
 		invertcr=cairo_create(inverted);
 		cairo_set_source_rgba(invertcr, color.get_r(), color.get_g(), color.get_b(), color.get_a());
 		cairo_paint_with_alpha(invertcr, get_amount());
 	}
-
+	
 	// Pango
 	PangoLayout *layout;
 	PangoFontDescription *font_description;
@@ -971,8 +1046,8 @@ Layer_Freetype::accelerated_cairorender(Context context,cairo_surface_t *surface
 	pango_font_description_set_weight (font_description, PangoWeight(weight));
 	pango_font_description_set_style (font_description, PangoStyle(style));
 	// The size is scaled to match Software render size (remove the scale?)
-	float sizex=1.75*fabs(size[0])*fabs(sx);
-	float sizey=1.75*fabs(size[1])*fabs(sy);
+	float sizex=1.75*fabs(size[0])*fabs(wsx);
+	float sizey=1.75*fabs(size[1])*fabs(wsy);
 	float vscale=sizey/sizex;
 	pango_font_description_set_absolute_size (font_description, sizex * PANGO_SCALE );
 	
@@ -987,14 +1062,14 @@ Layer_Freetype::accelerated_cairorender(Context context,cairo_surface_t *surface
 		pango_layout_set_alignment(layout, PANGO_ALIGN_RIGHT);
 	else
 		pango_layout_set_alignment(layout, PANGO_ALIGN_CENTER);
-		
+	
 	pango_layout_set_single_paragraph_mode(layout, false);
-
+	
 	// Calculate the logical and ink rectangles of the layout before add spacing
 	PangoRectangle ink_layout, logical_layout;
 	PangoRectangle ink_rect, logical_rect;
 	pango_layout_get_pixel_extents(layout, &ink_layout, &logical_layout);
-
+	
 	// Spacing
 	// Horizontal
 	PangoAttrList* attrlist=pango_attr_list_new();
@@ -1010,48 +1085,49 @@ Layer_Freetype::accelerated_cairorender(Context context,cairo_surface_t *surface
 	if(total_lines>1)
 		vspace=vspace_total/(total_lines-1);
 	pango_layout_set_spacing(layout, vspace*PANGO_SCALE);
-
+	
 	// Recalculate extents due to spacing changes
 	pango_layout_get_pixel_extents(layout, &ink_layout, &logical_layout);
-
+	
 	// Render text
 	cairo_save(subcr);
 	cairo_set_source_rgba(subcr, color.get_r(), color.get_g(), color.get_b(), color.get_a());
 	cairo_scale(subcr, 1.0, vscale);
 	pango_cairo_update_layout(subcr, layout);
-	cairo_move_to(subcr, tx-logical_layout.width*orient[0], (ty-(logical_layout.height+vspace_total)*vscale*orient[1])/vscale);
+	cairo_move_to(subcr, wtx-logical_layout.width*orient[0], (wty-(logical_layout.height+vspace_total)*vscale*orient[1])/vscale);
 	pango_cairo_show_layout(subcr, layout);
-
+	
 	// Debug ink and logical lines
 	if(0)
 	{
-		pango_layout_get_pixel_extents(layout, &ink_rect, &logical_rect);	
+		pango_layout_get_pixel_extents(layout, &ink_rect, &logical_rect);
 		// Render logical and ink rectangles
 		cairo_save(subcr);
 		cairo_set_source_rgb(subcr, 0.0, 1.0, 0.0);
 		cairo_set_line_width(subcr, 1.0);
-		cairo_rectangle(subcr, tx+ink_rect.x-0.5-logical_layout.width*orient[0],
-							ty+ink_rect.y-0.5-(logical_layout.height+vspace_total)*orient[1],
-							ink_rect.width,
-							ink_rect.height);
+		cairo_rectangle(subcr, wtx+ink_rect.x-0.5-logical_layout.width*orient[0],
+						wty+ink_rect.y-0.5-(logical_layout.height+vspace_total)*orient[1],
+						ink_rect.width,
+						ink_rect.height);
 		cairo_stroke(subcr);
 		cairo_restore(subcr);
-
+		
 		cairo_save(subcr);
 		cairo_set_line_width(subcr, 1.0);
 		cairo_set_source_rgb(subcr, 0.0, 0.0, 1.0);
-		cairo_rectangle(subcr, tx+logical_rect.x-0.5-logical_layout.width*orient[0],
-							ty+logical_rect.y-0.5-(logical_layout.height+vspace_total)*orient[1],
-							logical_rect.width,
-							logical_rect.height);
+		cairo_rectangle(subcr, wtx+logical_rect.x-0.5-logical_layout.width*orient[0],
+						wty+logical_rect.y-0.5-(logical_layout.height+vspace_total)*orient[1],
+						logical_rect.width,
+						logical_rect.height);
 		cairo_stroke(subcr);
-		cairo_move_to(subcr, tx+2, ty);
-		cairo_arc(subcr, tx, ty, 2.0, 0, 2*3.141516);
+		cairo_move_to(subcr, wtx+2, wty);
+		cairo_arc(subcr, wtx, wty, 2.0, 0, 2*3.141516);
 		cairo_fill(subcr);
 		cairo_restore(subcr);
 	}
 	cairo_restore(subcr);
 	
+	cairo_save(cr);
 	// Render the text on the target surface with the proper operator
 	if(invert)
 	{
@@ -1059,23 +1135,24 @@ Layer_Freetype::accelerated_cairorender(Context context,cairo_surface_t *surface
 		cairo_set_operator(invertcr, CAIRO_OPERATOR_DEST_OUT);
 		cairo_paint_with_alpha(invertcr, get_amount());
 	}
-	cairo_t* cr=cairo_create(surface);
-	cairo_save(cr);
+	// Need to scale down to user coordinates before pass to cr
+	cairo_translate(cr, wtlx, wtly);
+	cairo_scale(cr, wpw, wph);
 	if(invert)
 		cairo_set_source_surface(cr, inverted, 0, 0);
 	else
 		cairo_set_source_surface(cr, subimage, 0, 0);
 	if(is_solid_color())
 	{
-			cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
-			cairo_paint(cr);
+		cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+		cairo_paint(cr);
 	}
 	else
 	{
 		cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
 	}
 	cairo_restore(cr);
-
+	
 	// Destroy and return
 	cairo_surface_destroy(subimage);
 	cairo_destroy(subcr);
@@ -1084,7 +1161,6 @@ Layer_Freetype::accelerated_cairorender(Context context,cairo_surface_t *surface
 		cairo_surface_destroy(inverted);
 		cairo_destroy(invertcr);
 	}
-	cairo_destroy(cr);
 	pango_attr_list_unref(attrlist);
 	g_object_unref (layout);
 	pango_font_description_free (font_description);
