@@ -41,6 +41,7 @@
 #include <synfig/value.h>
 #include <synfig/valuenode.h>
 #include <synfig/segment.h>
+#include <synfig/cairo_renddesc.h>
 
 #include <cstring>
 #include <ETL/pen>
@@ -80,23 +81,27 @@ inline void clamp(synfig::Vector &v)
 
 Layer_Bevel::Layer_Bevel():
 	Layer_Composite	(0.75,Color::BLEND_ONTO),
-	softness(0.1),
-	type(Blur::FASTGAUSSIAN),
-	color1(Color::white()),
-	color2(Color::black()),
-	depth(0.2)
+	param_type(ValueBase(int(Blur::FASTGAUSSIAN))),
+	param_softness (ValueBase(Real(0.1))),
+	param_color1(ValueBase(Color::white())),
+	param_color2(ValueBase(Color::black())),
+	param_depth(ValueBase(Real(0.2)))
 {
-	angle=Angle::deg(135);
+	param_angle=ValueBase(Angle::deg(135));
 	calc_offset();
-	use_luma=false;
-	solid=false;
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	param_use_luma=ValueBase(false);
+	param_solid=ValueBase(false);
+
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 void
 Layer_Bevel::calc_offset()
 {
+	synfig::Angle angle=param_angle.get(synfig::Angle());
+	synfig::Real depth=param_depth.get(synfig::Real());
+	
 	offset[0]=Angle::cos(angle).get()*depth;
 	offset[1]=Angle::sin(angle).get()*depth;
 
@@ -107,14 +112,20 @@ Layer_Bevel::calc_offset()
 bool
 Layer_Bevel::set_param(const String &param, const ValueBase &value)
 {
-	IMPORT_PLUS(softness,softness=softness>0?softness:0);
-	IMPORT(color1);
-	IMPORT(color2);
-	IMPORT_PLUS(depth,calc_offset());
-	IMPORT_PLUS(angle,calc_offset());
-	IMPORT(type);
-	IMPORT(use_luma);
-	IMPORT(solid);
+	IMPORT_VALUE_PLUS(param_softness,
+		{
+			synfig::Real softness=param_softness.get(Real());
+			softness=softness>0?softness:0;
+			param_softness.set(softness);
+		}
+		);
+	IMPORT_VALUE(param_color1);
+	IMPORT_VALUE(param_color2);
+	IMPORT_VALUE_PLUS(param_depth,calc_offset());
+	IMPORT_VALUE_PLUS(param_angle,calc_offset());
+	IMPORT_VALUE(param_type);
+	IMPORT_VALUE(param_use_luma);
+	IMPORT_VALUE(param_solid);
 
 	return Layer_Composite::set_param(param,value);
 }
@@ -122,14 +133,14 @@ Layer_Bevel::set_param(const String &param, const ValueBase &value)
 ValueBase
 Layer_Bevel::get_param(const String &param)const
 {
-	EXPORT(type);
-	EXPORT(softness);
-	EXPORT(color1);
-	EXPORT(color2);
-	EXPORT(depth);
-	EXPORT(angle);
-	EXPORT(use_luma);
-	EXPORT(solid);
+	EXPORT_VALUE(param_type);
+	EXPORT_VALUE(param_softness);
+	EXPORT_VALUE(param_color1);
+	EXPORT_VALUE(param_color2);
+	EXPORT_VALUE(param_depth);
+	EXPORT_VALUE(param_angle);
+	EXPORT_VALUE(param_use_luma);
+	EXPORT_VALUE(param_solid);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -140,6 +151,11 @@ Layer_Bevel::get_param(const String &param)const
 Color
 Layer_Bevel::get_color(Context context, const Point &pos)const
 {
+	synfig::Real softness=param_softness.get(Real());
+	int type=param_type.get(int());
+	synfig::Color color1=param_color1.get(synfig::Color());
+	synfig::Color color2=param_color2.get(synfig::Color());
+	
 	const Vector size(softness,softness);
 	Point blurpos = Blur(size,type)(pos);
 
@@ -163,6 +179,13 @@ Layer_Bevel::get_color(Context context, const Point &pos)const
 bool
 Layer_Bevel::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	synfig::Real softness=param_softness.get(Real());
+	int type=param_type.get(int());
+	synfig::Color color1=param_color1.get(synfig::Color());
+	synfig::Color color2=param_color2.get(synfig::Color());
+	bool use_luma=param_use_luma.get(bool());
+	bool solid=param_solid.get(bool());
+	
 	int x,y;
 	SuperCallback stageone(cb,0,5000,10000);
 	SuperCallback stagetwo(cb,5000,10000,10000);
@@ -352,21 +375,31 @@ Layer_Bevel::accelerated_render(Context context,Surface *surface,int quality, co
 
 ////
 bool
-Layer_Bevel::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Layer_Bevel::accelerated_cairorender(Context context, cairo_t *cr,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	synfig::Real softness=param_softness.get(Real());
+	int type=param_type.get(int());
+	synfig::Color color1=param_color1.get(synfig::Color());
+	synfig::Color color2=param_color2.get(synfig::Color());
+	bool use_luma=param_use_luma.get(bool());
+	bool solid=param_solid.get(bool());
+
 	int x,y;
 	SuperCallback stageone(cb,0,5000,10000);
 	SuperCallback stagetwo(cb,5000,10000,10000);
 	
-	const int	w = renddesc.get_w(),
-	h = renddesc.get_h();
-	const Real	pw = renddesc.get_pw(),
-	ph = renddesc.get_ph();
-	const Vector size(softness,softness);
-	
 	RendDesc	workdesc(renddesc);
-	cairo_surface_t		*worksurface;
+	cairo_surface_t		*worksurface, *surface;
 	etl::surface<float> blurred;
+
+	// Untransform the render desc
+	if(!cairo_renddesc_untransform(cr, workdesc))
+		return false;
+
+	const int	w = workdesc.get_w(), h = workdesc.get_h();
+	const double	pw = workdesc.get_pw(),	ph = workdesc.get_ph();
+	const Point tl=workdesc.get_tl();
+	const Vector size(softness,softness);
 	
 	//callbacks depend on how long the blur takes
 	if(size[0] || size[1])
@@ -389,7 +422,6 @@ Layer_Bevel::accelerated_cairorender(Context context,cairo_surface_t *surface,in
 	}
 	
 	//expand the working surface to accommodate the blur
-	
 	//the expanded size = 1/2 the size in each direction rounded up
 	int	halfsizex = (int) (abs(size[0]*.5/pw) + 3),
 	halfsizey = (int) (abs(size[1]*.5/ph) + 3);
@@ -443,11 +475,24 @@ Layer_Bevel::accelerated_cairorender(Context context,cairo_surface_t *surface,in
 			break;
 		}
 	}
-	
+
+	// New expanded workdesc values
+	const int ww=workdesc.get_w();
+	const int wh=workdesc.get_h();
+	const double wtlx=workdesc.get_tl()[0];
+	const double wtly=workdesc.get_tl()[1];
+	const double wpw=workdesc.get_pw();
+	const double wph=workdesc.get_ph();
+
 	// setup the worksurface
-	worksurface=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, workdesc.get_w(), workdesc.get_h());
+	worksurface=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, ww, wh);
+	surface=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, w, h);
+	cairo_t* subcr=cairo_create(worksurface);
+	cairo_scale(subcr, 1/wpw, 1/wph);
+	cairo_translate(subcr, -wtlx, -wtly);
+
 	//render the background onto the expanded surface
-	if(!context.accelerated_cairorender(worksurface,quality,workdesc,&stageone))
+	if(!context.accelerated_cairorender(subcr,quality,workdesc,&stageone))
 		return false;
 	
 	// Extract the CairoSurface from the cairo_surface_t
@@ -487,12 +532,13 @@ Layer_Bevel::accelerated_cairorender(Context context,cairo_surface_t *surface,in
 	
 	//blur the image
 	Blur(size,type,&stagetwo)(blurred,workdesc.get_br()-workdesc.get_tl(),blurred);
-		
-	int u = halfsizex+abs(offset_u), v = halfsizey+abs(offset_v);
-	for(y=0;y<renddesc.get_h();y++,v++)
+
+	// Add the bevel effect
+	int u = halfsizex+abs(offset_u), v = halfsizey+abs(offset_v);	
+	for(y=0;y<h;y++,v++)
 	{
 		u = halfsizex+abs(offset_u);
-		for(x=0;x<renddesc.get_w();x++,u++)
+		for(x=0;x<w;x++,u++)
 		{
 			Real alpha(0);
 			Color shade;
@@ -540,25 +586,33 @@ Layer_Bevel::accelerated_cairorender(Context context,cairo_surface_t *surface,in
 			
 			
 			if(shade.get_a())
-			{
-				cairosurface[y][x]=CairoColor(Color::blend(shade,Color(cairoworksurface[v][u].demult_alpha()),get_amount(),get_blend_method())).premult_alpha();
-			}
-			else cairosurface[y][x] = cairoworksurface[v][u];
+				cairosurface[y][x]=CairoColor::blend(CairoColor(shade),cairoworksurface[v][u].demult_alpha(),get_amount(),get_blend_method()).premult_alpha();
+			else
+				cairosurface[y][x] = cairoworksurface[v][u];
 		}
 	}
 	cairoworksurface.unmap_cairo_image();
 	cairosurface.unmap_cairo_image();
 	cairo_surface_destroy(worksurface);
-	
+
+	// Now lets put the result in the output surface
+	cairo_save(cr);
+	cairo_translate(cr, tl[0], tl[1]);
+	cairo_scale(cr, pw, ph);
+	cairo_set_source_surface(cr, surface, 0, 0);
+	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+	cairo_paint(cr);
+	cairo_restore(cr);
+	cairo_surface_destroy(surface);
+
 	if(cb && !cb->amount_complete(10000,10000))
 	{
-		//if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Renderer Failure",__LINE__));
+		if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Renderer Failure",__LINE__));
 		return false;
 	}
 	
 	return true;
 }
-
 ////
 
 Layer::Vocab
@@ -607,6 +661,9 @@ Layer_Bevel::get_param_vocab(void)const
 Rect
 Layer_Bevel::get_full_bounding_rect(Context context)const
 {
+	synfig::Real softness=param_softness.get(Real());
+	synfig::Real depth=param_depth.get(synfig::Real());
+
 	if(is_disabled())
 		return context.get_full_bounding_rect();
 
