@@ -40,6 +40,7 @@
 #include <synfig/surface.h>
 #include <synfig/valuenode.h>
 #include <ETL/calculus>
+#include <synfig/cairo_renddesc.h>
 
 #endif
 
@@ -149,17 +150,23 @@ find_closest_to_bline(bool fast, const std::vector<synfig::BLinePoint>& bline,co
 inline void
 CurveWarp::sync()
 {
+	std::vector<synfig::BLinePoint> bline(param_bline.get_list().begin(),param_bline.get_list().end());
+	Point start_point=param_start_point.get(Point());
+	Point end_point=param_end_point.get(Point());
+	
 	curve_length_=calculate_distance(bline);
 	perp_ = (end_point - start_point).perp().norm();
 }
 
 CurveWarp::CurveWarp():
-	origin(0,0),
-	perp_width(1),
-	start_point(-2.5,-0.5),
-	end_point(2.5,-0.3),
-	fast(true)
+	param_origin(ValueBase(Point(0,0))),
+	param_perp_width(ValueBase(Real(1))),
+	param_start_point(ValueBase(Point(-2.5,-0.5))),
+	param_end_point(ValueBase(Point(2.5,-0.3))),
+	param_bline(ValueBase(std::vector<synfig::BLinePoint>())),
+	param_fast(ValueBase(true))
 {
+	std::vector<synfig::BLinePoint> bline;
 	bline.push_back(BLinePoint());
 	bline.push_back(BLinePoint());
 	bline[0].set_vertex(Point(-2.5,0));
@@ -168,15 +175,23 @@ CurveWarp::CurveWarp():
 	bline[1].set_tangent(Point(1, -0.1));
 	bline[0].set_width(1.0f);
 	bline[1].set_width(1.0f);
-
+	param_bline.set(bline);
 	sync();
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 inline Point
 CurveWarp::transform(const Point &point_, Real *dist, Real *along, int quality)const
 {
+	std::vector<synfig::BLinePoint> bline(param_bline.get_list().begin(),param_bline.get_list().end());
+	Point start_point=param_start_point.get(Point());
+	Point end_point=param_end_point.get(Point());
+	Point origin=param_origin.get(Point());
+	bool fast=param_fast.get(bool());
+	Real perp_width=param_perp_width.get(Real());
+
 	Vector tangent;
 	Vector diff;
 	Point p1;
@@ -345,21 +360,15 @@ CurveWarp::hit_check(synfig::Context context, const synfig::Point &point)const
 bool
 CurveWarp::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT(origin);
-	IMPORT(start_point);
-	IMPORT(end_point);
-	IMPORT(fast);
-	IMPORT(perp_width);
+	IMPORT_VALUE(param_origin);
+	IMPORT_VALUE(param_start_point);
+	IMPORT_VALUE(param_end_point);
+	IMPORT_VALUE(param_fast);
+	IMPORT_VALUE(param_perp_width);
+	IMPORT_VALUE_PLUS(param_bline, sync());
 
-	if(param=="bline" && value.get_type()==ValueBase::TYPE_LIST)
-	{
-		bline=value;
-		sync();
-
-		return true;
-	}
-
-	IMPORT_AS(origin,"offset");
+	if(param=="offset")
+		return set_param("origin", value);
 
 	return false;
 }
@@ -367,12 +376,12 @@ CurveWarp::set_param(const String & param, const ValueBase &value)
 ValueBase
 CurveWarp::get_param(const String & param)const
 {
-	EXPORT(origin);
-	EXPORT(start_point);
-	EXPORT(end_point);
-	EXPORT(bline);
-	EXPORT(fast);
-	EXPORT(perp_width);
+	EXPORT_VALUE(param_origin);
+	EXPORT_VALUE(param_start_point);
+	EXPORT_VALUE(param_end_point);
+	EXPORT_VALUE(param_bline);
+	EXPORT_VALUE(param_fast);
+	EXPORT_VALUE(param_perp_width);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -387,7 +396,7 @@ CurveWarp::get_param_vocab()const
 
 	ret.push_back(ParamDesc("origin")
 				  .set_local_name(_("Origin"))
-				  .set_description(_("Position of the source line"))
+				  .set_description(_("Position of the destiny Spline line"))
 	);
 	ret.push_back(ParamDesc("perp_width")
 				  .set_local_name(_("Width"))
@@ -425,6 +434,9 @@ CurveWarp::get_color(Context context, const Point &point)const
 bool
 CurveWarp::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Point start_point=param_start_point.get(Point());
+	Point end_point=param_end_point.get(Point());
+
 	SuperCallback stageone(cb,0,9000,10000);
 	SuperCallback stagetwo(cb,9000,10000,10000);
 
@@ -601,11 +613,19 @@ CurveWarp::accelerated_render(Context context,Surface *surface,int quality, cons
 /////
 
 bool
-CurveWarp::accelerated_cairorender(Context context,cairo_surface_t *csurface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+CurveWarp::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc_, ProgressCallback *cb)const
 {
+	Point start_point=param_start_point.get(Point());
+	Point end_point=param_end_point.get(Point());
+
 	SuperCallback stageone(cb,0,9000,10000);
 	SuperCallback stagetwo(cb,9000,10000,10000);
 	
+	RendDesc renddesc(renddesc_);
+	// Untransform the render desc
+	if(!cairo_renddesc_untransform(cr, renddesc))
+		return false;
+	
 	int x,y;
 	
 	const Real pw(renddesc.get_pw()),ph(renddesc.get_ph());
@@ -691,7 +711,7 @@ if (along > max_along) max_along = along
 	
 #define MAXPIX 10000
 	if (src_w > MAXPIX) src_w = MAXPIX;
-		if (src_h > MAXPIX) src_h = MAXPIX;
+	if (src_h > MAXPIX) src_h = MAXPIX;
 	
 	// this is an attempt to remove artifacts around tile edges - the
 	// cubic interpolation uses at most 2 pixels either side of the
@@ -711,25 +731,35 @@ if (along > max_along) max_along = along
 	src_desc.set_br(src_br);
 	src_desc.set_wh(src_w, src_h);
 	
-			// render the context onto a new surface
-//		Surface source;
-//	source.set_wh(src_w,src_h);
 	
-	cairo_surface_t* csource;
-	csource=cairo_surface_create_similar(csurface,CAIRO_CONTENT_COLOR_ALPHA, src_w, src_h);
-	if(!context.accelerated_cairorender(csource,quality,src_desc,&stageone))
+	// New expanded renddesc values
+	const double wpw=src_desc.get_pw();
+	const double wph=src_desc.get_ph();
+	const double wtlx=src_desc.get_tl()[0];
+	const double wtly=src_desc.get_tl()[1];
+
+	// render the context onto a new surface
+	
+	cairo_surface_t* csource, *cresult;
+	csource=cairo_surface_create_similar(cairo_get_target(cr),CAIRO_CONTENT_COLOR_ALPHA, src_w, src_h);
+	cresult=cairo_surface_create_similar(cairo_get_target(cr),CAIRO_CONTENT_COLOR_ALPHA, w, h);
+	cairo_t *subcr=cairo_create(csource);
+	cairo_scale(subcr, 1/wpw, 1/wph);
+	cairo_translate(subcr, -wtlx, -wtly);
+
+	if(!context.accelerated_cairorender(subcr,quality,src_desc,&stageone))
 		return false;
+	// don't needed anymore
+	cairo_destroy(subcr);
+	//access to pixels
 	CairoSurface source(csource);
 	source.map_cairo_image();
 	
-	CairoSurface surface(csurface);
-	surface.map_cairo_image();
+	CairoSurface result(cresult);
+	result.map_cairo_image();
 	float u,v;
 	Point pos, tmp;
-	
-//	surface->set_wh(w,h);
-//	surface.clear();
-	
+		
 	if(quality<=4)				// CUBIC
 		for(y=0,pos[1]=tl[1];y<h;y++,pos[1]+=ph)
 		{
@@ -739,9 +769,9 @@ if (along > max_along) max_along = along
 				u=(tmp[0]-src_tl[0])/src_pw;
 				v=(tmp[1]-src_tl[1])/src_ph;
 				if(u<0 || v<0 || u>=src_w || v>=src_h || isnan(u) || isnan(v))
-					surface[y][x]=CairoColor(context.get_color(tmp)).premult_alpha();
+					result[y][x]=CairoColor(context.get_color(tmp)).premult_alpha();
 				else
-					surface[y][x]=source.cubic_sample_cooked(u,v);
+					result[y][x]=source.cubic_sample_cooked(u,v);
 			}
 			if((y&31)==0 && cb && !stagetwo.amount_complete(y,h)) return false;
 		}
@@ -754,9 +784,9 @@ if (along > max_along) max_along = along
 				u=(tmp[0]-src_tl[0])/src_pw;
 				v=(tmp[1]-src_tl[1])/src_ph;
 				if(u<0 || v<0 || u>=src_w || v>=src_h || isnan(u) || isnan(v))
-					surface[y][x]=CairoColor(context.get_color(tmp)).premult_alpha();
+					result[y][x]=CairoColor(context.get_color(tmp)).premult_alpha();
 				else
-					surface[y][x]=source.linear_sample_cooked(u,v);
+					result[y][x]=source.linear_sample_cooked(u,v);
 			}
 			if((y&31)==0 && cb && !stagetwo.amount_complete(y,h)) return false;
 		}
@@ -769,15 +799,30 @@ if (along > max_along) max_along = along
 				u=(tmp[0]-src_tl[0])/src_pw;
 				v=(tmp[1]-src_tl[1])/src_ph;
 				if(u<0 || v<0 || u>=src_w || v>=src_h || isnan(u) || isnan(v))
-					surface[y][x]=CairoColor(context.get_color(tmp)).premult_alpha();
+					result[y][x]=CairoColor(context.get_color(tmp)).premult_alpha();
 				else
-					surface[y][x]=source[floor_to_int(v)][floor_to_int(u)];
+					result[y][x]=source[floor_to_int(v)][floor_to_int(u)];
 			}
 			if((y&31)==0 && cb && !stagetwo.amount_complete(y,h)) return false;
 		}
 	
-	surface.unmap_cairo_image();
+	result.unmap_cairo_image();
 	source.unmap_cairo_image();
+	
+	cairo_surface_destroy(csource);
+	// Now paint it on the context
+	cairo_save(cr);
+	
+	cairo_translate(cr, tl[0], tl[1]);
+	cairo_scale(cr, pw, ph);
+	cairo_set_source_surface(cr, cresult, 0, 0);
+	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+	cairo_paint(cr);
+	
+	cairo_restore(cr);
+	
+	cairo_surface_destroy(cresult);
+	
 	// Mark our progress as finished
 	if(cb && !cb->amount_complete(10000,10000))
 		return false;
