@@ -62,23 +62,23 @@ SYNFIG_LAYER_SET_CVS_ID(InsideOut,"$Id$");
 /* === M E T H O D S ======================================================= */
 
 InsideOut::InsideOut():
-	origin(0,0)
+	param_origin(ValueBase(Point(0,0)))
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 bool
 InsideOut::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT(origin);
+	IMPORT_VALUE(param_origin);
 	return false;
 }
 
 ValueBase
 InsideOut::get_param(const String & param)const
 {
-	EXPORT(origin);
+	EXPORT_VALUE(param_origin);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -89,6 +89,7 @@ InsideOut::get_param(const String & param)const
 synfig::Layer::Handle
 InsideOut::hit_check(synfig::Context context, const synfig::Point &p)const
 {
+	Point origin=param_origin.get(Point());
 	Point pos(p-origin);
 	Real inv_mag=pos.inv_mag();
 	Point invpos(pos*inv_mag*inv_mag);
@@ -98,12 +99,23 @@ InsideOut::hit_check(synfig::Context context, const synfig::Point &p)const
 Color
 InsideOut::get_color(Context context, const Point &p)const
 {
+	Point origin=param_origin.get(Point());
 	Point pos(p-origin);
 	Real inv_mag=pos.inv_mag();
 	Point invpos(pos*inv_mag*inv_mag);
 	return context.get_color(invpos+origin);
 }
 
+CairoColor
+InsideOut::get_cairocolor(Context context, const Point &p)const
+{
+	Point origin=param_origin.get(Point());
+	Point pos(p-origin);
+	Real inv_mag=pos.inv_mag();
+	Point invpos(pos*inv_mag*inv_mag);
+	return context.get_cairocolor(invpos+origin);
+}
+
 class InsideOut_Trans : public Transform
 {
 	etl::handle<const InsideOut> layer;
@@ -112,21 +124,28 @@ public:
 
 	synfig::Vector perform(const synfig::Vector& x)const
 	{
-		Point pos(x-layer->origin);
+		Point origin=layer->param_origin.get(Point());
+		Point pos(x-origin);
 		Real inv_mag=pos.inv_mag();
 		if(!isnan(inv_mag))
-			return (pos*(inv_mag*inv_mag)+layer->origin);
+			return (pos*(inv_mag*inv_mag)+origin);
 		return x;
 	}
 
 	synfig::Vector unperform(const synfig::Vector& x)const
 	{
-		Point pos(x-layer->origin);
+		Point origin=layer->param_origin.get(Point());
+		Point pos(x-origin);
 		Real inv_mag=pos.inv_mag();
 		if(!isnan(inv_mag))
-			return (pos*(inv_mag*inv_mag)+layer->origin);
+			return (pos*(inv_mag*inv_mag)+origin);
 		return x;
 	}
+
+	synfig::String get_string()const
+	{
+		return "insideout";
+	}
 };
 etl::handle<Transform>
 InsideOut::get_transform()const
