@@ -65,13 +65,13 @@ SYNFIG_LAYER_SET_CVS_ID(Rotate,"$Id$");
 /* === E N T R Y P O I N T ================================================= */
 
 Rotate::Rotate():
-	origin	(0,0),
-	amount	(Angle::deg(0)),
+	param_origin (ValueBase(Vector(0,0))),
+	param_amount (ValueBase(Angle::deg(0))),
 	sin_val	(0),
 	cos_val	(1)
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 Rotate::~Rotate()
@@ -81,16 +81,17 @@ Rotate::~Rotate()
 bool
 Rotate::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT(origin);
+	IMPORT_VALUE(param_origin);
 
-	if(param=="amount" && value.same_type_as(amount))
+	IMPORT_VALUE_PLUS(param_amount,
 	{
-		amount=value.get(amount);
+		Angle amount=value.get(Angle());
 		sin_val=Angle::sin(amount).get();
 		cos_val=Angle::cos(amount).get();
-		set_param_static(param, value.get_static());
+		param_amount.set(amount);
 		return true;
 	}
+	);
 
 	return false;
 }
@@ -98,8 +99,8 @@ Rotate::set_param(const String & param, const ValueBase &value)
 ValueBase
 Rotate::get_param(const String &param)const
 {
-	EXPORT(origin);
-	EXPORT(amount);
+	EXPORT_VALUE(param_origin);
+	EXPORT_VALUE(param_amount);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -134,14 +135,21 @@ public:
 
 	synfig::Vector perform(const synfig::Vector& x)const
 	{
-		Point pos(x-layer->origin);
-		return Point(layer->cos_val*pos[0]-layer->sin_val*pos[1],layer->sin_val*pos[0]+layer->cos_val*pos[1])+layer->origin;
+		Vector origin=layer->param_origin.get(Vector());
+		Point pos(x-origin);
+		return Point(layer->cos_val*pos[0]-layer->sin_val*pos[1],layer->sin_val*pos[0]+layer->cos_val*pos[1])+origin;
 	}
 
 	synfig::Vector unperform(const synfig::Vector& x)const
 	{
-		Point pos(x-layer->origin);
-		return Point(layer->cos_val*pos[0]+layer->sin_val*pos[1],-layer->sin_val*pos[0]+layer->cos_val*pos[1])+layer->origin;
+		Vector origin=layer->param_origin.get(Vector());
+		Point pos(x-origin);
+		return Point(layer->cos_val*pos[0]+layer->sin_val*pos[1],-layer->sin_val*pos[0]+layer->cos_val*pos[1])+origin;
+	}
+
+	synfig::String get_string()const
+	{
+		return "rotate";
 	}
 };
 etl::handle<Transform>
@@ -153,6 +161,7 @@ Rotate::get_transform()const
 synfig::Layer::Handle
 Rotate::hit_check(synfig::Context context, const synfig::Point &p)const
 {
+	Vector origin=param_origin.get(Vector());
 	Point pos(p-origin);
 	Point newpos(cos_val*pos[0]+sin_val*pos[1],-sin_val*pos[0]+cos_val*pos[1]);
 	newpos+=origin;
@@ -162,6 +171,7 @@ Rotate::hit_check(synfig::Context context, const synfig::Point &p)const
 Color
 Rotate::get_color(Context context, const Point &p)const
 {
+	Vector origin=param_origin.get(Vector());
 	Point pos(p-origin);
 	Point newpos(cos_val*pos[0]+sin_val*pos[1],-sin_val*pos[0]+cos_val*pos[1]);
 	newpos+=origin;
@@ -171,6 +181,9 @@ Rotate::get_color(Context context, const Point &p)const
 bool
 Rotate::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Vector origin=param_origin.get(Vector());
+	Angle amount=param_amount.get(Angle());
+	
 	if(amount.dist(Angle::deg(0))==Angle::deg(0))
 		return context.accelerated_render(surface,quality,renddesc,cb);
 	if(amount.dist(Angle::deg(180))==Angle::deg(0))
@@ -337,114 +350,38 @@ Rotate::accelerated_render(Context context,Surface *surface,int quality, const R
 	return true;
 }
 
-////////
+///////////
+
 bool
-Rotate::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Rotate::accelerated_cairorender(Context context, cairo_t *cr,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Vector origin=param_origin.get(Vector());
+	Angle amount=param_amount.get(Angle());
 
-	const Point	otl(renddesc.get_tl());
-	const Point obr(renddesc.get_br());
-	const int	w(renddesc.get_w());
-	const int	h(renddesc.get_h());
-	
-	// Width and Height of a pixel
-	const Real pw = (obr[0] - otl[0]) / w;
-	const Real ph = (obr[1] - otl[1]) / h;
-	
-	// These are the scale values
-	const double sx(1/pw);
-	const double sy(1/ph);
-
-//	if(amount.dist(Angle::deg(0))==Angle::deg(0))
-//		return context.accelerated_cairorender(surface,quality,renddesc,cb);
-//	if(amount.dist(Angle::deg(180))==Angle::deg(0))
-//	{
-//		RendDesc desc(renddesc);
-//		desc.clear_flags();
-//		Point tmp;
-//		tmp=renddesc.get_tl()-origin;
-//		desc.set_tl(Point(-tmp[0],-tmp[1])+origin);
-//		tmp=renddesc.get_br()-origin;
-//		desc.set_br(Point(-tmp[0],-tmp[1])+origin);
-//		return context.accelerated_cairorender(surface,quality,desc,cb);
-//	}
-	Point tl(renddesc.get_tl()-origin);
-	Point br(renddesc.get_br()-origin);
-	
-	{
-		Point rot_tl(cos_val*tl[0]+sin_val*tl[1],-sin_val*tl[0]+cos_val*tl[1]);
-		Point rot_br(cos_val*br[0]+sin_val*br[1],-sin_val*br[0]+cos_val*br[1]);
-		Point rot_tr(cos_val*br[0]+sin_val*tl[1],-sin_val*br[0]+cos_val*tl[1]);
-		Point rot_bl(cos_val*tl[0]+sin_val*br[1],-sin_val*tl[0]+cos_val*br[1]);
-		rot_tl+=origin;
-		rot_br+=origin;
-		rot_tr+=origin;
-		rot_bl+=origin;
-		
-		Point min_point(min(min(min(rot_tl[0],rot_br[0]),rot_tr[0]),rot_bl[0]),min(min(min(rot_tl[1],rot_br[1]),rot_tr[1]),rot_bl[1]));
-		Point max_point(max(max(max(rot_tl[0],rot_br[0]),rot_tr[0]),rot_bl[0]),max(max(max(rot_tl[1],rot_br[1]),rot_tr[1]),rot_bl[1]));
-		
-		if(tl[0]>br[0])
-		{
-			tl[0]=max_point[0];
-			br[0]=min_point[0];
-		}
-		else
-		{
-			br[0]=max_point[0];
-			tl[0]=min_point[0];
-		}
-		if(tl[1]>br[1])
-		{
-			tl[1]=max_point[1];
-			br[1]=min_point[1];
-		}
-		else
-		{
-			br[1]=max_point[1];
-			tl[1]=min_point[1];
-		}
-	}
-	Real pww=(renddesc.get_w())/(renddesc.get_br()[0]-renddesc.get_tl()[0]);
-	Real phh=(renddesc.get_h())/(renddesc.get_br()[1]-renddesc.get_tl()[1]);
-	// we're going to round the canvas size to an integer number of pixels, so round the
-	// tl-br rectangle accordingly - otherwise we see the jittering described in bug 2152666
-	br[0] -= (pww*(br[0]-tl[0]) - round_to_int(pww*(br[0]-tl[0]))) / pww;
-	br[1] -= (phh*(br[1]-tl[1]) - round_to_int(phh*(br[1]-tl[1]))) / phh;
-		
-	RendDesc desc(renddesc);
-	desc.clear_flags();
-	desc.set_tl(tl);
-	desc.set_br(br);
-	desc.set_wh(round_to_int(pww*(br[0]-tl[0])),round_to_int(phh*(br[1]-tl[1])));
-	
-	cairo_surface_t* source=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, desc.get_w(), desc.get_h());
-
-	if(!context.accelerated_cairorender(source,quality,desc,cb))
-		return false;
-	
-	cairo_t *cr=cairo_create(surface);
-	cairo_save(cr);
+	const double rtx(origin[0]);
+	const double rty(origin[1]);
 
 	float angle=Angle::rad(amount).get();
-	cairo_translate(cr, (origin[0]-otl[0])*sx, (origin[1]-otl[1])*sy);
-	cairo_rotate(cr, -1.0*angle);
-	cairo_translate(cr, -(origin[0]-otl[0])*sx, -(origin[1]-otl[1])*sy);
-	cairo_set_source_surface(cr, source, (tl[0]-otl[0])*sx, (tl[1]-otl[1])*sy);
+	
+	cairo_save(cr);
+	cairo_translate(cr, rtx, rty);
+	cairo_rotate(cr, angle);
+	cairo_translate(cr, -rtx, -rty);
 
+	// is this really useful?
 	if(quality>8) cairo_pattern_set_filter(cairo_get_source(cr), CAIRO_FILTER_FAST);
 	else if(quality>=4) cairo_pattern_set_filter(cairo_get_source(cr), CAIRO_FILTER_GOOD);
 	else cairo_pattern_set_filter(cairo_get_source(cr), CAIRO_FILTER_BEST);
-
-	cairo_set_operator(cr, CAIRO_OPERATOR_OVER); // again this has to be the real operator
-	cairo_paint(cr);
+	
+	if(!context.accelerated_cairorender(cr,quality,renddesc,cb))
+	{
+		cairo_restore(cr);
+		return false;
+	}
 	cairo_restore(cr);
-	cairo_surface_destroy(source);
-	cairo_destroy(cr);
-	return true;
-
+	return true;	
 }
-///////////
+
 Rect
 Rotate::get_full_bounding_rect(Context context)const
 {
