@@ -42,6 +42,7 @@
 #include <synfig/value.h>
 #include <synfig/valuenode.h>
 #include <synfig/segment.h>
+#include <synfig/cairo_renddesc.h>
 
 #include <cstring>
 #include <ETL/pen>
@@ -79,28 +80,48 @@ inline void clamp(synfig::Vector &v)
 
 Layer_Shade::Layer_Shade():
 	Layer_Composite	(0.75,Color::BLEND_BEHIND),
-	size(0.1,0.1),
-	type(Blur::FASTGAUSSIAN),
-	color(Color::black()),
-	origin(0.2,-0.2),
-	invert(false)
+	param_size(ValueBase(Vector(0.1,0.1))),
+	param_type(ValueBase(int(Blur::FASTGAUSSIAN))),
+	param_color(ValueBase(Color::black())),
+	param_origin(ValueBase(Vector(0.2,-0.2))),
+	param_invert(ValueBase(false))
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 bool
 Layer_Shade::set_param(const String &param, const ValueBase &value)
 {
-	IMPORT_PLUS(size,clamp(size));
-	IMPORT(type);
-	IMPORT_PLUS(color, { if (color.get_a() == 0) { if (converted_blend_) {
+	IMPORT_VALUE_PLUS(param_size,
+		{
+			synfig::Vector size=param_size.get(synfig::Vector());
+			clamp(size);
+			param_size.set(size);
+		}
+		);
+	IMPORT_VALUE(param_type);
+	IMPORT_VALUE_PLUS(param_color,
+		{
+			synfig::Color color=param_color.get(synfig::Color());
+			if (color.get_a() == 0)
+			{
+				if (converted_blend_)
+				{
 					set_blend_method(Color::BLEND_ALPHA_OVER);
-					color.set_a(1); } else transparent_color_ = true; } });
-	IMPORT(origin);
-	IMPORT(invert);
+					color.set_a(1);
+					param_color.set(color);
+				}
+				else
+					transparent_color_ = true;
+			}
+		}
+		);
+	IMPORT_VALUE(param_origin);
+	IMPORT_VALUE(param_invert);
 
-	IMPORT_AS(origin,"offset");
+	if(param=="offset")
+		return set_param("origin", value);
 
 	return Layer_Composite::set_param(param,value);
 }
@@ -108,11 +129,11 @@ Layer_Shade::set_param(const String &param, const ValueBase &value)
 ValueBase
 Layer_Shade::get_param(const String &param)const
 {
-	EXPORT(size);
-	EXPORT(type);
-	EXPORT(color);
-	EXPORT(origin);
-	EXPORT(invert);
+	EXPORT_VALUE(param_size);
+	EXPORT_VALUE(param_type);
+	EXPORT_VALUE(param_color);
+	EXPORT_VALUE(param_origin);
+	EXPORT_VALUE(param_invert);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -123,6 +144,12 @@ Layer_Shade::get_param(const String &param)const
 Color
 Layer_Shade::get_color(Context context, const Point &pos)const
 {
+	synfig::Vector size=param_size.get(synfig::Vector());
+	int type=param_type.get(int());
+	synfig::Color color=param_color.get(synfig::Color());
+	synfig::Vector origin=param_origin.get(synfig::Vector());
+	bool invert=param_invert.get(bool());
+	
 	Point blurpos = Blur(size,type)(pos);
 
 	if(get_amount()==0.0)
@@ -141,6 +168,12 @@ Layer_Shade::get_color(Context context, const Point &pos)const
 bool
 Layer_Shade::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	synfig::Vector size=param_size.get(synfig::Vector());
+	int type=param_type.get(int());
+	synfig::Color color=param_color.get(synfig::Color());
+	synfig::Vector origin=param_origin.get(synfig::Vector());
+	bool invert=param_invert.get(bool());
+
 	int x,y;
 
 	const int	w = renddesc.get_w(),
@@ -372,20 +405,51 @@ Layer_Shade::accelerated_render(Context context,Surface *surface,int quality, co
 
 ///
 bool
-Layer_Shade::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Layer_Shade::accelerated_cairorender(Context context,cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
-	int x,y;
+	synfig::Vector size=param_size.get(synfig::Vector());
+	int type=param_type.get(int());
+	synfig::Color color=param_color.get(synfig::Color());
+	synfig::Vector origin=param_origin.get(synfig::Vector());
+	bool invert=param_invert.get(bool());
 
-	const int	w = renddesc.get_w(),
-	h = renddesc.get_h();
-	const Real	pw = renddesc.get_pw(),
-	ph = renddesc.get_ph();
+	int x,y;
+	SuperCallback stageone(cb,0,5000,10000);
+	SuperCallback stagetwo(cb,5000,10000,10000);
 	
 	RendDesc	workdesc(renddesc);
 	cairo_surface_t		*worksurface;
 	etl::surface<float> blurred;
 	
-	//expand the working surface to accommodate the blur	
+	// Untransform the render desc
+	if(!cairo_renddesc_untransform(cr, workdesc))
+		return false;
+	
+	const int w=workdesc.get_w(), h=workdesc.get_h();
+	const double pw=(workdesc.get_br()[0]-workdesc.get_tl()[0])/w;
+	const double ph=(workdesc.get_br()[1]-workdesc.get_tl()[1])/h;
+
+	//callbacks depend on how long the blur takes
+	if(size[0] || size[1])
+	{
+		if(type == Blur::DISC)
+		{
+			stageone = SuperCallback(cb,0,5000,10000);
+			stagetwo = SuperCallback(cb,5000,10000,10000);
+		}
+		else
+		{
+			stageone = SuperCallback(cb,0,9000,10000);
+			stagetwo = SuperCallback(cb,9000,10000,10000);
+		}
+	}
+	else
+	{
+		stageone = SuperCallback(cb,0,9999,10000);
+		stagetwo = SuperCallback(cb,9999,10000,10000);
+	}
+
+	//expand the working surface to accommodate the blur
 	//the expanded size = 1/2 the size in each direction rounded up
 	int	halfsizex = (int) (abs(size[0]*.5/pw) + 3),
 	halfsizey = (int) (abs(size[1]*.5/ph) + 3);
@@ -460,44 +524,33 @@ Layer_Shade::accelerated_cairorender(Context context,cairo_surface_t *surface,in
 			break;
 		}
 	}
-	SuperCallback stageone(cb,0,5000,10000);
-	SuperCallback stagetwo(cb,5000,10000,10000);
-	
-	//callbacks depend on how long the blur takes
-	if(size[0] || size[1])
-	{
-		if(type == Blur::DISC)
-		{
-			stageone = SuperCallback(cb,0,5000,10000);
-			stagetwo = SuperCallback(cb,5000,10000,10000);
-		}
-		else
-		{
-			stageone = SuperCallback(cb,0,9000,10000);
-			stagetwo = SuperCallback(cb,9000,10000,10000);
-		}
-	}
-	else
-	{
-		stageone = SuperCallback(cb,0,9999,10000);
-		stagetwo = SuperCallback(cb,9999,10000,10000);
-	}
 	
+	// New expanded workdesc values
+	const int ww=workdesc.get_w();
+	const int wh=workdesc.get_h();
+	const double wtlx=workdesc.get_tl()[0];
+	const double wtly=workdesc.get_tl()[1];
 	
 	// setup the worksurface
-	worksurface=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, workdesc.get_w(), workdesc.get_h());
+	worksurface=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, ww, wh);
+	cairo_t* subcr=cairo_create(worksurface);
+	cairo_scale(subcr, 1/pw, 1/ph);
+	cairo_translate(subcr, -wtlx, -wtly);
 
 	//render the background onto the expanded surface
-	if(!context.accelerated_cairorender(worksurface,quality,workdesc,&stageone))
+	if(!context.accelerated_cairorender(subcr,quality,workdesc,&stageone))
 		return false;
 	// copy the background on the target surface if applies
 	if(!is_solid_color())
 	{
-		cairo_t *cr=cairo_create(surface);
-		cairo_set_source_surface(cr, worksurface, -halfsizex+(origin_u<0?origin_u:0), -halfsizey+(origin_v<0?origin_v:0));
+		//cairo_translate(subcr, origin[0], origin[1]);
+		cairo_save(cr);
+		cairo_translate(cr, wtlx, wtly);
+		cairo_scale(cr, pw, ph);
+		cairo_set_source_surface(cr, worksurface, 0, 0);
 		cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
 		cairo_paint(cr);
-		cairo_destroy(cr);
+		cairo_restore(cr);
 	}
 	
 	// Extract the CairoSurface from the cairo_surface_t
@@ -532,18 +585,21 @@ Layer_Shade::accelerated_cairorender(Context context,cairo_surface_t *surface,in
 		}
 	
 	cairoworksurface.unmap_cairo_image();
-
-	// Now lets blend the result in the output surface
-	cairo_t *cr=cairo_create(surface);
-	cairo_set_source_surface(cr, worksurface, -halfsizex+(origin_u<0?origin_u:0)-origin_u, -halfsizey+(origin_v<0?origin_v:0)-origin_v);
-	cairo_paint_with_alpha_operator(cr, 1.0, get_blend_method()); // TODO: add cairo_paint_opertor function when alpha=1.0 (it is quicker)
-	cairo_destroy(cr);
 	
+	// Now lets blend the result in the output surface
+	cairo_save(cr);
+	cairo_translate(cr, origin[0], origin[1]);
+	cairo_translate(cr, wtlx, wtly);
+	cairo_scale(cr, pw, ph);
+	cairo_set_source_surface(cr, worksurface, 0, 0);
+	cairo_paint_with_alpha_operator(cr, 1.0, get_blend_method());
+	// TODO: add cairo_paint_opertor function when alpha=1.0 (it is quicker)
+	cairo_restore(cr);
 	cairo_surface_destroy(worksurface);
-		
+	
 #undef GAUSSIAN_ADJUSTMENT
 #undef SCALE_FACTOR
-
+	
 	if(cb && !cb->amount_complete(10000,10000))
 	{
 		//if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Renderer Failure",__LINE__));
@@ -593,6 +649,10 @@ Layer_Shade::get_param_vocab(void)const
 Rect
 Layer_Shade::get_full_bounding_rect(Context context)const
 {
+	synfig::Vector size=param_size.get(synfig::Vector());
+	synfig::Vector origin=param_origin.get(synfig::Vector());
+	bool invert=param_invert.get(bool());
+
 	if(is_disabled())
 		return context.get_full_bounding_rect();
 
