@@ -42,11 +42,16 @@ class Layer_Shade : public synfig::Layer_Composite
 {
 	SYNFIG_LAYER_MODULE_EXT
 private:
-	synfig::Vector 	size;
-	int				type;
-	synfig::Color	color;
-	synfig::Vector	origin;
-	bool invert;
+	//!Parameter: (synfig::Vector)
+	ValueBase param_size;
+	//!Parameter: (int)
+	ValueBase param_type;
+	//!Parameter: (synfig::Color)
+	ValueBase param_color;
+	//!Parameter: (synfig::Vector)
+	ValueBase param_origin;
+	//!Parameter: (bool)
+	ValueBase param_invert;
 
 public:
 	Layer_Shade();
@@ -58,7 +63,7 @@ public:
 	virtual Color get_color(Context context, const Point &pos)const;
 
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
-	virtual bool accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context,cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 	virtual Rect get_full_bounding_rect(Context context)const;
 	virtual Vocab get_param_vocab()const;
 	virtual bool reads_context()const { return true; }
