@@ -66,19 +66,19 @@ SYNFIG_LAYER_SET_CVS_ID(Layer_Stretch,"$Id$");
 /* === E N T R Y P O I N T ================================================= */
 
 Layer_Stretch::Layer_Stretch():
-	amount(1,1),
-	center(0,0)
+	param_amount(ValueBase(Point(1,1))),
+	param_center(ValueBase(Point(0,0)))
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 
 bool
 Layer_Stretch::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT(amount);
-	IMPORT(center);
+	IMPORT_VALUE(param_amount);
+	IMPORT_VALUE(param_center);
 
 	return false;
 }
@@ -86,8 +86,8 @@ Layer_Stretch::set_param(const String & param, const ValueBase &value)
 ValueBase
 Layer_Stretch::get_param(const String &param)const
 {
-	EXPORT(amount);
-	EXPORT(center);
+	EXPORT_VALUE(param_amount);
+	EXPORT_VALUE(param_center);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -117,6 +117,9 @@ Layer_Stretch::get_param_vocab()const
 synfig::Layer::Handle
 Layer_Stretch::hit_check(synfig::Context context, const synfig::Point &pos)const
 {
+	Vector amount=param_amount.get(Vector());
+	Point center=param_center.get(Point());
+	
 	Point npos(pos);
 	npos[0]=(npos[0]-center[0])/amount[0]+center[0];
 	npos[1]=(npos[1]-center[1])/amount[1]+center[1];
@@ -126,6 +129,9 @@ Layer_Stretch::hit_check(synfig::Context context, const synfig::Point &pos)const
 Color
 Layer_Stretch::get_color(Context context, const Point &pos)const
 {
+	Vector amount=param_amount.get(Vector());
+	Point center=param_center.get(Point());
+
 	Point npos(pos);
 	npos[0]=(npos[0]-center[0])/amount[0]+center[0];
 	npos[1]=(npos[1]-center[1])/amount[1]+center[1];
@@ -140,14 +146,25 @@ public:
 
 	synfig::Vector perform(const synfig::Vector& x)const
 	{
-		return Vector((x[0]-layer->center[0])*layer->amount[0]+layer->center[0],
-					  (x[1]-layer->center[1])*layer->amount[1]+layer->center[1]);
+		Vector amount=layer->param_amount.get(Vector());
+		Point center=layer->param_center.get(Point());
+
+		return Vector((x[0]-center[0])*amount[0]+center[0],
+					  (x[1]-center[1])*amount[1]+center[1]);
 	}
 
 	synfig::Vector unperform(const synfig::Vector& x)const
 	{
-		return Vector((x[0]-layer->center[0])/layer->amount[0]+layer->center[0],
-					  (x[1]-layer->center[1])/layer->amount[1]+layer->center[1]);
+		Vector amount=layer->param_amount.get(Vector());
+		Point center=layer->param_center.get(Point());
+
+		return Vector((x[0]-center[0])/amount[0]+center[0],
+					  (x[1]-center[1])/amount[1]+center[1]);
+	}
+
+	synfig::String get_string()const
+	{
+		return "stretch";
 	}
 };
 etl::handle<Transform>
@@ -159,6 +176,9 @@ Layer_Stretch::get_transform()const
 bool
 Layer_Stretch::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Vector amount=param_amount.get(Vector());
+	Point center=param_center.get(Point());
+
 	if (amount[0] == 0 || amount[1] == 0)
 	{
 		surface->set_wh(renddesc.get_w(), renddesc.get_h());
@@ -182,40 +202,44 @@ Layer_Stretch::accelerated_render(Context context,Surface *surface,int quality,
 	return context.accelerated_render(surface,quality,desc,cb);
 }
 
-//////
+
+
 bool
-Layer_Stretch::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Layer_Stretch::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Vector amount=param_amount.get(Vector());
+	Point center=param_center.get(Point());
+
 	if (amount[0] == 0 || amount[1] == 0)
 	{
-		cairo_t* cr=cairo_create(surface);
 		cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
 		cairo_fill(cr);
 		return true;
 	}
+	const double stx(center[0]);
+	const double sty(center[1]);
 	
-	RendDesc desc(renddesc);
-	desc.clear_flags();
-    // Adjust the top_left and bottom_right points
-	// for our zoom amount
-	Point npos;
-	npos[0]=(desc.get_tl()[0]-center[0])/amount[0]+center[0];
-	npos[1]=(desc.get_tl()[1]-center[1])/amount[1]+center[1];
-	desc.set_tl(npos);
-	npos[0]=(desc.get_br()[0]-center[0])/amount[0]+center[0];
-	npos[1]=(desc.get_br()[1]-center[1])/amount[1]+center[1];
-	desc.set_br(npos);
-	
-	// Render the scene
-	return context.accelerated_cairorender(surface,quality,desc,cb);
-}
+	cairo_save(cr);
+	cairo_translate(cr, stx, sty);
+	cairo_scale(cr, amount[0], amount[1]);
+	cairo_translate(cr, -stx, -sty);
 
-/////
+	if(!context.accelerated_cairorender(cr,quality,renddesc,cb))
+	{
+		cairo_restore(cr);
+		return false;
+	}
+	cairo_restore(cr);
+	return true;
+}
 
 
 Rect
 Layer_Stretch::get_full_bounding_rect(Context context)const
 {
+	Vector amount=param_amount.get(Vector());
+	Point center=param_center.get(Point());
+
 	Rect rect(context.get_full_bounding_rect());
 	Point min(rect.get_min()), max(rect.get_max());
 
