@@ -47,9 +47,10 @@ class Layer_Stretch : public Layer
 	friend class ::Stretch_Trans;
 
 private:
-
-	Vector amount;
-	Point center;
+	//!Parameter (Vector)
+	ValueBase param_amount;
+	//!Parameter (Point)
+	ValueBase param_center;
 
 public:
 
@@ -62,7 +63,7 @@ public:
 	virtual Color get_color(Context context, const Point &pos)const;
 
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
-	virtual bool accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context, cairo_t *cr,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 	synfig::Layer::Handle hit_check(synfig::Context context, const synfig::Point &point)const;
 
 	virtual Vocab get_param_vocab()const;
