@@ -68,7 +68,9 @@ SYNFIG_LAYER_SET_CVS_ID(Layer_Stroboscope,"$Id$");
 
 Layer_Stroboscope::Layer_Stroboscope()
 {
-	frequency=2.0;
+	param_frequency=ValueBase(float(2.0));
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 Layer_Stroboscope::~Layer_Stroboscope()
@@ -78,7 +80,7 @@ Layer_Stroboscope::~Layer_Stroboscope()
 bool
 Layer_Stroboscope::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT(frequency);
+	IMPORT_VALUE(param_frequency);
 
 	return Layer::set_param(param,value);
 }
@@ -86,7 +88,7 @@ Layer_Stroboscope::set_param(const String & param, const ValueBase &value)
 ValueBase
 Layer_Stroboscope::get_param(const String & param)const
 {
-	EXPORT(frequency);
+	EXPORT_VALUE(param_frequency);
 	EXPORT_NAME();
 	EXPORT_VERSION();
 
@@ -107,8 +109,10 @@ Layer_Stroboscope::get_param_vocab()const
 }
 
 void
-Layer_Stroboscope::set_time(Context context, Time t)const
+Layer_Stroboscope::set_time(IndependentContext context, Time t)const
 {
+	float frequency=param_frequency.get(float());
+	
 	Time ret_time=Time::begin();
 	if(frequency > 0.0)
 		ret_time = Time(1.0)/frequency*floor(t*frequency);
@@ -128,8 +132,9 @@ Layer_Stroboscope::accelerated_render(Context context,Surface *surface,int quali
 	return context.accelerated_render(surface,quality,renddesc,cb);
 }
 
+
 bool
-Layer_Stroboscope::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Layer_Stroboscope::accelerated_cairorender(Context context,cairo_t *cr,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
-	return context.accelerated_cairorender(surface,quality,renddesc,cb);
+	return context.accelerated_cairorender(cr,quality,renddesc,cb);
 }
