@@ -41,6 +41,7 @@
 #include <synfig/surface.h>
 #include <synfig/value.h>
 #include <synfig/valuenode.h>
+#include <synfig/cairo_renddesc.h>
 
 #include <synfig/target.h>
 #include <synfig/render.h>
@@ -62,29 +63,40 @@ SYNFIG_LAYER_SET_CVS_ID(SuperSample,"$Id$");
 
 /* === M E T H O D S ======================================================= */
 
-SuperSample::SuperSample():width(2),height(2)
+SuperSample::SuperSample():
+param_width(ValueBase(int(2))),
+param_height(ValueBase(int(2)))
 {
-	scanline=false;
-	alpha_aware=true;
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	param_scanline=ValueBase(false);
+	param_alpha_aware=ValueBase(true);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
+
 }
 
 bool
 SuperSample::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT_PLUS(width,
-		if(value.get(int()) < 1) width = 1;
-		else width=value.get(int());
-		return true;
-		)
-	IMPORT_PLUS(height,
-		if(value.get(int()) < 1) height = 1;
-		else height=value.get(int());
-		return true;
-		)
-	IMPORT(scanline);
-	IMPORT(alpha_aware);
+	IMPORT_VALUE_PLUS(param_width,
+		{
+			int width=param_width.get(int());
+			if(value.get(int()) < 1) width = 1;
+			else width=value.get(int());
+			param_width.set(width);
+			return true;
+		}
+		);
+	IMPORT_VALUE_PLUS(param_height,
+		{
+			int height=param_height.get(int());
+			if(value.get(int()) < 1) height = 1;
+			else height=value.get(int());
+			param_height.set(height);
+			return true;
+		}
+		);
+	IMPORT_VALUE(param_scanline);
+	IMPORT_VALUE(param_alpha_aware);
 
 	return false;
 }
@@ -92,10 +104,10 @@ SuperSample::set_param(const String & param, const ValueBase &value)
 ValueBase
 SuperSample::get_param(const String& param)const
 {
-	EXPORT(width);
-	EXPORT(height);
-    EXPORT(scanline);
-    EXPORT(alpha_aware);
+	EXPORT_VALUE(param_width);
+	EXPORT_VALUE(param_height);
+    EXPORT_VALUE(param_scanline);
+    EXPORT_VALUE(param_alpha_aware);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -106,6 +118,11 @@ SuperSample::get_param(const String& param)const
 bool
 SuperSample::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	int width=param_width.get(int());
+	int height=param_height.get(int());
+	bool scanline=param_scanline.get(bool());
+	bool alpha_aware=param_alpha_aware.get(bool());
+	
 	// don't bother supersampling if our quality is too low.
 	if(quality>=10)
 		return context.accelerated_render(surface,quality,renddesc,cb);
@@ -131,7 +148,7 @@ SuperSample::accelerated_render(Context context,Surface *surface,int quality, co
 		}
 		target->set_rend_desc(&desc);
 
-		if(!render(context-1,target,desc,&subcb))
+		if(!render(context.get_previous(),target,desc,&subcb))
 		{
 			if(cb)cb->error(strprintf(__FILE__"%d: Scanline Renderer Failure",__LINE__));
 			return false;
@@ -216,26 +233,52 @@ SuperSample::accelerated_render(Context context,Surface *surface,int quality, co
 	return true;
 }
 
+
 ////
 bool
-SuperSample::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+SuperSample::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	int width=param_width.get(int());
+	int height=param_height.get(int());
+
 	// don't bother supersampling if our quality is too low.
 	if(quality>=10 || (width==1 && height==1))
-		return context.accelerated_cairorender(surface,quality,renddesc,cb);
+		return context.accelerated_cairorender(cr,quality,renddesc,cb);
 	
 	RendDesc desc(renddesc);
-		
+	// Untransform the render desc
+	if(!cairo_renddesc_untransform(cr, desc))
+		return false;
+
+	//grab values before expand
+	const double pw=desc.get_pw();
+	const double ph=desc.get_ph();
+	const double tlx=desc.get_tl()[0];
+	const double tly=desc.get_tl()[1];
+	
+	// Expand the renddesc
 	desc.clear_flags();
 	desc.set_wh(desc.get_w()*width,desc.get_h()*height);
-	cairo_surface_t* tempsurface=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, renddesc.get_w()*width, renddesc.get_h()*height);
-	// Render the scene
-	if(!context.accelerated_cairorender(tempsurface,quality,desc,cb))
-		{
-			if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Renderer Failure",__LINE__));
-			return false;
-		}
 	
+	// New expanded desc values
+	const int ww=desc.get_w();
+	const int wh=desc.get_h();
+	const double wtlx=desc.get_tl()[0];
+	const double wtly=desc.get_tl()[1];
+	const double wpw=desc.get_pw();
+	const double wph=desc.get_ph();
+	
+	cairo_surface_t* tempsurface=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, ww, wh);
+	cairo_t* tempcr=cairo_create(tempsurface);
+	cairo_scale(tempcr, 1/wpw, 1/wph);
+	cairo_translate(tempcr, -wtlx, -wtly);
+	// Render the scene
+	if(!context.accelerated_cairorender(tempcr,quality,desc,cb))
+	{
+		if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Renderer Failure",__LINE__));
+		return false;
+	}
+	cairo_destroy(tempcr);
 	// Calculate the scales values
 	float scalex=1.0/width;
 	float scaley=1.0/height;
@@ -264,18 +307,22 @@ SuperSample::accelerated_cairorender(Context context,cairo_surface_t *surface,in
 			anti=CAIRO_ANTIALIAS_FAST;
 			break;
 	}
-	cairo_t* cr=cairo_create(surface);
+	cairo_save(cr);
+
+	cairo_translate(cr, tlx, tly);
+	cairo_scale(cr, pw, ph);
+
 	cairo_scale(cr, scalex, scaley);
 	cairo_set_source_surface(cr, tempsurface, 0,0);
-	cairo_pattern_set_filter(cairo_get_source(cr), filter);
-	
 	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+	cairo_pattern_set_filter(cairo_get_source(cr), filter);
 	cairo_set_antialias(cr, anti);
 	cairo_paint(cr);
 	
-	cairo_destroy(cr);
+	cairo_restore(cr);
+	
 	cairo_surface_destroy(tempsurface);
-		
+	
 	return true;
 }
 
