@@ -47,8 +47,10 @@ class SuperSample : public synfig::Layer
 	SYNFIG_LAYER_MODULE_EXT
 
 private:
-	int width, height;
-	bool scanline,alpha_aware;
+	//!Parameter (int)
+	ValueBase param_width, param_height;
+	//!Parameter (bool)
+	ValueBase param_scanline, param_alpha_aware;
 public:
 	SuperSample();
 
@@ -57,7 +59,7 @@ public:
 	virtual ValueBase get_param(const String & param)const;
 
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
-	virtual bool accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context,cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 	virtual synfig::Rect get_bounding_rect(Context context)const;
 
 	virtual Vocab get_param_vocab()const;
