@@ -48,15 +48,20 @@ class Layer_TimeLoop : public Layer
 	SYNFIG_LAYER_MODULE_EXT
 
 private:
-	Time	link_time;
-	Time	local_time;
-	Time	duration;
+	//! Parameter: (Time)
+	ValueBase param_link_time;
+	//! Parameter: (Time)
+	ValueBase param_local_time;
+	//! Parameter: (Time)
+	ValueBase param_duration;
+	//! Parameter: (bool)
+	ValueBase param_only_for_positive_duration;
+	//! Parameter: (bool)
+	ValueBase param_symmetrical; // the 0.1 version of this layer behaved differently before 'start_time' was reached
 
 	Time	start_time;
 	Time	end_time;
 	bool	old_version;
-	bool	only_for_positive_duration;
-	bool	symmetrical; // the 0.1 version of this layer behaved differently before 'start_time' was reached
 
 protected:
 	Layer_TimeLoop();
@@ -73,9 +78,9 @@ public:
 	virtual void reset_version();
 	virtual Color get_color(Context context, const Point &pos)const;
 
-	virtual void set_time(Context context, Time time)const;
+	virtual void set_time(IndependentContext context, Time time)const;
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
-	virtual bool accelerated_cairorender(Context context, cairo_surface_t *surface, int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 };
 
 /* === E N D =============================================================== */
