@@ -43,6 +43,7 @@
 #include <synfig/value.h>
 #include <synfig/valuenode.h>
 #include <synfig/transform.h>
+#include <synfig/cairo_renddesc.h>
 #include <ETL/misc>
 
 #endif
@@ -65,18 +66,18 @@ SYNFIG_LAYER_SET_CVS_ID(Warp,"$Id$");
 /* === E N T R Y P O I N T ================================================= */
 
 Warp::Warp():
-	src_tl	(-2,2),
-	src_br	(2,-2),
-	dest_tl	(-1.8,2.1),
-	dest_tr	(1.8,2.1),
-	dest_bl	(-2.2,-2),
-	dest_br	(2.2,-2),
-	clip	(true)
+	param_src_tl  (ValueBase(Point(-2,2))),
+	param_src_br  (ValueBase(Point(2,-2))),
+	param_dest_tl (ValueBase(Point(-1.8,2.1))),
+	param_dest_tr (ValueBase(Point(1.8,2.1))),
+	param_dest_bl (ValueBase(Point(-2.2,-2))),
+	param_dest_br (ValueBase(Point(2.2,-2))),
+	param_clip	  (ValueBase(true))
 {
+	param_horizon=ValueBase(Real(4));
 	sync();
-	horizon=4;
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 Warp::~Warp()
@@ -208,7 +209,12 @@ Warp::sync()
 	matrix[0][2]=matrix[0][0]*dest_tl[0] + matrix[0][1]*dest_tl[1];
 	matrix[1][2]=matrix[1][0]*dest_tl[0] + matrix[1][1]*dest_tl[1];
 */
-
+	Point src_tl=param_src_tl.get(Point());
+	Point src_br=param_src_br.get(Point());
+	Point dest_tl=param_dest_tl.get(Point());
+	Point dest_tr=param_dest_tr.get(Point());
+	Point dest_bl=param_dest_bl.get(Point());
+	Point dest_br=param_dest_br.get(Point());
 #define matrix tmp
 	Real tmp[3][3];
 
@@ -335,14 +341,14 @@ Warp::sync()
 bool
 Warp::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT_PLUS(src_tl,sync());
-	IMPORT_PLUS(src_br,sync());
-	IMPORT_PLUS(dest_tl,sync());
-	IMPORT_PLUS(dest_tr,sync());
-	IMPORT_PLUS(dest_bl,sync());
-	IMPORT_PLUS(dest_br,sync());
-	IMPORT(clip);
-	IMPORT(horizon);
+	IMPORT_VALUE_PLUS(param_src_tl,sync());
+	IMPORT_VALUE_PLUS(param_src_br,sync());
+	IMPORT_VALUE_PLUS(param_dest_tl,sync());
+	IMPORT_VALUE_PLUS(param_dest_tr,sync());
+	IMPORT_VALUE_PLUS(param_dest_bl,sync());
+	IMPORT_VALUE_PLUS(param_dest_br,sync());
+	IMPORT_VALUE(param_clip);
+	IMPORT_VALUE(param_horizon);
 
 	return false;
 }
@@ -350,14 +356,14 @@ Warp::set_param(const String & param, const ValueBase &value)
 ValueBase
 Warp::get_param(const String &param)const
 {
-	EXPORT(src_tl);
-	EXPORT(src_br);
-	EXPORT(dest_tl);
-	EXPORT(dest_tr);
-	EXPORT(dest_bl);
-	EXPORT(dest_br);
-	EXPORT(clip);
-	EXPORT(horizon);
+	EXPORT_VALUE(param_src_tl);
+	EXPORT_VALUE(param_src_br);
+	EXPORT_VALUE(param_dest_tl);
+	EXPORT_VALUE(param_dest_tr);
+	EXPORT_VALUE(param_dest_bl);
+	EXPORT_VALUE(param_dest_br);
+	EXPORT_VALUE(param_clip);
+	EXPORT_VALUE(param_horizon);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -438,6 +444,11 @@ public:
 		//Point pos(x-layer->origin);
 		//return Point(layer->cos_val*pos[0]+layer->sin_val*pos[1],-layer->sin_val*pos[0]+layer->cos_val*pos[1])+layer->origin;
 	}
+
+	synfig::String get_string()const
+	{
+		return "warp";
+	}
 };
 etl::handle<Transform>
 Warp::get_transform()const
@@ -448,6 +459,10 @@ Warp::get_transform()const
 synfig::Layer::Handle
 Warp::hit_check(synfig::Context context, const synfig::Point &p)const
 {
+	Point src_tl=param_src_tl.get(Point());
+	Point src_br=param_src_br.get(Point());
+	bool clip=param_clip.get(bool());
+	
 	Point newpos(transform_forward(p));
 
 	if(clip)
@@ -463,6 +478,11 @@ Warp::hit_check(synfig::Context context, const synfig::Point &p)const
 Color
 Warp::get_color(Context context, const Point &p)const
 {
+	Point src_tl=param_src_tl.get(Point());
+	Point src_br=param_src_br.get(Point());
+	Real horizon=param_horizon.get(Real());
+	bool clip=param_clip.get(bool());
+
 	Point newpos(transform_forward(p));
 
 	if(clip)
@@ -484,6 +504,15 @@ Warp::get_color(Context context, const Point &p)const
 bool
 Warp::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Point src_tl=param_src_tl.get(Point());
+	Point src_br=param_src_br.get(Point());
+	Point dest_tl=param_dest_tl.get(Point());
+	Point dest_tr=param_dest_tr.get(Point());
+	Point dest_bl=param_dest_bl.get(Point());
+	Point dest_br=param_dest_br.get(Point());
+	Real horizon=param_horizon.get(Real());
+	bool clip=param_clip.get(bool());
+
 	SuperCallback stageone(cb,0,9000,10000);
 	SuperCallback stagetwo(cb,9000,10000,10000);
 
@@ -802,11 +831,26 @@ Warp::accelerated_render(Context context,Surface *surface,int quality, const Ren
 
 //////////
 bool
-Warp::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Warp::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc_, ProgressCallback *cb)const
 {
+	Point src_tl=param_src_tl.get(Point());
+	Point src_br=param_src_br.get(Point());
+	Point dest_tl=param_dest_tl.get(Point());
+	Point dest_tr=param_dest_tr.get(Point());
+	Point dest_bl=param_dest_bl.get(Point());
+	Point dest_br=param_dest_br.get(Point());
+	Real horizon=param_horizon.get(Real());
+	bool clip=param_clip.get(bool());
+
 	SuperCallback stageone(cb,0,9000,10000);
 	SuperCallback stagetwo(cb,9000,10000,10000);
 	
+	
+	RendDesc renddesc(renddesc_);
+	// Untransform the render desc
+	if(!cairo_renddesc_untransform(cr, renddesc))
+		return false;
+	
 	Real pw=(renddesc.get_w())/(renddesc.get_br()[0]-renddesc.get_tl()[0]);
 	Real ph=(renddesc.get_h())/(renddesc.get_br()[1]-renddesc.get_tl()[1]);
 	
@@ -827,10 +871,10 @@ Warp::accelerated_cairorender(Context context,cairo_surface_t *surface,int quali
 	// Quick exclusion clip, if necessary
 	if(clip && !intersect(render_rect,dest_rect))
 	{
-		cairo_t* cr=cairo_create(surface);
+		cairo_save(cr);
 		cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
 		cairo_paint(cr);
-		cairo_destroy(cr);
+		cairo_restore(cr);
 		return true;
 	}
 	
@@ -910,10 +954,10 @@ Warp::accelerated_cairorender(Context context,cairo_surface_t *surface,int quali
 		
 		if(!init_point_set)
 		{
-			cairo_t* cr=cairo_create(surface);
+			cairo_save(cr);
 			cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
 			cairo_paint(cr);
-			cairo_destroy(cr);
+			cairo_restore(cr);
 			return true;
 		}
 		zoom_factor=(1+(maxz-minz));
@@ -921,7 +965,7 @@ Warp::accelerated_cairorender(Context context,cairo_surface_t *surface,int quali
 	}
 	
 #ifdef ACCEL_WARP_IS_BROKEN
-	return Layer::accelerated_cairorender(context,surface,quality,renddesc, cb);
+	return Layer::accelerated_cairorender(context,cr,quality,renddesc, cb);
 #else
 	
 	/*swap(tl[1],br[1]);
@@ -976,8 +1020,6 @@ Warp::accelerated_cairorender(Context context,cairo_surface_t *surface,int quali
 		tl[1]=min_point[1];
 	}
 	
-	
-	
 	const int tmp_d(max(renddesc.get_w(),renddesc.get_h()));
 	Real src_pw=(tmp_d*zoom_factor)/(br[0]-tl[0]);
 	Real src_ph=(tmp_d*zoom_factor)/(br[1]-tl[1]);
@@ -993,10 +1035,10 @@ Warp::accelerated_cairorender(Context context,cairo_surface_t *surface,int quali
 	//synfig::warning("surface to render: [%f,%f]-[%f,%f] %dx%d",desc.get_tl()[0],desc.get_tl()[1],desc.get_br()[0],desc.get_br()[1],desc.get_w(),desc.get_h());
 	if(desc.get_w()==0 && desc.get_h()==0)
 	{
-		cairo_t* cr=cairo_create(surface);
+		cairo_save(cr);
 		cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
 		cairo_paint(cr);
-		cairo_destroy(cr);
+		cairo_restore(cr);
 		return true;
 	}
 	
@@ -1004,16 +1046,17 @@ Warp::accelerated_cairorender(Context context,cairo_surface_t *surface,int quali
 	src_pw=(desc.get_w())/(desc.get_br()[0]-desc.get_tl()[0]);
 	src_ph=(desc.get_h())/(desc.get_br()[1]-desc.get_tl()[1]);
 	
-	cairo_surface_t* source=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, desc.get_w(),desc.get_h());
-	
-	if(!context.accelerated_cairorender(source,quality,desc,&stageone))
+	cairo_surface_t* source=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, desc.get_w(),desc.get_h());
+	cairo_surface_t* surface=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA,renddesc.get_w(), renddesc.get_h());
+	cairo_t* subcr=cairo_create(source);
+	cairo_scale(subcr, 1/desc.get_pw(), 1/desc.get_ph());
+	cairo_translate(subcr, -desc.get_tl()[0], -desc.get_tl()[1]);
+
+	if(!context.accelerated_cairorender(subcr,quality,desc,&stageone))
 		return false;
 	
-	cairo_t* cr=cairo_create(surface);
-	cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
-	cairo_paint(cr);
-	cairo_destroy(cr);
-	
+	cairo_destroy(subcr);
+		
 	int surfacew, surfaceh, sourcew, sourceh;
 	
 	CairoSurface csurface(surface);
@@ -1026,7 +1069,7 @@ Warp::accelerated_cairorender(Context context,cairo_surface_t *surface,int quali
 	surfaceh=csurface.get_h();
 	sourcew=csource.get_w();
 	sourceh=csource.get_h();
-
+	
 	CairoSurface::pen pen(csurface.begin());
 	
 	// Do the warp
@@ -1060,7 +1103,7 @@ Warp::accelerated_cairorender(Context context,cairo_surface_t *surface,int quali
 					else if(quality<=6)
 						csurface[y][x]=csource.linear_sample_cooked(u,v);
 					else
-					// NEAREST_NEIGHBOR
+						// NEAREST_NEIGHBOR
 						csurface[y][x]=csource[floor_to_int(v)][floor_to_int(u)];
 				}
 			}
@@ -1079,7 +1122,18 @@ Warp::accelerated_cairorender(Context context,cairo_surface_t *surface,int quali
 	csurface.unmap_cairo_image();
 	csource.unmap_cairo_image();
 	cairo_surface_destroy(source);
-
+	
+	cairo_save(cr);
+	
+	cairo_translate(cr, renddesc.get_tl()[0], renddesc.get_tl()[1]);
+	cairo_scale(cr, renddesc.get_pw(), renddesc.get_ph());
+	cairo_set_source_surface(cr, surface, 0, 0);
+	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+	cairo_paint(cr);
+	
+	cairo_restore(cr);
+	
+	cairo_surface_destroy(surface);
 	return true;
 }
 
@@ -1094,6 +1148,10 @@ Warp::get_full_bounding_rect(Context context)const
 {
 //	return Rect::full_plane();
 
+	Point src_tl=param_src_tl.get(Point());
+	Point src_br=param_src_br.get(Point());
+	bool clip=param_clip.get(bool());
+
 	Rect under(context.get_full_bounding_rect());
 
 	if(clip)
