@@ -61,19 +61,19 @@ SYNFIG_LAYER_SET_CVS_ID(Zoom,"$Id$");
 /* === E N T R Y P O I N T ================================================= */
 
 Zoom::Zoom():
-	center(0,0),
-	amount(0)
+	param_center(ValueBase(Vector(0,0))),
+	param_amount(ValueBase(Real(0)))
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 bool
 Zoom::set_param(const String & param, const ValueBase &value)
 {
 
-	IMPORT(center);
-	IMPORT(amount);
+	IMPORT_VALUE(param_center);
+	IMPORT_VALUE(param_amount);
 
 	return false;
 }
@@ -81,8 +81,8 @@ Zoom::set_param(const String & param, const ValueBase &value)
 ValueBase
 Zoom::get_param(const String &param)const
 {
-	EXPORT(center);
-	EXPORT(amount);
+	EXPORT_VALUE(param_center);
+	EXPORT_VALUE(param_amount);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -98,6 +98,8 @@ Zoom::get_param_vocab()const
 	ret.push_back(ParamDesc("amount")
 		.set_local_name(_("Amount"))
 		.set_description(_("Amount to scale to"))
+		.set_origin("center")
+		.set_exponential()
 	);
 
 	ret.push_back(ParamDesc("center")
@@ -111,13 +113,15 @@ Zoom::get_param_vocab()const
 synfig::Layer::Handle
 Zoom::hit_check(synfig::Context context, const synfig::Point &pos)const
 {
-	return context.hit_check((pos-center)/exp(amount)+center);
+	Vector center=param_center.get(Vector());
+	return context.hit_check((pos-center)/exp(param_amount.get(Real()))+center);
 }
 
 Color
 Zoom::get_color(Context context, const Point &pos)const
 {
-	return context.get_color((pos-center)/exp(amount)+center);
+	Vector center=param_center.get(Vector());
+	return context.get_color((pos-center)/exp(param_amount.get(Real()))+center);
 }
 
 class Zoom_Trans : public Transform
@@ -128,12 +132,21 @@ public:
 
 	synfig::Vector perform(const synfig::Vector& x)const
 	{
-		return (x-layer->center)*exp(layer->amount)+layer->center;
+		Vector center=layer->param_center.get(Vector());
+		Real amount=layer->param_amount.get(Real());
+		return (x-center)*exp(amount)+center;
 	}
 
 	synfig::Vector unperform(const synfig::Vector& x)const
 	{
-		return (x-layer->center)/exp(layer->amount)+layer->center;
+		Vector center=layer->param_center.get(Vector());
+		Real amount=layer->param_amount.get(Real());
+		return (x-center)/exp(amount)+center;
+	}
+
+	synfig::String get_string()const
+	{
+		return "zoom";
 	}
 };
 etl::handle<Transform>
@@ -145,6 +158,9 @@ Zoom::get_transform()const
 bool
 Zoom::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Vector center=param_center.get(Vector());
+	Real amount=param_amount.get(Real());
+
 	Vector::value_type zoomfactor=1.0/exp(amount);
 	RendDesc desc(renddesc);
 	desc.clear_flags();
@@ -158,21 +174,28 @@ Zoom::accelerated_render(Context context,Surface *surface,int quality, const Ren
 	return context.accelerated_render(surface,quality,desc,cb);
 }
 
+
 /////
 bool
-Zoom::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Zoom::accelerated_cairorender(Context context, cairo_t *cr,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
-	Vector::value_type zoomfactor=1.0/exp(amount);
-	RendDesc desc(renddesc);
-	desc.clear_flags();
+	Vector center=param_center.get(Vector());
+	Real amount=param_amount.get(Real());
+
+	double zoomfactor=exp(amount);
 	
-    // Adjust the top_left and bottom_right points
-	// for our zoom amount
-	desc.set_tl((desc.get_tl()-center)*zoomfactor+center);
-	desc.set_br((desc.get_br()-center)*zoomfactor+center);
-	
-	// Render the scene
-	return context.accelerated_cairorender(surface,quality,desc,cb);
+	cairo_save(cr);
+	cairo_translate(cr, center[0], center[1]);
+	cairo_scale(cr, zoomfactor, zoomfactor);
+	cairo_translate(cr, -center[0], -center[1]);
+
+	if(!context.accelerated_cairorender(cr,quality,renddesc,cb))
+	{
+		cairo_restore(cr);
+		return false;
+	}
+	cairo_restore(cr);
+	return true;
 }
 /////
 
@@ -180,6 +203,7 @@ Zoom::accelerated_cairorender(Context context,cairo_surface_t *surface,int quali
 synfig::Rect
 Zoom::get_full_bounding_rect(synfig::Context context)const
 {
-	return (context.get_full_bounding_rect()-center)*exp(amount)+center;
+	Vector center=param_center.get(Vector());
+	return (context.get_full_bounding_rect()-center)*exp(param_amount.get(Real()))+center;
 }
 
