@@ -42,6 +42,7 @@
 #include <synfig/value.h>
 #include <synfig/valuenode.h>
 #include <synfig/segment.h>
+#include <synfig/cairo_renddesc.h>
 
 #include <cstring>
 #include <ETL/pen>
@@ -80,19 +81,24 @@ inline void clamp(synfig::Vector &v)
 
 Blur_Layer::Blur_Layer():
 	Layer_Composite(1.0,Color::BLEND_STRAIGHT),
-	size(0.1,0.1),
-	type(Blur::FASTGAUSSIAN)
+	param_size(ValueBase(Point(0.1,0.1))),
+	param_type(ValueBase(int(Blur::FASTGAUSSIAN)))
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
-	set_param_static("blend_method", true);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 bool
 Blur_Layer::set_param(const String &param, const ValueBase &value)
 {
-	IMPORT_PLUS(size,clamp(size));
-	IMPORT(type);
+	IMPORT_VALUE_PLUS(param_size,
+		{
+			synfig::Point size=param_size.get(Point());
+			clamp(size);
+			param_size.set(size);
+		});
+		
+	IMPORT_VALUE(param_type);
 
 	return Layer_Composite::set_param(param,value);
 }
@@ -100,8 +106,8 @@ Blur_Layer::set_param(const String &param, const ValueBase &value)
 ValueBase
 Blur_Layer::get_param(const String &param)const
 {
-	EXPORT(size);
-	EXPORT(type);
+	EXPORT_VALUE(param_size);
+	EXPORT_VALUE(param_type);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -112,6 +118,9 @@ Blur_Layer::get_param(const String &param)const
 Color
 Blur_Layer::get_color(Context context, const Point &pos)const
 {
+	synfig::Point size=param_size.get(Point());
+	int type=param_type.get(int());
+	
 	Point blurpos = Blur(size,type)(pos);
 
 	if(get_amount()==1.0 && get_blend_method()==Color::BLEND_STRAIGHT)
@@ -126,6 +135,9 @@ Blur_Layer::get_color(Context context, const Point &pos)const
 bool
 Blur_Layer::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	synfig::Point size=param_size.get(Point());
+	int type=param_type.get(int());
+
 	// don't do anything at quality 10
 	if (quality == 10)
 		return context.accelerated_render(surface,quality,renddesc,cb);
@@ -239,26 +251,19 @@ Blur_Layer::accelerated_render(Context context,Surface *surface,int quality, con
 
 /////
 bool
-Blur_Layer::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Blur_Layer::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	synfig::Point size=param_size.get(Point());
+	int type=param_type.get(int());
+
 	// don't do anything at quality 10
 	if (quality == 10)
-		return context.accelerated_cairorender(surface,quality,renddesc,cb);
+		return context.accelerated_cairorender(cr,quality,renddesc,cb);
 	
-	// int x,y;
 	SuperCallback stageone(cb,0,5000,10000);
 	SuperCallback stagetwo(cb,5000,10000,10000);
-	
-	const int	w = renddesc.get_w(), h = renddesc.get_h();
-	const Real	pw = renddesc.get_pw(),	ph = renddesc.get_ph();
-	
-	const Point	tl(renddesc.get_tl());
-	const Point br(renddesc.get_br());
-	
-	RendDesc	workdesc(renddesc);
-	cairo_surface_t	*worksurface, *blurred;
-
-	//callbacks depend on how long the blur takes
+	// Calculate the callbacks sizes
+	// callbacks depend on how long the blur takes
 	if(size[0] || size[1])
 	{
 		if(type == Blur::DISC)
@@ -277,12 +282,20 @@ Blur_Layer::accelerated_cairorender(Context context,cairo_surface_t *surface,int
 		stageone = SuperCallback(cb,0,9999,10000);
 		stagetwo = SuperCallback(cb,9999,10000,10000);
 	}
+
+	RendDesc	workdesc(renddesc);
+	cairo_surface_t	*worksurface, *blurred;
 	
-	//expand the working surface to accommodate the blur
+	if(!cairo_renddesc_untransform(cr, workdesc))
+		return false;
 	
+	// Expand the working surface to accommodate the blur
 	//the expanded size = 1/2 the size in each direction rounded up
-	int	halfsizex = (int) (abs(size[0]*.5/pw) + 3),
-	halfsizey = (int) (abs(size[1]*.5/ph) + 3);
+	int w=workdesc.get_w(), h=workdesc.get_h();
+	const double wpw=(workdesc.get_br()[0]-workdesc.get_tl()[0])/w;
+	const double wph=(workdesc.get_br()[1]-workdesc.get_tl()[1])/h;
+	int	halfsizex = (int) (abs(size[0]*.5/wpw) + 3),
+	halfsizey = (int) (abs(size[1]*.5/wph) + 3);
 	
 	//expand by 1/2 size in each direction on either side
 	switch(type)
@@ -324,31 +337,60 @@ Blur_Layer::accelerated_cairorender(Context context,cairo_surface_t *surface,int
 		}
 	}
 	
-	worksurface=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, workdesc.get_w(), workdesc.get_h());
-	//render the background onto the expanded surface
-	if(!context.accelerated_cairorender(worksurface,quality,workdesc,&stageone))
+	// New expanded workdesc values
+	const int ww=workdesc.get_w();
+	const int wh=workdesc.get_h();
+	const double wtlx=workdesc.get_tl()[0];
+	const double wtly=workdesc.get_tl()[1];
+	
+	// Create a surface to work on
+	worksurface=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, ww, wh);
+	// Create one new cairo_t* to render the layer's context on worksurface
+	cairo_t* cr2=cairo_create(worksurface);
+	// We need to scale up the surface to the pixel size to be able to make the blur
+	cairo_scale(cr2, 1/wpw, 1/wph);
+	cairo_translate(cr2, -wtlx, -wtly);
+	// Lets render the background onto the expanded surface
+	if(!context.accelerated_cairorender(cr2,quality,workdesc,&stageone))
 	{
 		if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
 		cairo_surface_destroy(worksurface);
 		return false;
 	}
-	blurred=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, workdesc.get_w(), workdesc.get_h());
-	
+	// If the context was rendered, then blur it.
+	blurred=cairo_surface_create_similar(worksurface, CAIRO_CONTENT_COLOR_ALPHA, ww, wh);
 	//blur the image
 	Blur(size,type,&stagetwo)(worksurface,workdesc.get_br()-workdesc.get_tl(),blurred);
-
-	cairo_t *cr=cairo_create(surface);
+	// We are done with cr2, destroy it
+	cairo_destroy(cr2);
+	// Let's composite the two surfaces: blurred over worksurface
+	cr2=cairo_create(worksurface);
+	cairo_set_source_surface(cr2, blurred, 0, 0);
+	cairo_paint_with_alpha_operator(cr2, get_amount(), get_blend_method());
+	cairo_destroy(cr2);
+	// Now let's paint the blurred result on the cairo context.
+	// But first we need to scale it down to the user coordinate space so when the
+	// blurred surface is treated as the rest of layers it scales right to pixel size.
+	// WE need to scale down the same that we scaled up to access the pixels.
+	// This iwhat we would obtain:
+	// [T][S][DRAW1][T1'][S1'][Blur][T1][S1][DRAW2]
+	//                              ------CR2-----
+	//                        -----image surface--
+	// --------------CR---------------------------
+	// Where:
+	// [T][S] are the user to device transformation from renddesc
+	// [DRAW1] are potential cairo operations before the blur layer
+	// [T1][S1] are the user to device transformations from workdesc
+	// [T1'][S1'] are the inverse of above
+	// [DRAW2]is the cairo drawing operations below the blur
 	cairo_save(cr);
-	cairo_set_source_surface(cr, worksurface, -halfsizex, -halfsizey);
+	cairo_translate(cr, wtlx, wtly);
+	cairo_scale(cr, wpw, wph);
+	// then set the source surface the worksurface
+	cairo_set_source_surface(cr, worksurface, 0, 0);
 	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
 	cairo_paint(cr);
 	cairo_restore(cr);
-	
-	cairo_save(cr);
-	cairo_set_source_surface(cr, blurred, -halfsizex, -halfsizey);
-	cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
-	cairo_restore(cr);
-
 	if(cb && !cb->amount_complete(10000,10000))
 	{
 		if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Renderer Failure",__LINE__));
@@ -356,12 +398,11 @@ Blur_Layer::accelerated_cairorender(Context context,cairo_surface_t *surface,int
 	}
 	cairo_surface_destroy(blurred);
 	cairo_surface_destroy(worksurface);
-	cairo_destroy(cr);
-	return true;
-	
+	return true;	
 }
 
-/////
+
+
 
 Layer::Vocab
 Blur_Layer::get_param_vocab(void)const
@@ -389,6 +430,8 @@ Blur_Layer::get_param_vocab(void)const
 Rect
 Blur_Layer::get_full_bounding_rect(Context context)const
 {
+	synfig::Point size=param_size.get(Point());
+
 	if(is_disabled() || Color::is_onto(get_blend_method()))
 		return context.get_full_bounding_rect();
 
