@@ -39,6 +39,7 @@
 #include <synfig/surface.h>
 #include <synfig/value.h>
 #include <synfig/valuenode.h>
+#include <synfig/cairo_renddesc.h>
 
 #endif
 
@@ -64,20 +65,26 @@ SYNFIG_LAYER_SET_CVS_ID(Layer_ColorCorrect,"$Id$");
 /* === E N T R Y P O I N T ================================================= */
 
 Layer_ColorCorrect::Layer_ColorCorrect():
-	hue_adjust(Angle::zero()),
-	brightness(0),
-	contrast(1.0),
-	exposure(0.0)
+	param_hue_adjust(ValueBase(Angle::zero())),
+	param_brightness(ValueBase(Real(0))),
+	param_contrast(ValueBase(Real(1.0))),
+	param_exposure(ValueBase(Real(0.0))),
+	param_gamma(ValueBase(Real(1.0)))
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 inline Color
 Layer_ColorCorrect::correct_color(const Color &in)const
 {
+	Angle hue_adjust=param_hue_adjust.get(Angle());
+	Real _brightness=param_brightness.get(Real());
+	Real contrast=param_contrast.get(Real());
+	Real exposure=param_exposure.get(Real());
+	
 	Color ret(in);
-	Real brightness((this->brightness-0.5)*this->contrast+0.5);
+	Real brightness((_brightness-0.5)*contrast+0.5);
 
 	if(gamma.get_gamma_r()!=1.0)
 	{
@@ -167,32 +174,31 @@ Layer_ColorCorrect::correct_color(const Color &in)const
 bool
 Layer_ColorCorrect::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT(hue_adjust);
-	IMPORT(brightness);
-	IMPORT(contrast);
-	IMPORT(exposure);
+	IMPORT_VALUE(param_hue_adjust);
+	IMPORT_VALUE(param_brightness);
+	IMPORT_VALUE(param_contrast);
+	IMPORT_VALUE(param_exposure);
 
-	if(param=="gamma" && value.get_type()==ValueBase::TYPE_REAL)
-	{
-		gamma.set_gamma(1.0/value.get(Real()));
-		set_param_static(param, value.get_static());
-		return true;
-	}
+	IMPORT_VALUE_PLUS(param_gamma,
+		{
+			gamma.set_gamma(1.0/param_gamma.get(Real()));
+			return true;
+		});
 	return false;
 }
 
 ValueBase
 Layer_ColorCorrect::get_param(const String &param)const
 {
-	EXPORT(hue_adjust);
-	EXPORT(brightness);
-	EXPORT(contrast);
-	EXPORT(exposure);
+	EXPORT_VALUE(param_hue_adjust);
+	EXPORT_VALUE(param_brightness);
+	EXPORT_VALUE(param_contrast);
+	EXPORT_VALUE(param_exposure);
 
 	if(param=="gamma")
 	{
-		ValueBase ret(1.0/gamma.get_gamma());
-		ret.set_static(get_param_static(param));
+		ValueBase ret=param_gamma;
+		ret.set(1.0/gamma.get_gamma());
 		return ret;
 	}
 
@@ -260,14 +266,34 @@ Layer_ColorCorrect::accelerated_render(Context context,Surface *surface,int qual
 }
 
 bool
-Layer_ColorCorrect::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Layer_ColorCorrect::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc_, ProgressCallback *cb)const
 {
-	SuperCallback supercb(cb,0,9500,10000);
+	RendDesc	renddesc(renddesc_);
 	
-	if(!context.accelerated_cairorender(surface,quality,renddesc,&supercb))
+	// Untransform the render desc
+	if(!cairo_renddesc_untransform(cr, renddesc))
 		return false;
 	
+	const Real pw(renddesc.get_pw()),ph(renddesc.get_ph());
+	const Point tl(renddesc.get_tl());
+	const int w(renddesc.get_w());
+	const int h(renddesc.get_h());
+
 	int x,y;
+
+	SuperCallback supercb(cb,0,9500,10000);
+
+	cairo_surface_t *surface;
+	
+	surface=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, w, h);
+	cairo_t* subcr=cairo_create(surface);
+	cairo_scale(subcr, 1/pw, 1/ph);
+	cairo_translate(subcr, -tl[0], -tl[1]);
+	
+	if(!context.accelerated_cairorender(subcr,quality,renddesc,&supercb))
+		return false;
+
+	cairo_destroy(subcr);
 	
 	CairoSurface csurface(surface);
 	csurface.map_cairo_image();
@@ -279,6 +305,17 @@ Layer_ColorCorrect::accelerated_cairorender(Context context,cairo_surface_t *sur
 			pen.put_value(CairoColor(correct_color(Color(pen.get_value().demult_alpha())).clamped()).premult_alpha());
 	
 	csurface.unmap_cairo_image();
+	
+	// paint surface on cr
+	cairo_save(cr);
+	cairo_translate(cr, tl[0], tl[1]);
+	cairo_scale(cr, pw, ph);
+	cairo_set_source_surface(cr, surface, 0, 0);
+	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+	cairo_paint(cr);
+	cairo_restore(cr);
+	
+	cairo_surface_destroy(surface);
 	// Mark our progress as finished
 	if(cb && !cb->amount_complete(10000,10000))
 		return false;
@@ -286,7 +323,6 @@ Layer_ColorCorrect::accelerated_cairorender(Context context,cairo_surface_t *sur
 	return true;
 }
 
-
 Rect
 Layer_ColorCorrect::get_full_bounding_rect(Context context)const
 {
