@@ -42,6 +42,7 @@
 #include <synfig/surface.h>
 #include <synfig/value.h>
 #include <synfig/valuenode.h>
+#include <synfig/cairo_renddesc.h>
 
 #endif
 
@@ -65,23 +66,25 @@ SYNFIG_LAYER_SET_CVS_ID(Halftone2,"$Id$");
 /* === M E T H O D S ======================================================= */
 
 Halftone2::Halftone2():
-	color_dark(Color::black()),
-	color_light(Color::white())
+	Layer_Composite(1.0,Color::BLEND_STRAIGHT),
+	param_color_dark(ValueBase(Color::black())),
+	param_color_light(ValueBase(Color::white()))
 {
-	halftone.origin=(synfig::Point(0,0));
-	halftone.size=(synfig::Vector(0.25,0.25));
-	halftone.angle=(Angle::zero());
-	halftone.type=TYPE_SYMMETRIC;
+	halftone.param_origin=ValueBase(synfig::Point(0,0));
+	halftone.param_size=ValueBase(synfig::Vector(0.25,0.25));
+	halftone.param_angle=ValueBase(Angle::zero());
+	halftone.param_type=ValueBase(int(TYPE_SYMMETRIC));
 
-	set_blend_method(Color::BLEND_STRAIGHT);
-
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 inline Color
 Halftone2::color_func(const Point &point, float supersample,const Color& color)const
 {
+	Color color_dark=param_color_dark.get(Color());
+	Color color_light=param_color_light.get(Color());
+
 	const float amount(halftone(point,color.get_y(),supersample));
 	Color halfcolor;
 
@@ -100,7 +103,7 @@ Halftone2::color_func(const Point &point, float supersample,const Color& color)c
 inline float
 Halftone2::calc_supersample(const synfig::Point &/*x*/, float pw,float /*ph*/)const
 {
-	return abs(pw/(halftone.size).mag());
+	return abs(pw/(halftone.param_size.get(Vector())).mag());
 }
 
 synfig::Layer::Handle
@@ -112,15 +115,16 @@ Halftone2::hit_check(synfig::Context /*context*/, const synfig::Point &/*point*/
 bool
 Halftone2::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT(color_dark);
-	IMPORT(color_light);
+	IMPORT_VALUE(param_color_dark);
+	IMPORT_VALUE(param_color_light);
 
-	IMPORT_AS(halftone.size,"size");
-	IMPORT_AS(halftone.type,"type");
-	IMPORT_AS(halftone.angle,"angle");
-	IMPORT_AS(halftone.origin,"origin");
+	HALFTONE2_IMPORT_VALUE(halftone.param_size);
+	HALFTONE2_IMPORT_VALUE(halftone.param_type);
+	HALFTONE2_IMPORT_VALUE(halftone.param_angle);
+	HALFTONE2_IMPORT_VALUE(halftone.param_origin);
 
-	IMPORT_AS(halftone.origin,"offset");
+	if(param=="offset")
+		return set_param("origin", value);
 
 	return Layer_Composite::set_param(param,value);
 }
@@ -128,13 +132,13 @@ Halftone2::set_param(const String & param, const ValueBase &value)
 ValueBase
 Halftone2::get_param(const String & param)const
 {
-	EXPORT_AS(halftone.size,"size");
-	EXPORT_AS(halftone.type,"type");
-	EXPORT_AS(halftone.angle,"angle");
-	EXPORT_AS(halftone.origin,"origin");
+	EXPORT_VALUE(param_color_dark);
+	EXPORT_VALUE(param_color_light);
 
-	EXPORT(color_dark);
-	EXPORT(color_light);
+	HALFTONE2_EXPORT_VALUE(halftone.param_size);
+	HALFTONE2_EXPORT_VALUE(halftone.param_type);
+	HALFTONE2_EXPORT_VALUE(halftone.param_angle);
+	HALFTONE2_EXPORT_VALUE(halftone.param_origin);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -205,7 +209,7 @@ Halftone2::accelerated_render(Context context,Surface *surface,int quality, cons
 	const Point tl(renddesc.get_tl());
 	const int w(surface->get_w());
 	const int h(surface->get_h());
-	const float supersample_size(abs(pw/(halftone.size).mag()));
+	const float supersample_size(abs(pw/(halftone.param_size.get(Vector())).mag()));
 
 	Surface::pen pen(surface->begin());
 	Point pos;
@@ -248,28 +252,48 @@ Halftone2::accelerated_render(Context context,Surface *surface,int quality, cons
 	return true;
 }
 
-
+///
 ///
 bool
-Halftone2::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Halftone2::accelerated_cairorender(Context context,cairo_t *cr,int quality, const RendDesc &renddesc_, ProgressCallback *cb)const
 {
-	SuperCallback supercb(cb,0,9500,10000);
+	RendDesc	renddesc(renddesc_);
 	
-	if(!context.accelerated_cairorender(surface,quality,renddesc,&supercb))
-		return false;
-	if(get_amount()==0)
-		return true;
-	
-	CairoSurface csurface(surface);
-	if(!csurface.map_cairo_image())
+	// Untransform the render desc
+	if(!cairo_renddesc_untransform(cr, renddesc))
 		return false;
+
 	const Real pw(renddesc.get_pw()),ph(renddesc.get_ph());
 	const Point tl(renddesc.get_tl());
-	const int w(csurface.get_w());
-	const int h(csurface.get_h());
-	const float supersample_size(abs(pw/(halftone.size).mag()));
+	const int w(renddesc.get_w());
+	const int h(renddesc.get_h());
+	const float supersample_size(abs(pw/(halftone.param_size.get(Vector())).mag()));
+
+	SuperCallback supercb(cb,0,9500,10000);
 	
+	if(get_amount()==0)
+		return true;
+		
+	cairo_surface_t *surface;
+
+	surface=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, w, h);
+	cairo_t* subcr=cairo_create(surface);
+	cairo_scale(subcr, 1/pw, 1/ph);
+	cairo_translate(subcr, -tl[0], -tl[1]);
+	if(!context.accelerated_cairorender(subcr,quality,renddesc,&supercb))
+	{
+		if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
+		return false;
+	}
+	cairo_destroy(subcr);
+
+	CairoSurface csurface(surface);
+
+	if(!csurface.map_cairo_image())
+		return false;
+
 	CairoSurface::pen pen(csurface.begin());
+	
 	Point pos;
 	int x,y;
 	
@@ -278,13 +302,13 @@ Halftone2::accelerated_cairorender(Context context,cairo_surface_t *surface,int
 		for(y=0,pos[1]=tl[1];y<h;y++,pen.inc_y(),pen.dec_x(x),pos[1]+=ph)
 			for(x=0,pos[0]=tl[0];x<w;x++,pen.inc_x(),pos[0]+=pw)
 				pen.put_value(
-					CairoColor(color_func(
-										  pos,
-										  supersample_size,
-										  Color(pen.get_value().demult_alpha())
-										 )
-					).premult_alpha()
-				);
+							  CairoColor(color_func(
+													pos,
+													supersample_size,
+													Color(pen.get_value().demult_alpha())
+													)
+										 ).premult_alpha()
+							  );
 	}
 	else
 	{
@@ -293,23 +317,33 @@ Halftone2::accelerated_cairorender(Context context,cairo_surface_t *surface,int
 			{
 				Color val=Color(pen.get_value().demult_alpha());
 				pen.put_value(
-						CairoColor(Color::blend(
-											  color_func(
-														 pos,
-														 supersample_size,
-														 val
-														 ),
-											  val,
-											  get_amount(),
-											  get_blend_method()
-											  ).clamped()
-						).premult_alpha()
-				);
+							  CairoColor(Color::blend(
+													  color_func(
+																 pos,
+																 supersample_size,
+																 val
+																 ),
+													  val,
+													  get_amount(),
+													  get_blend_method()
+													  ).clamped()
+										 ).premult_alpha()
+							  );
 			}
-				
+		
 	}
-
+	
 	csurface.unmap_cairo_image();
+	// paint surface on cr
+	cairo_save(cr);
+	cairo_translate(cr, tl[0], tl[1]);
+	cairo_scale(cr, pw, ph);
+	cairo_set_source_surface(cr, surface, 0, 0);
+	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+	cairo_paint(cr);
+	cairo_restore(cr);
+	
+	cairo_surface_destroy(surface);
 	// Mark our progress as finished
 	if(cb && !cb->amount_complete(10000,10000))
 		return false;
