@@ -42,6 +42,7 @@
 #include <synfig/surface.h>
 #include <synfig/value.h>
 #include <synfig/valuenode.h>
+#include <synfig/cairo_renddesc.h>
 
 #endif
 
@@ -67,32 +68,33 @@ SYNFIG_LAYER_SET_CVS_ID(Halftone3,"$Id$");
 
 /* === M E T H O D S ======================================================= */
 
-Halftone3::Halftone3()
+Halftone3::Halftone3():
+Layer_Composite(1.0,Color::BLEND_STRAIGHT)
 {
-	size=(synfig::Vector(0.25,0.25));
-	type=TYPE_SYMMETRIC;
+	param_size=ValueBase(synfig::Vector(0.25,0.25));
+	param_type=ValueBase(int(TYPE_SYMMETRIC));
 
 	for(int i=0;i<3;i++)
 	{
-		tone[i].size=size;
-		tone[i].type=type;
-		tone[i].origin=(synfig::Point(0,0));
-		tone[i].angle=Angle::deg(30.0)*(float)i;
+		tone[i].param_size=param_size;
+		tone[i].param_type=param_type;
+		tone[i].param_origin=ValueBase(synfig::Point(0,0));
+		tone[i].param_angle=ValueBase(Angle::deg(30.0)*(float)i);
 	}
 
-	subtractive=true;
+	param_subtractive=ValueBase(true);
 
-	if(subtractive)
+	if(param_subtractive.get(bool()))
 	{
-		color[0]=Color::cyan();
-		color[1]=Color::magenta();
-		color[2]=Color::yellow();
+		param_color[0].set(Color::cyan());
+		param_color[1].set(Color::magenta());
+		param_color[2].set(Color::yellow());
 	}
 	else
 	{
-		color[0]=Color::red();
-		color[1]=Color::green();
-		color[2]=Color::blue();
+		param_color[0].set(Color::red());
+		param_color[1].set(Color::green());
+		param_color[2].set(Color::blue());
 	}
 
 	set_blend_method(Color::BLEND_STRAIGHT);
@@ -102,23 +104,28 @@ Halftone3::Halftone3()
 			inverse_matrix[i][j]=(j==i)?1.0f:0.0f;
 
 	sync();
-
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 void
 Halftone3::sync()
 {
+	bool subtractive=param_subtractive.get(bool());
+	Color color[3];
+	for(int i=0;i<3;i++)
+		color[i]=param_color[i].get(Color());
+
+	// Is this needed? set_param does the same!
 	for(int i=0;i<3;i++)
 	{
-		tone[i].size=size;
-		tone[i].type=type;
+		tone[i].param_size=param_size;
+		tone[i].param_type=param_type;
 	}
 
 #define matrix inverse_matrix
 	//float matrix[3][3];
-
 	if(subtractive)
 	{
 		for(int i=0;i<3;i++)
@@ -198,6 +205,11 @@ Halftone3::sync()
 inline Color
 Halftone3::color_func(const Point &point, float supersample,const Color& in_color)const
 {
+	bool subtractive=param_subtractive.get(bool());
+	Color color[3];
+	for(int i=0;i<3;i++)
+		color[i]=param_color[i].get(Color());
+
 	Color halfcolor;
 
 	float chan[3];
@@ -236,7 +248,7 @@ Halftone3::color_func(const Point &point, float supersample,const Color& in_colo
 inline float
 Halftone3::calc_supersample(const synfig::Point &/*x*/, float pw,float /*ph*/)const
 {
-	return abs(pw/(tone[0].size).mag());
+	return abs(pw/(tone[0].param_size.get(Vector())).mag());
 }
 
 synfig::Layer::Handle
@@ -248,27 +260,37 @@ Halftone3::hit_check(synfig::Context /*context*/, const synfig::Point &/*point*/
 bool
 Halftone3::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT_PLUS(size, {tone[0].size=size; tone[1].size=size; tone[2].size=size;});
-	IMPORT_PLUS(type, {tone[0].type=type; tone[1].type=type; tone[2].type=type;});
+	IMPORT_VALUE_PLUS(param_size,
+		{
+			for(int i=0;i<3;i++)
+				tone[i].param_size=param_size;
 
-	IMPORT_PLUS(color[0],sync());
-	IMPORT_PLUS(color[1],sync());
-	IMPORT_PLUS(color[2],sync());
+		});
+	IMPORT_VALUE_PLUS(param_type,
+		{
+			for(int i=0;i<3;i++)
+				tone[i].param_type=param_type;
+		});
 
-	IMPORT_PLUS(subtractive,sync());
+	IMPORT_VALUE_PLUS(param_color[0],sync());
+	IMPORT_VALUE_PLUS(param_color[1],sync());
+	IMPORT_VALUE_PLUS(param_color[2],sync());
 
-	IMPORT(tone[0].angle);
-	IMPORT(tone[0].origin);
-
-	IMPORT(tone[1].angle);
-	IMPORT(tone[1].origin);
-
-	IMPORT(tone[2].angle);
-	IMPORT(tone[2].origin);
-
-	IMPORT_AS(tone[0].origin,"tone[0].offset");
-	IMPORT_AS(tone[1].origin,"tone[1].offset");
-	IMPORT_AS(tone[2].origin,"tone[2].offset");
+	IMPORT_VALUE_PLUS(param_subtractive,sync());
+	for(int i=0;i<3;i++)
+		if ((param==strprintf("tone[%d].angle",i) || param==strprintf("tone[%d].angle",i))
+			&& tone[i].param_angle.get_type()==value.get_type())
+		{
+			tone[i].param_angle=value;
+			return true;
+		}
+	for(int i=0;i<3;i++)
+		if ((param==strprintf("tone[%d].origin",i) || param==strprintf("tone[%d].origin",i))
+			&& tone[i].param_origin.get_type()==value.get_type())
+		{
+			tone[i].param_origin=value;
+			return true;
+		}
 
 	return Layer_Composite::set_param(param,value);
 }
@@ -276,23 +298,20 @@ Halftone3::set_param(const String & param, const ValueBase &value)
 ValueBase
 Halftone3::get_param(const String & param)const
 {
-	EXPORT(size);
-	EXPORT(type);
+	EXPORT_VALUE(param_size);
+	EXPORT_VALUE(param_type);
 
-	EXPORT(color[0]);
-	EXPORT(color[1]);
-	EXPORT(color[2]);
+	EXPORT_VALUE(param_color[0]);
+	EXPORT_VALUE(param_color[1]);
+	EXPORT_VALUE(param_color[2]);
 
-	EXPORT(subtractive);
-
-	EXPORT(tone[0].angle);
-	EXPORT(tone[0].origin);
-
-	EXPORT(tone[1].angle);
-	EXPORT(tone[1].origin);
-
-	EXPORT(tone[2].angle);
-	EXPORT(tone[2].origin);
+	EXPORT_VALUE(param_subtractive);
+	for(int i=0;i<3;i++)
+		if (param==strprintf("tone[%d].angle",i))
+			return tone[i].param_angle;
+	for(int i=0;i<3;i++)
+		if (param==strprintf("tone[%d].origin",i))
+			return tone[i].param_origin;
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -368,7 +387,7 @@ Halftone3::accelerated_render(Context context,Surface *surface,int quality, cons
 	const Point tl(renddesc.get_tl());
 	const int w(surface->get_w());
 	const int h(surface->get_h());
-	const float supersample_size(abs(pw/(tone[0].size).mag()));
+	const float supersample_size(abs(pw/(tone[0].param_size.get(Vector())).mag()));
 
 	Surface::pen pen(surface->begin());
 	Point pos;
@@ -411,27 +430,48 @@ Halftone3::accelerated_render(Context context,Surface *surface,int quality, cons
 	return true;
 }
 
+////
 ////
 bool
-Halftone3::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Halftone3::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc_, ProgressCallback *cb)const
 {
+	RendDesc	renddesc(renddesc_);
+	
+	// Untransform the render desc
+	if(!cairo_renddesc_untransform(cr, renddesc))
+		return false;
+	
+	const Real pw(renddesc.get_pw()),ph(renddesc.get_ph());
+	const Point tl(renddesc.get_tl());
+	const int w(renddesc.get_w());
+	const int h(renddesc.get_h());
+	const float supersample_size(abs(pw/(tone[0].param_size.get(Vector())).mag()));
+	
 	SuperCallback supercb(cb,0,9500,10000);
 	
-	if(!context.accelerated_cairorender(surface,quality,renddesc,&supercb))
-		return false;
 	if(get_amount()==0)
 		return true;
 	
+	cairo_surface_t *surface;
+	
+	surface=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, w, h);
+	cairo_t* subcr=cairo_create(surface);
+	cairo_scale(subcr, 1/pw, 1/ph);
+	cairo_translate(subcr, -tl[0], -tl[1]);
+	if(!context.accelerated_cairorender(subcr,quality,renddesc,&supercb))
+	{
+		if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
+		return false;
+	}
+	cairo_destroy(subcr);
+	
 	CairoSurface csurface(surface);
+	
 	if(!csurface.map_cairo_image())
 		return false;
-	const Real pw(renddesc.get_pw()),ph(renddesc.get_ph());
-	const Point tl(renddesc.get_tl());
-	const int w(csurface.get_w());
-	const int h(csurface.get_h());
-	const float supersample_size(abs(pw/(tone[0].size).mag()));
 	
 	CairoSurface::pen pen(csurface.begin());
+	
 	Point pos;
 	int x,y;
 	
@@ -440,12 +480,13 @@ Halftone3::accelerated_cairorender(Context context,cairo_surface_t *surface,int
 		for(y=0,pos[1]=tl[1];y<h;y++,pen.inc_y(),pen.dec_x(x),pos[1]+=ph)
 			for(x=0,pos[0]=tl[0];x<w;x++,pen.inc_x(),pos[0]+=pw)
 				pen.put_value(
-					CairoColor(color_func(
-							pos,
-							supersample_size,
-							Color(pen.get_value().demult_alpha())
-							)).premult_alpha()
-				);
+							  CairoColor(color_func(
+													pos,
+													supersample_size,
+													Color(pen.get_value().demult_alpha())
+													)
+										 ).premult_alpha()
+							  );
 	}
 	else
 	{
@@ -454,23 +495,33 @@ Halftone3::accelerated_cairorender(Context context,cairo_surface_t *surface,int
 			{
 				Color val=Color(pen.get_value().demult_alpha());
 				pen.put_value(
-					  CairoColor(Color::blend(
-								  color_func(
-											 pos,
-											 supersample_size,
-											 val
-											 ),
-								  val,
-								  get_amount(),
-								  get_blend_method()
-								  ).clamped()
-						 ).premult_alpha()
-				);
+							  CairoColor(Color::blend(
+													  color_func(
+																 pos,
+																 supersample_size,
+																 val
+																 ),
+													  val,
+													  get_amount(),
+													  get_blend_method()
+													  ).clamped()
+										 ).premult_alpha()
+							  );
 			}
 		
 	}
 	
 	csurface.unmap_cairo_image();
+	// paint surface on cr
+	cairo_save(cr);
+	cairo_translate(cr, tl[0], tl[1]);
+	cairo_scale(cr, pw, ph);
+	cairo_set_source_surface(cr, surface, 0, 0);
+	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+	cairo_paint(cr);
+	cairo_restore(cr);
+	
+	cairo_surface_destroy(surface);
 	// Mark our progress as finished
 	if(cb && !cb->amount_complete(10000,10000))
 		return false;
