@@ -49,9 +49,12 @@ class RadialBlur : public Layer_Composite
 	SYNFIG_LAYER_MODULE_EXT
 	friend class RadialBlur_Trans;
 private:
-	Vector origin;
-	Real size;
-	bool fade_out;
+	//! Parameter: (Vector)
+	ValueBase param_origin;
+	//! Parameter: (Real)
+	ValueBase param_size;
+	//! Parameter: (bool)
+	ValueBase param_fade_out;
 
 public:
 	RadialBlur();
@@ -61,6 +64,7 @@ public:
 	virtual ValueBase get_param(const synfig::String & param)const;
 	virtual Color get_color(Context context, const Point &pos)const;
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 	virtual Vocab get_param_vocab()const;
 	virtual bool reads_context()const { return true; }
 }; // END of class RadialBlur
