@@ -72,18 +72,22 @@ SYNFIG_LAYER_SET_CVS_ID(Advanced_Outline,"$Id$");
 Point line_intersection( const Point& p1, const Vector& t1, const Point& p2, const Vector& t2 );
 /* === M E T H O D S ======================================================= */
 
-Advanced_Outline::Advanced_Outline()
+Advanced_Outline::Advanced_Outline():
+param_bline(ValueBase(std::vector<synfig::BLinePoint>())),
+param_wplist(ValueBase(std::vector<synfig::WidthPoint>())),
+param_dilist(ValueBase(std::vector<synfig::DashItem>()))
 {
-	cusp_type_=TYPE_SHARP;
-	start_tip_= end_tip_= WidthPoint::TYPE_ROUNDED;
-	width_=1.0f;
-	expand_=0;
-	smoothness_=0.5;
-	dash_offset_=0.0;
-	homogeneous_=false;
-	dash_enabled_=false;
+	param_cusp_type = ValueBase(int(TYPE_SHARP));
+	param_start_tip = param_end_tip = ValueBase(int(WidthPoint::TYPE_ROUNDED));
+	param_width = ValueBase(Real(1.0f));
+	param_expand = ValueBase(Real(0));
+	param_smoothness = ValueBase(Real(0.5));
+	param_dash_offset = ValueBase(Real(0.0));
+	param_homogeneous = ValueBase(false);
+	param_dash_enabled = ValueBase(false);
+	param_fast = ValueBase(false);
+	
 	old_version=false;
-	fast_=false;
 	clear();
 
 	vector<BLinePoint> bline_point_list;
@@ -99,7 +103,8 @@ Advanced_Outline::Advanced_Outline()
 	bline_point_list[0].set_width(1.0f);
 	bline_point_list[1].set_width(1.0f);
 	bline_point_list[2].set_width(1.0f);
-	bline_=bline_point_list;
+	param_bline.set(bline_point_list);
+	
 	vector<WidthPoint> wpoint_list;
 	wpoint_list.push_back(WidthPoint());
 	wpoint_list.push_back(WidthPoint());
@@ -109,14 +114,14 @@ Advanced_Outline::Advanced_Outline()
 	wpoint_list[1].set_width(1.0);
 	wpoint_list[0].set_side_type_before(WidthPoint::TYPE_INTERPOLATE);
 	wpoint_list[1].set_side_type_after(WidthPoint::TYPE_INTERPOLATE);
-	wplist_=wpoint_list;
+	param_wplist.set(wpoint_list);
+	
 	vector<DashItem> ditem_list;
 	ditem_list.push_back(DashItem());
-	dilist_=ditem_list;
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
-	set_param_static("fast", true);
-
+	param_dilist.set(ditem_list);
+	
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 
@@ -127,6 +132,20 @@ Advanced_Outline::Advanced_Outline()
 void
 Advanced_Outline::sync()
 {
+	ValueBase bline_=param_bline;
+	ValueBase wplist_=param_wplist;
+	ValueBase dilist_=param_dilist;
+	int start_tip_=param_start_tip.get(int());
+	int end_tip_=param_end_tip.get(int());
+	int cusp_type_=param_cusp_type.get(int());
+	Real width_=param_width.get(Real());
+	Real expand_=param_expand.get(Real());
+	Real smoothness_=param_smoothness.get(Real());
+	bool homogeneous_=param_homogeneous.get(bool());
+	Real dash_offset_=param_dash_offset.get(Real());
+	bool dash_enabled_=param_dash_enabled.get(bool());
+	bool fast_=param_fast.get(bool());
+	
 	clear();
 	if (!bline_.get_list().size())
 	{
@@ -229,7 +248,8 @@ Advanced_Outline::sync()
 		// initialize the blinepoints positions iterators
 		hbpiter=hbline_pos.begin();
 		bpiter=bline_pos.begin();
-		Real biter_pos(*bpiter), hbiter_pos(*hbpiter);
+		// todo: why 'hbiter_pos' never used?
+		Real biter_pos(*bpiter) /* , hbiter_pos(*hbpiter) */ ;
 		bpiter++;
 		hbpiter++;
 		Real bnext_pos(*bpiter), hbnext_pos(*hbpiter);
@@ -838,7 +858,8 @@ Advanced_Outline::sync()
 					bnext++;
 					// Update blinepoints positions
 					biter_pos = bnext_pos;
-					hbiter_pos = hbnext_pos;
+					// todo: why 'hbiter_pos' never used?
+					//hbiter_pos = hbnext_pos;
 					bpiter++;
 					hbpiter++;
 					bnext_pos=*bpiter;
@@ -955,7 +976,8 @@ Advanced_Outline::sync()
 					bnext++;
 					// Update blinepoints positions
 					biter_pos = bnext_pos;
-					hbiter_pos = hbnext_pos;
+					// todo: why 'hbiter_pos' never used?
+					//hbiter_pos = hbnext_pos;
 					bpiter++;
 					hbpiter++;
 					bnext_pos=*bpiter;
@@ -1031,52 +1053,40 @@ Advanced_Outline::sync()
 bool
 Advanced_Outline::set_param(const String & param, const ValueBase &value)
 {
-	if(param=="bline" && value.get_type()==ValueBase::TYPE_LIST)
-	{
-		bline_=value;
-		return true;
-	}
-	IMPORT_AS(cusp_type_, "cusp_type");
-	IMPORT_AS(start_tip_, "start_tip");
-	IMPORT_AS(end_tip_, "end_tip");
-	IMPORT_AS(width_,"width");
-	IMPORT_AS(expand_, "expand");
-	IMPORT_AS(dash_offset_,"dash_offset");
-	IMPORT_AS(homogeneous_,"homogeneous");
-	IMPORT_AS(dash_enabled_, "dash_enabled");
-	IMPORT_AS(fast_, "fast");
-	if(param=="smoothness" && value.get_type()==ValueBase::TYPE_REAL)
-	{
-		if(value > 1.0) smoothness_=1.0;
-		else if(value < 0.0) smoothness_=0.0;
-		else smoothness_=value;
-		set_param_static("smoothness", value.get_static());
-		return true;
-	}
-	if(param=="wplist" && value.get_type()==ValueBase::TYPE_LIST)
-	{
-		wplist_=value;
-		return true;
-	}
-	if(param=="dilist" && value.get_type()==ValueBase::TYPE_LIST)
-	{
-		dilist_=value;
-		return true;
-	}
+
+	IMPORT_VALUE(param_bline);
+	IMPORT_VALUE(param_wplist);
+	IMPORT_VALUE(param_dilist);
+	IMPORT_VALUE(param_start_tip);
+	IMPORT_VALUE(param_end_tip);
+	IMPORT_VALUE(param_cusp_type);
+	IMPORT_VALUE(param_width);
+	IMPORT_VALUE(param_expand);
+	IMPORT_VALUE_PLUS(param_smoothness,
+		{
+			if(value > 1.0) param_smoothness.set(Real(1.0));
+			else if(value < 0.0) param_smoothness.set(Real(0.0));
+		}
+	);
+	IMPORT_VALUE(param_homogeneous);
+	IMPORT_VALUE(param_dash_offset);
+	IMPORT_VALUE(param_dash_enabled);
+	IMPORT_VALUE(param_fast);
+
 	if(param=="vector_list")
 		return false;
 	return Layer_Polygon::set_param(param,value);
 }
 
 void
-Advanced_Outline::set_time(Context context, Time time)const
+Advanced_Outline::set_time(IndependentContext context, Time time)const
 {
 	const_cast<Advanced_Outline*>(this)->sync();
 	context.set_time(time);
 }
 
 void
-Advanced_Outline::set_time(Context context, Time time, Vector pos)const
+Advanced_Outline::set_time(IndependentContext context, Time time, Vector pos)const
 {
 	const_cast<Advanced_Outline*>(this)->sync();
 	context.set_time(time,pos);
@@ -1093,19 +1103,20 @@ Advanced_Outline::set_version(const synfig::String &ver)
 ValueBase
 Advanced_Outline::get_param(const String& param)const
 {
-	EXPORT_AS(bline_, "bline");
-	EXPORT_AS(expand_, "expand");
-	EXPORT_AS(smoothness_, "smoothness");
-	EXPORT_AS(cusp_type_, "cusp_type");
-	EXPORT_AS(start_tip_,"start_tip");
-	EXPORT_AS(end_tip_,"end_tip");
-	EXPORT_AS(width_, "width");
-	EXPORT_AS(wplist_, "wplist");
-	EXPORT_AS(dash_offset_,"dash_offset");
-	EXPORT_AS(dilist_, "dilist");
-	EXPORT_AS(homogeneous_, "homogeneous");
-	EXPORT_AS(dash_enabled_,"dash_enabled");
-	EXPORT_AS(fast_, "fast");
+	EXPORT_VALUE(param_bline);
+	EXPORT_VALUE(param_wplist);
+	EXPORT_VALUE(param_dilist);
+	EXPORT_VALUE(param_start_tip);
+	EXPORT_VALUE(param_end_tip);
+	EXPORT_VALUE(param_cusp_type);
+	EXPORT_VALUE(param_width);
+	EXPORT_VALUE(param_expand);
+	EXPORT_VALUE(param_smoothness);
+	EXPORT_VALUE(param_homogeneous);
+	EXPORT_VALUE(param_dash_offset);
+	EXPORT_VALUE(param_dash_enabled);
+	EXPORT_VALUE(param_fast);
+
 	EXPORT_NAME();
 	EXPORT_VERSION();
 	if(param=="vector_list")
@@ -1296,6 +1307,9 @@ Advanced_Outline::bezier_to_bline(Real bezier_pos, Real origin, Real bezier_size
 void
 Advanced_Outline::add_tip(std::vector<Point> &side_a, std::vector<Point> &side_b, const Point vertex, const Vector tangent, const WidthPoint wp, const Real gv)
 {
+	Real width_=param_width.get(Real());
+	Real expand_=param_expand.get(Real());
+
 	Real w(gv*(expand_+width_*0.5*wp.get_width()));
 	// Side Before
 	switch (wp.get_side_type_before())
@@ -1413,6 +1427,8 @@ Advanced_Outline::add_tip(std::vector<Point> &side_a, std::vector<Point> &side_b
 void
 Advanced_Outline::add_cusp(std::vector<Point> &side_a, std::vector<Point> &side_b, const Point vertex, const Vector curr, const Vector last, Real w)
 {
+	int cusp_type_=param_cusp_type.get(int());
+
 	static int counter=0;
 	counter++;
 	const Vector t1(last.perp().norm());
@@ -1498,183 +1514,3 @@ Advanced_Outline::add_cusp(std::vector<Point> &side_a, std::vector<Point> &side_
 		break;
 	}
 }
-
-bool
-Advanced_Outline::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
-{
-	// Grab the rgba values
-	const float r(color.get_r());
-	const float g(color.get_g());
-	const float b(color.get_b());
-	const float a(color.get_a());
-	
-	// Window Boundaries
-	const Point	tl(renddesc.get_tl());
-	const Point br(renddesc.get_br());
-	const int	w(renddesc.get_w());
-	const int	h(renddesc.get_h());
-	
-	// Width and Height of a pixel
-	const Real pw = (br[0] - tl[0]) / w;
-	const Real ph = (br[1] - tl[1]) / h;
-	
-	// These are the scale and translation values
-	const double sx(1/pw);
-	const double sy(1/ph);
-	const double tx((-tl[0]+origin[0])*sx);
-	const double ty((-tl[1]+origin[1])*sy);
-
-	cairo_t* cr=cairo_create(surface);
-
-	// Let's render the outline in other surface
-	// Initially I'll fill it completely with the alpha color
-	cairo_surface_t* subimage;
-	// Let's calculate the subimage dimensions based on the feather value
-	//so make a separate surface
-	RendDesc	workdesc(renddesc);
-	int halfsizex(0), halfsizey(0);
-	if(feather && quality != 10)
-	{
-		//the expanded size = 1/2 the size in each direction rounded up
-		halfsizex = (int) (abs(feather*.5/pw) + 3),
-		halfsizey = (int) (abs(feather*.5/ph) + 3);
-		
-		//expand by 1/2 size in each direction on either side
-		switch(blurtype)
-		{
-			case Blur::DISC:
-			case Blur::BOX:
-			case Blur::CROSS:
-			{
-				workdesc.set_subwindow(-max(1,halfsizex),-max(1,halfsizey),w+2*max(1,halfsizex),h+2*max(1,halfsizey));
-				break;
-			}
-			case Blur::FASTGAUSSIAN:
-			{
-				if(quality < 4)
-				{
-					halfsizex*=2;
-					halfsizey*=2;
-				}
-				workdesc.set_subwindow(-max(1,halfsizex),-max(1,halfsizey),w+2*max(1,halfsizex),h+2*max(1,halfsizey));
-				break;
-			}
-			case Blur::GAUSSIAN:
-			{
-#define GAUSSIAN_ADJUSTMENT		(0.05)
-				Real	pw = (Real)workdesc.get_w()/(workdesc.get_br()[0]-workdesc.get_tl()[0]);
-				Real 	ph = (Real)workdesc.get_h()/(workdesc.get_br()[1]-workdesc.get_tl()[1]);
-				
-				pw=pw*pw;
-				ph=ph*ph;
-				
-				halfsizex = (int)(abs(pw)*feather*GAUSSIAN_ADJUSTMENT+0.5);
-				halfsizey = (int)(abs(ph)*feather*GAUSSIAN_ADJUSTMENT+0.5);
-				
-				halfsizex = (halfsizex + 1)/2;
-				halfsizey = (halfsizey + 1)/2;
-				workdesc.set_subwindow( -halfsizex, -halfsizey, w+2*halfsizex, h+2*halfsizey );
-				break;
-#undef GAUSSIAN_ADJUSTMENT
-			}
-		}
-	}
-	subimage=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, workdesc.get_w(), workdesc.get_h());
-	cairo_t* subcr=cairo_create(subimage);
-
-	cairo_save(subcr);
-	cairo_set_source_rgba(subcr, r, g, b, a);
-	// Now let's check if it is inverted
-	if(invert)
-	{
-		cairo_paint(subcr);
-	}
-	// Draw the outline
-	// Calculate new translations after expand the tile
-	const double extx((-workdesc.get_tl()[0]+origin[0])*sx);
-	const double exty((-workdesc.get_tl()[1]+origin[1])*sy);
-	
-	cairo_translate(subcr, extx , exty);
-	cairo_scale(subcr, sx, sy);
-	switch(winding_style)
-	{
-		case WINDING_NON_ZERO:
-			cairo_set_fill_rule(subcr, CAIRO_FILL_RULE_WINDING);
-			break;
-		default:
-			cairo_set_fill_rule(subcr, CAIRO_FILL_RULE_EVEN_ODD);
-			break;
-	}
-	if(!antialias)
-		cairo_set_antialias(subcr, CAIRO_ANTIALIAS_NONE);
-	if(invert)
-		cairo_set_operator(subcr, CAIRO_OPERATOR_CLEAR);
-	else
-		cairo_set_operator(subcr, CAIRO_OPERATOR_OVER);
-
-	// For any quality...
-	Layer_Shape::shape_to_cairo(subcr);
-	cairo_clip(subcr);
-	cairo_paint(subcr);
-	
-	cairo_restore(subcr);
-
-	if(feather && quality!=10)
-	{
-		etl::surface<float>	shapesurface;
-		shapesurface.set_wh(workdesc.get_w(),workdesc.get_h());
-		shapesurface.clear();
-		
-		CairoSurface cairosubimage(subimage);
-		if(!cairosubimage.map_cairo_image())
-		{
-			synfig::info("map cairo image failed");
-			return false;
-		}
-		// Extract the alpha values:
-		int x, y;
-		int wh(workdesc.get_h()), ww(workdesc.get_w());
-		float div=1.0/((float)(CairoColor::ceil));
-		for(y=0; y<wh; y++)
-			for(x=0;x<ww;x++)
-				shapesurface[y][x]=cairosubimage[y][x].get_a()*div;
-		// Blue the alpha values
-		Blur(feather,feather,blurtype,cb)(shapesurface,workdesc.get_br()-workdesc.get_tl(),shapesurface);
-		// repaint the cairosubimage with the result
-		Color ccolor(color);
-		for(y=0; y<wh; y++)
-			for(x=0;x<ww;x++)
-			{
-				float a=shapesurface[y][x];
-				ccolor.set_a(a);
-				ccolor.clamped();
-				cairosubimage[y][x]=CairoColor(ccolor).premult_alpha();
-			}
-		
-		cairosubimage.unmap_cairo_image();
-	}
-	// Put the (feathered) outline on the surface
-	if(!is_solid_color()) // we need to render the context before
-		if(!context.accelerated_cairorender(surface,quality,renddesc,cb))
-		{
-			if(cb)
-				cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
-			cairo_destroy(cr);
-			cairo_destroy(subcr);
-			cairo_surface_destroy(subimage);
-			return false;
-		}
-	cairo_save(cr);
-	const double px(tl[0]-workdesc.get_tl()[0]);
-	const double py(tl[1]-workdesc.get_tl()[1]);
-	cairo_set_source_surface(cr, subimage, -px*sx, -py*sy);
-	cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
-	cairo_restore(cr);
-	cairo_surface_destroy(subimage);
-	cairo_destroy(subcr);
-	cairo_destroy(cr);
-	
-	return true;
-
-}
-
