@@ -66,17 +66,20 @@ SYNFIG_LAYER_SET_CVS_ID(CheckerBoard,"$Id$");
 
 CheckerBoard::CheckerBoard():
 	Layer_Composite	(1.0,Color::BLEND_COMPOSITE),
-	color			(Color::black()),
-	origin			(Point(0.125,0.125)),
-	size			(Point(0.25,0.25))
+	param_color (ValueBase(Color::black())),
+	param_origin (ValueBase(Point(0.125,0.125))),
+	param_size (ValueBase(Point(0.25,0.25)))
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 inline bool
 CheckerBoard::point_test(const synfig::Point& getpos)const
 {
+	Point origin=param_origin.get(Point());
+	Point size=param_size.get(Point());
+	
 	int val=((int)((getpos[0]-origin[0])/size[0])+(int)((getpos[1]-origin[1])/size[1]));
 	if(getpos[0]-origin[0] < 0.0)
 		val++;
@@ -88,15 +91,36 @@ CheckerBoard::point_test(const synfig::Point& getpos)const
 bool
 CheckerBoard::set_param(const String &param, const ValueBase &value)
 {
-	IMPORT_PLUS(color, { if (color.get_a() == 0) { if (converted_blend_) {
-					set_blend_method(Color::BLEND_ALPHA_OVER);
-					color.set_a(1); } else transparent_color_ = true; } });
-	IMPORT(origin);
-	IMPORT(size);
+	IMPORT_VALUE_PLUS(param_color,
+	  {
+		  Color color(param_color.get(Color()));
+		  if (color.get_a() == 0)
+		  {
+			  if(converted_blend_)
+			  {
+				  set_blend_method(Color::BLEND_ALPHA_OVER);
+				  color.set_a(1);
+				  param_color.set(color);
+			  }
+			  else
+				  transparent_color_ = true;
+		  }
+	  }
+	  );
+	IMPORT_VALUE(param_origin);
+	IMPORT_VALUE(param_size);
 
-	IMPORT_AS(origin,"pos");
-	IMPORT_AS(origin[0],"pos[0]");
-	IMPORT_AS(origin[1],"pos[1]");
+	if(param=="pos")
+		return set_param("origin", value);
+
+	for(int i=0;i<2;i++)
+		if(param==strprintf("pos[%d]",i) && value.get_type()==ValueBase::TYPE_REAL)
+		{
+			Point p=param_origin.get(Point());
+			p[i]=value.get(Real());
+			param_origin.set(p);
+			return true;
+		}
 
 	return Layer_Composite::set_param(param,value);
 }
@@ -104,9 +128,9 @@ CheckerBoard::set_param(const String &param, const ValueBase &value)
 ValueBase
 CheckerBoard::get_param(const String &param)const
 {
-	EXPORT(color);
-	EXPORT(origin);
-	EXPORT(size);
+	EXPORT_VALUE(param_color);
+	EXPORT_VALUE(param_origin);
+	EXPORT_VALUE(param_size);
 	EXPORT_NAME();
 	EXPORT_VERSION();
 
@@ -154,6 +178,8 @@ CheckerBoard::hit_check(synfig::Context context, const synfig::Point &getpos)con
 Color
 CheckerBoard::get_color(Context context, const Point &getpos)const
 {
+	Color color=param_color.get(Color());
+
 	if(get_amount()!=0.0 && point_test(getpos))
 	{
 		if(get_amount()==1.0 && get_blend_method()==Color::BLEND_STRAIGHT)
@@ -168,6 +194,8 @@ CheckerBoard::get_color(Context context, const Point &getpos)const
 bool
 CheckerBoard::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Color color=param_color.get(Color());
+
 	SuperCallback supercb(cb,0,9500,10000);
 
 	if(!context.accelerated_render(surface,quality,renddesc,&supercb))
@@ -202,19 +230,22 @@ CheckerBoard::accelerated_render(Context context,Surface *surface,int quality, c
 }
 
 //////////
-
 bool
-CheckerBoard::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+CheckerBoard::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
-	SuperCallback supercb(cb,0,9500,10000);
+	Color color=param_color.get(Color());
+	Point origin=param_origin.get(Point());
+	Point size=param_size.get(Point());
 
+	SuperCallback supercb(cb,0,9500,10000);
+	
 	if(!is_solid_color())
-		if(!context.accelerated_cairorender(surface,quality,renddesc,&supercb))
+		if(!context.accelerated_cairorender(cr,quality,renddesc,&supercb))
 			return false;
 	
 	if(get_amount()==0)
 		return true;
-
+	
 	const float r(color.get_r());
 	const float g(color.get_g());
 	const float b(color.get_b());
@@ -229,10 +260,8 @@ CheckerBoard::accelerated_cairorender(Context context,cairo_surface_t *surface,i
 	// Width and Height of a pixel
 	const Real pw = (br[0] - tl[0]) / w;
 	const Real ph = (br[1] - tl[1]) / h;
-
+	
 	// These are translation and scale values
-	const double tx(-tl[0]/pw);
-	const double ty(-tl[1]/ph);
 	const double sx(1/pw);
 	const double sy(1/ph);
 	
@@ -244,15 +273,20 @@ CheckerBoard::accelerated_cairorender(Context context,cairo_surface_t *surface,i
 	newsize[0]=((int)(2.0*newsize[0]*sx))/(2.0*sx);
 	newsize[1]=((int)(2.0*newsize[1]*sy))/(2.0*sy);
 	
+	if(!context.accelerated_cairorender(cr,quality,renddesc,cb))
+	{
+		if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
+		return false;
+	}
 	// Now let's render the minimum checkerboard in other surface
 	// Initially I'll fill it completely with the alpha color
 	// Create a similar image with the same dimensions than the minimum checkerboard
-	
 	RendDesc desc(renddesc);
 	// this will modify the w and h values in pixels.
 	desc.set_flags(RendDesc::PX_ASPECT|RendDesc::IM_SPAN);
 	desc.set_tl_br(Point(-newsize[0], +newsize[1]), Point(+newsize[0], -newsize[1]));
-	cairo_surface_t* subimage=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, desc.get_w(), desc.get_h());
+	cairo_surface_t* subimage=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, desc.get_w(), desc.get_h());
+	
 	cairo_t* subcr=cairo_create(subimage);
 	cairo_save(subcr);
 	cairo_set_source_rgba(subcr, r, g, b, a);
@@ -266,12 +300,12 @@ CheckerBoard::accelerated_cairorender(Context context,cairo_surface_t *surface,i
 	cairo_clip(subcr);
 	cairo_paint(subcr);
 	cairo_restore(subcr);
-	
-	cairo_t* cr=cairo_create(surface);
+
 	cairo_save(cr);
-	
-	cairo_translate(cr, tx+origin[0]*sx -newsize[0]*sx, ty+origin[1]*sy+newsize[1]*sy);
+	cairo_translate(cr, origin[0], origin[1]);
+	cairo_scale(cr, 1/sx, 1/sy);
 	cairo_pattern_t* pattern=cairo_pattern_create_for_surface(subimage);
+
 	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
 	cairo_set_source(cr, pattern);
 	if(is_solid_color())
@@ -284,11 +318,9 @@ CheckerBoard::accelerated_cairorender(Context context,cairo_surface_t *surface,i
 		cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
 	}
 	cairo_restore(cr);
-	cairo_surface_destroy(subimage);
-	cairo_pattern_destroy(pattern);
-	cairo_destroy(subcr);
-	cairo_destroy(cr);
 
+	cairo_pattern_destroy(pattern);
+	
 	// Mark our progress as finished
 	if(cb && !cb->amount_complete(10000,10000))
 		return false;
