@@ -45,9 +45,12 @@ class CheckerBoard : public synfig::Layer_Composite, public synfig::Layer_NoDefo
 {
 	SYNFIG_LAYER_MODULE_EXT
 private:
-	synfig::Color color;
-	synfig::Point origin;
-	synfig::Point size;
+	//! Parameter: (Color)
+	synfig::ValueBase param_color;
+	//! Parameter: (Point)
+	synfig::ValueBase param_origin;
+	//! Parameter: (Point)
+	synfig::ValueBase param_size;
 
 	bool point_test(const synfig::Point& x)const;
 
@@ -65,7 +68,7 @@ public:
 	synfig::Layer::Handle hit_check(synfig::Context context, const synfig::Point &point)const;
 
 	virtual bool accelerated_render(synfig::Context context,synfig::Surface *surface,int quality, const synfig::RendDesc &renddesc, synfig::ProgressCallback *cb)const;
-	virtual bool accelerated_cairorender(synfig::Context context,cairo_surface_t *surface,int quality, const synfig::RendDesc &renddesc, synfig::ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(synfig::Context context, cairo_t *cr, int quality, const synfig::RendDesc &renddesc, synfig::ProgressCallback *cb)const;
 };
 
 /* === E N D =============================================================== */
