@@ -61,32 +61,50 @@ SYNFIG_LAYER_SET_CVS_ID(Circle,"$Id$");
 /* -- F U N C T I O N S ----------------------------------------------------- */
 
 Circle::Circle():
-	Layer_Composite	(1.0,Color::BLEND_COMPOSITE),
-	color			(Color::black()),
-	origin			(0,0),
-	radius			(1),
-	feather			(0),
-	invert			(false),
-	falloff			(FALLOFF_INTERPOLATION_LINEAR)
+	Layer_Composite (1.0,Color::BLEND_COMPOSITE),
+	param_color     (Color::black()),
+	param_origin    (Point(0,0)),
+	param_radius    (Real(1)),
+	param_feather   (Real(0)),
+	param_invert    (bool(false)),
+	param_falloff   (FALLOFF_INTERPOLATION_LINEAR)
 {
 	constructcache();
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 bool
 Circle::ImportParameters(const String &param, const ValueBase &value)
 {
-	IMPORT_PLUS(color, { if (color.get_a() == 0) { if (converted_blend_) {
-					set_blend_method(Color::BLEND_ALPHA_OVER);
-					color.set_a(1); } else transparent_color_ = true; } });
-	IMPORT(radius);
-	IMPORT_PLUS(feather, if(feather<0)feather=0;);
-	IMPORT(invert);
-	IMPORT(origin);
-	IMPORT(falloff);
-
-	IMPORT_AS(origin,"pos");
+	IMPORT_VALUE_PLUS(param_color,
+		  {
+			  Color color(param_color.get(Color()));
+			  if (color.get_a() == 0)
+			  {
+				  if(converted_blend_)
+				  {
+					  set_blend_method(Color::BLEND_ALPHA_OVER);
+					  color.set_a(1);
+					  param_color.set(color);
+				  }
+				  else
+					  transparent_color_ = true;
+			  }
+		  }
+	);
+	IMPORT_VALUE(param_radius);
+	IMPORT_VALUE_PLUS(param_feather,
+		if(param_feather.get(Real())<0)
+		{
+			param_feather.set(Real(0));
+		}
+	);
+	IMPORT_VALUE(param_invert);
+	IMPORT_VALUE(param_origin);
+	IMPORT_VALUE(param_falloff);
+	if(param=="pos")
+		return ImportParameters("origin",value);
 
 	return Layer_Composite::set_param(param,value);
 }
@@ -106,12 +124,12 @@ Circle::set_param(const String &param, const ValueBase &value)
 ValueBase
 Circle::get_param(const String &param)const
 {
-	EXPORT(color);
-	EXPORT(radius);
-	EXPORT(feather);
-	EXPORT(invert);
-	EXPORT(origin);
-	EXPORT(falloff);
+	EXPORT_VALUE(param_color);
+	EXPORT_VALUE(param_radius);
+	EXPORT_VALUE(param_feather);
+	EXPORT_VALUE(param_invert);
+	EXPORT_VALUE(param_origin);
+	EXPORT_VALUE(param_falloff);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -165,6 +183,11 @@ Circle::get_param_vocab()const
 synfig::Layer::Handle
 Circle::hit_check(synfig::Context context, const synfig::Point &point)const
 {
+	Real radius = param_radius.get(Real());
+	Real feather = param_feather.get(Real());
+	Point origin = param_origin.get(Point());
+	bool invert = param_invert.get(bool());
+	
 	Point temp=origin-point;
 
 	if(get_amount()==0)
@@ -278,6 +301,8 @@ Circle::InvCosineFalloff(const Circle::CircleDataCache &c, const Real &mag_sqd)
 
 void Circle::constructcache()
 {
+	Real radius = param_radius.get(Real());
+	Real feather = param_feather.get(Real());
 	cache.inner_radius = radius - feather;
 	if(cache.inner_radius < 0)
 		cache.inner_radius = 0;
@@ -295,6 +320,8 @@ void Circle::constructcache()
 
 Circle::FALLOFF_FUNC *Circle::GetFalloffFunc()const
 {
+	bool invert = param_invert.get(bool());
+	Falloff falloff = param_falloff.get(FALLOFF_SQUARED);
 	switch(falloff)
 	{
 	case FALLOFF_SQUARED:	return invert?InvSqdFalloff:SqdFalloff;
@@ -313,6 +340,11 @@ Circle::FALLOFF_FUNC *Circle::GetFalloffFunc()const
 Color
 Circle::get_color(Context context, const Point &point)const
 {
+	Real radius = param_radius.get(Real());
+	Real feather = param_feather.get(Real());
+	Point origin = param_origin.get(Point());
+	bool invert = param_invert.get(bool());
+	Color color(param_color.get(Color()));
 	if(is_disabled() || (radius==0 && invert==false && !feather))
 		return context.get_color(point);
 
@@ -421,6 +453,11 @@ Circle::get_color(Context context, const Point &point)const
 bool
 Circle::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Real radius = param_radius.get(Real());
+	Real feather = param_feather.get(Real());
+	Point origin = param_origin.get(Point());
+	bool invert = param_invert.get(bool());
+	Color color(param_color.get(Color()));
 	// trivial case
 	if(is_disabled() || (radius==0 && invert==false && !feather))
 		return context.accelerated_render(surface,quality, renddesc, cb);
@@ -753,14 +790,19 @@ Circle::accelerated_render(Context context,Surface *surface,int quality, const R
 }
 
 ///////////
-
 bool
-Circle::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Circle::accelerated_cairorender(Context context,cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Real radius = param_radius.get(Real());
+	Real feather = param_feather.get(Real());
+	Point origin = param_origin.get(Point());
+	bool invert = param_invert.get(bool());
+	Color color(param_color.get(Color()));
+
 	// trivial case
 	if(is_disabled() || (radius==0 && invert==false && !feather))
-		return context.accelerated_cairorender(surface,quality, renddesc, cb);
-
+		return context.accelerated_cairorender(cr,quality, renddesc, cb);
+	
 	// Grab the rgba values
 	const float r(color.get_r());
 	const float g(color.get_g());
@@ -770,40 +812,19 @@ Circle::accelerated_cairorender(Context context,cairo_surface_t *surface,int qua
 	// Another trivial case
 	if(invert && radius==0 && is_solid_color())
 	{
-		cairo_t* cr=cairo_create(surface);
 		cairo_set_source_rgba(cr, r, g, b, a); // a=1.0
 		cairo_paint(cr);
 		cairo_restore(cr);
-		cairo_destroy(cr);
 		return true;
 	}
-	
-	// Window Boundaries
-	const Point	tl(renddesc.get_tl());
-	const Point br(renddesc.get_br());
-	const int	w(renddesc.get_w());
-	const int	h(renddesc.get_h());
-		
-	// Width and Height of a pixel
-	const Real pw = (br[0] - tl[0]) / w;
-	const Real ph = (br[1] - tl[1]) / h;
 
-	// These are the scale and translation values
-	const double tx(-tl[0]/pw);
-	const double ty(-tl[1]/ph);
-	const double sx(1/pw);
-	const double sy(1/ph);
-	
-	// True if circle is degenerated (out_radius <0)
-	bool degenerated(false);
-	
 	// Don't render feathering at all when quality is 10
 	// Cairo will take care of the anti-aliased appaerance
 	const Real newfeather = (quality == 10) ? 0 : feather;
-
+	
 	const Real in_radius=radius-newfeather>0?radius-newfeather:0;
 	const Real out_radius=radius+newfeather;
-
+	
 	const Real inner_radius = radius-newfeather>0 ? radius-newfeather : 0;
 	const Real outer_radius = radius+newfeather;
 	
@@ -823,342 +844,47 @@ Circle::accelerated_cairorender(Context context,cairo_surface_t *surface,int qua
 		diff_radii_sqd,double_feather
 	};
 	
-	const Point
-			out_bl(origin-Point(out_radius, out_radius)),
-			out_tr(origin+Point(out_radius, out_radius)),
-			in_bl(origin-Point(1.0, 1.0).norm()*in_radius),
-			in_tr(origin+Point(1.0, 1.0).norm()*in_radius);
-	
-	// if tr and bl are swaped then the circle is degenerated
-	if(out_bl[0] >= out_tr[0] || out_bl[1]>=out_tr[1]) degenerated=true;
-	
-	cairo_t* cr=cairo_create(surface);
-	// This is a rectangle with the same dimensions of the outer circle
-	const Rect shape_out(out_bl, out_tr);
-	const Rect shape_in(in_bl, in_tr);
-	// This is a rectangle with the same dimensions of the canvas
-	const Rect dest(tl, br);
-	Rect inter_out(dest);
-	Rect inter_in(dest);
-	// inter holds the intersection rectangle of canvas and circle.
-	inter_out&=shape_out;
-	inter_in&=shape_in;
-	
-	Point inter_out_min(inter_out.get_min());
-	Point inter_out_max(inter_out.get_max());
-	Point inter_in_min(inter_out.get_min());
-	Point inter_in_max(inter_out.get_max());
-
-	//let the rendering begin
-	SuperCallback supercb(cb,0,9000,10000);
+	if(!context.accelerated_cairorender(cr,quality,renddesc,cb))
+	{
+		if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
+		return false;
+	}
 	if(!newfeather)
 	{
 		if(invert)
 		{
-			if(
-			   is_solid_color()
-			   ||
-			   (get_blend_method() == Color::BLEND_COMPOSITE &&
-			   get_amount() == 1.0f &&
-			   color.get_a() == 1.0f)
-			   )
-			{
-				// Fill the surface with the color intially
-				cairo_save(cr);
-				cairo_set_source_rgba(cr, r, g, b, a); // a=1.0
-				cairo_paint(cr);
-				cairo_restore(cr);
-				
-				// Check for the case where there is nothing else to render
-				if (degenerated)
-				{
-					cairo_destroy(cr);
-					return true;
-				}
-				// Now clear a hole on the surface with out_radius
-				cairo_save(cr);
-				cairo_translate(cr, tx , ty);
-				cairo_scale(cr, sx, sy);
-				cairo_arc(cr, origin[0], origin[1], out_radius, 0., 2*M_PI);
-				cairo_clip(cr);
-				cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
-				cairo_paint(cr);
-				cairo_restore(cr);
-				if(!is_solid_color())  // this means that it is just opaque.
-				{
-					// Since it is opaque it means that we don't
-					// need to render the context beyond the outer radius
-					// So first, modify the Render Description to render on
-					// the intersection only.
-					RendDesc desc(renddesc);
-					//desc.set_flags(0);
-					// this will modify the w and h values in pixels.
-					desc.set_tl(Point(inter_out_min[0], inter_out_max[1]));
-					desc.set_br(Point(inter_out_max[0], inter_out_min[1]));
-					// create a new similar surface with the wxh dimensions
-					cairo_surface_t* subimage=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, desc.get_w(), desc.get_h());
-					// Render what is behind us
-					if(!context.accelerated_cairorender(subimage,quality,desc,cb))
-					{
-						if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
-						{
-							cairo_surface_destroy(subimage);
-							cairo_destroy(cr);
-							return false;
-						}
-					}
-					// Now, place the rendered behind the inverted circle.
-					// remember that the circle has Composite blend method at
-					// this point
-					double width (inter_out_max[0]-inter_out_min[0]);
-					double height(inter_out_max[1]-inter_out_min[1]);
-					cairo_save(cr);
-					cairo_set_source_surface(cr, subimage, (inter_out_min[0]-tl[0])/pw, (inter_out_max[1]-tl[1])/ph);
-					cairo_translate(cr, tx , ty);
-					cairo_scale(cr, sx, sy);
-					cairo_rectangle(cr, inter_out_min[0], inter_out_min[1], width, height);
-					cairo_clip(cr);
-					cairo_set_operator(cr, CAIRO_OPERATOR_DEST_OVER);
-					cairo_paint(cr);
-					cairo_restore(cr);
-				}
-				cairo_destroy(cr);
-				return true;
-			}
-			else // inverted but semitransparent
-			{
-				// Initially render what's behind us
-				if(!context.accelerated_cairorender(surface,quality,renddesc,cb))
-				{
-					if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
-					cairo_destroy(cr);
-					return false;
-				}
-				// Now let's render the inverted circle in other surface
-				// Initially I'll fill it completely with the alpha color
-				// Create a similar image with the same dimensions than the current renddesc
-				cairo_surface_t* subimage=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, renddesc.get_w(), renddesc.get_h());
-				cairo_t* subcr=cairo_create(subimage);
-				cairo_set_source_rgba(subcr, r, g, b, a);
-				cairo_paint(subcr);
-				// now remove the area of the intersection circle
-				cairo_save(subcr);
-				cairo_translate(subcr, tx , ty);
-				cairo_scale(subcr, sx, sy);
-				cairo_arc(subcr, origin[0], origin[1], out_radius, 0., 2*M_PI);
-				cairo_clip(subcr);
-				cairo_set_operator(subcr, CAIRO_OPERATOR_CLEAR);
-				cairo_paint(subcr);
-				cairo_restore(subcr);
-				// now let's paint the inverted circle with the hole on the rendered context
-				cairo_save(cr);
-				cairo_set_source_surface(cr, subimage, 0, 0);
-				cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
-				cairo_restore(cr);
-				cairo_surface_destroy(subimage);
-				cairo_destroy(subcr);
-				cairo_destroy(cr);
-				return true;
-			}
-			
+			cairo_push_group(cr);
+			cairo_set_source_rgba(cr, r, g, b, a);
+			cairo_reset_clip(cr);
+			cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+			cairo_paint(cr);
+			cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+			cairo_arc(cr, origin[0], origin[1], out_radius, 0., 2*M_PI);
+			cairo_clip(cr);
+			cairo_paint(cr);
+			cairo_pop_group_to_source(cr);
+			cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
 		}
-		else // not inverted
+		else
 		{
-		// check if the circle covers completely the canvas
-		// and it is full opaque
-			if(
-			   (
-			   is_solid_color()
-			   ||
-			   (get_blend_method() == Color::BLEND_COMPOSITE &&
-				get_amount() == 1.0f &&
-				color.get_a() == 1.0f)
-			   )
-			   &&
-			   inter_in==dest
-			  )
-				{
-					// Fill the surface with the color
-					cairo_save(cr);
-					cairo_set_source_rgba(cr, r, g, b, a); // a=1.0
-					cairo_paint(cr);
-					cairo_restore(cr);
-					cairo_destroy(cr);
-					// Mark our progress as finished
-					if(cb && !cb->amount_complete(10000,10000))
-						return false;
-					return true;
-				}
-			else // need to render the background first
-			   {
-				   // Initially render what's behind us
-				   if(!context.accelerated_cairorender(surface,quality,renddesc,cb))
-				   {
-					   if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
-					   cairo_destroy(cr);
-					   return false;
-				   }
-				   // Now draw the circle with the out_radius
-				   cairo_save(cr);
-				   cairo_translate(cr, tx , ty);
-				   cairo_scale(cr, sx, sy);
-				   cairo_arc(cr, origin[0], origin[1], out_radius, 0., 2*M_PI);
-				   cairo_clip(cr);
-				   cairo_set_source_rgba(cr, r, g, b, a);
-				   cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
-				   cairo_restore(cr);
-				   cairo_destroy(cr);
-				   return true;
-   			   }
+			cairo_save(cr);
+			cairo_set_source_rgba(cr, r, g, b, a);
+			cairo_arc(cr, origin[0], origin[1], out_radius, 0., 2*M_PI);
+			cairo_clip(cr);
+			cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
+			cairo_restore(cr);
 		}
 	}
-	else // feathered circle
+	else
 	{
-		if(invert)
-		{
-			if(
-			   is_solid_color()
-			   ||
-			   (get_blend_method() == Color::BLEND_COMPOSITE &&
-				get_amount() == 1.0f &&
-				color.get_a() == 1.0f)
-			   )
-			{
-				// Draw the inverted feathered circle
-				cairo_pattern_t* gradient=cairo_pattern_create_radial(origin[0], origin[1], in_radius, origin[0], origin[1], out_radius);
-				compile_gradient(gradient, cache, func);
-				cairo_save(cr);
-				cairo_translate(cr, tx , ty);
-				cairo_scale(cr, sx, sy);
-				cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
-				cairo_set_source(cr, gradient);
-				cairo_paint(cr);
-				cairo_pattern_destroy(gradient); // Not needed more
-				cairo_restore(cr);
-				// Check for the case where there is nothing else to render
-				if (degenerated)
-				{
-					cairo_destroy(cr);
-					return true;
-				}
-				if(!is_solid_color())  // this means that it is just opaque.
-				{
-					// Since it is opaque it means that we don't
-					// need to render the context beyond the outer radius
-					// So first, modify the Render Description to render on
-					// the intersection only.
-					RendDesc desc(renddesc);
-					//desc.set_flags(0);
-					// this will modify the w and h values in pixels.
-					desc.set_tl(Point(inter_out_min[0], inter_out_max[1]));
-					desc.set_br(Point(inter_out_max[0], inter_out_min[1]));
-					// create a new similar surface with the wxh dimensions
-					cairo_surface_t* subimage=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, desc.get_w(), desc.get_h());
-					// Render what is behind us
-					if(!context.accelerated_cairorender(subimage,quality,desc,cb))
-					{
-						if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
-						{
-							cairo_surface_destroy(subimage);
-							cairo_destroy(cr);
-							return false;
-						}
-					}
-					// Now, place the rendered surface behind the inverted circle.
-					// remember that the circle has Composite blend method at
-					// this point
-					double width (inter_out_max[0]-inter_out_min[0]);
-					double height(inter_out_max[1]-inter_out_min[1]);
-					cairo_save(cr);
-					cairo_set_source_surface(cr, subimage, (inter_out_min[0]-tl[0])/pw, (inter_out_max[1]-tl[1])/ph);
-					cairo_translate(cr, tx , ty);
-					cairo_scale(cr, sx, sy);
-					cairo_rectangle(cr, inter_out_min[0], inter_out_min[1], width, height);
-					cairo_clip(cr);
-					cairo_set_operator(cr, CAIRO_OPERATOR_DEST_OVER);
-					cairo_paint(cr);
-					cairo_restore(cr);
-					cairo_surface_destroy(subimage);
-					cairo_destroy(cr);
-					return true;
-				}
-			}
-			else //inverted and semi transparent
-			{
-				// Initially render what's behind us
-				if(!context.accelerated_cairorender(surface,quality,renddesc,cb))
-				{
-					if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
-					cairo_destroy(cr);
-					return false;
-				}
-				// Draw the inverted feathered circle
-				cairo_pattern_t* gradient=cairo_pattern_create_radial(origin[0], origin[1], in_radius, origin[0], origin[1], out_radius);
-				compile_gradient(gradient, cache, func);
-				cairo_save(cr);
-				cairo_translate(cr, tx , ty);
-				cairo_scale(cr, sx, sy);
-				cairo_set_source(cr, gradient);
-				cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
-				cairo_pattern_destroy(gradient); // Not needed more
-				cairo_restore(cr);
-				cairo_destroy(cr);
-				return true;
-			}
-			
-		}
-		else // not inverted
-		{
-			// check if the circle covers completely the canvas
-			// and it is full opaque
-			if(
-			   (
-				is_solid_color()
-				||
-				(get_blend_method() == Color::BLEND_COMPOSITE &&
-				 get_amount() == 1.0f &&
-				 color.get_a() == 1.0f)
-				)
-			   &&
-			   inter_in==dest
-			   )
-			{
-				// Fill the surface with the color
-				cairo_save(cr);
-				cairo_set_source_rgba(cr, r, g, b, a); // a=1.0
-				cairo_paint(cr);
-				cairo_restore(cr);
-				cairo_destroy(cr);
-				// Mark our progress as finished
-				if(cb && !cb->amount_complete(10000,10000))
-					return false;
-				return true;	
-			}
-			else
-			{
-				// Initially render what's behind us
-				if(!context.accelerated_cairorender(surface,quality,renddesc,cb))
-				{
-					if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
-					cairo_destroy(cr);
-					return false;
-				}
-				// Draw the feathered circle
-				cairo_pattern_t* gradient=cairo_pattern_create_radial(origin[0], origin[1], in_radius, origin[0], origin[1], out_radius);
-				compile_gradient(gradient, cache, func);
-				cairo_save(cr);
-				cairo_translate(cr, tx , ty);
-				cairo_scale(cr, sx, sy);
-				cairo_set_source(cr, gradient);
-				cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
-				cairo_pattern_destroy(gradient); // Not needed more
-				cairo_restore(cr);
-				cairo_destroy(cr);
-				return true;
-			}
-		}		
+		cairo_save(cr);
+		cairo_pattern_t* gradient=cairo_pattern_create_radial(origin[0], origin[1], in_radius, origin[0], origin[1], out_radius);
+		compile_gradient(gradient, cache, func);
+		cairo_set_source(cr, gradient);
+		cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
+		cairo_pattern_destroy(gradient);
+		cairo_restore(cr);
 	}
-	cairo_destroy(cr);
 	return true;
 }
 
@@ -1167,6 +893,7 @@ void
 Circle::compile_gradient(cairo_pattern_t* gradient, CircleDataCache mycache, FALLOFF_FUNC *func)const
 {
 	double index;
+	Color color = param_color.get(Color());
 	const float r(color.get_r());
 	const float g(color.get_g());
 	const float b(color.get_b());
@@ -1187,6 +914,10 @@ Circle::compile_gradient(cairo_pattern_t* gradient, CircleDataCache mycache, FAL
 Rect
 Circle::get_bounding_rect()const
 {
+	Real radius = param_radius.get(Real());
+	Real feather = param_feather.get(Real());
+	Point origin = param_origin.get(Point());
+	bool invert = param_invert.get(bool());
 	if(invert)
 		return Rect::full_plane();
 
@@ -1203,6 +934,11 @@ Circle::get_bounding_rect()const
 Rect
 Circle::get_full_bounding_rect(Context context)const
 {
+	Real radius = param_radius.get(Real());
+	Color color = param_color.get(Color());
+	Real feather = param_feather.get(Real());
+	Point origin = param_origin.get(Point());
+	bool invert = param_invert.get(bool());
 	if(invert)
 	{
 		if(is_solid_color() && color.get_a()==0)
