@@ -32,6 +32,7 @@
 #include <synfig/layer_composite.h>
 #include <synfig/color.h>
 #include <synfig/vector.h>
+#include <synfig/value.h>
 
 using namespace synfig;
 using namespace std;
@@ -47,12 +48,18 @@ class Circle : public synfig::Layer_Composite, public synfig::Layer_NoDeform
 {
 	SYNFIG_LAYER_MODULE_EXT
 private:
-	synfig::Color color;
-	synfig::Point origin;
-	synfig::Real radius;
-	synfig::Real feather;
-	bool invert;
-	int falloff;
+	//!Parameter: (Color)
+	ValueBase param_color;
+	//!Parameter: (Point)
+	ValueBase param_origin;
+	//!Parameter: (Real)
+	ValueBase param_radius;
+	//!Parameter: (Real)
+	ValueBase param_feather;
+	//!Parameter: (bool)
+	ValueBase param_invert;
+	//!Parameter: (int)
+	ValueBase param_falloff;
 
 	//Caching system for circle
 	struct CircleDataCache
@@ -108,7 +115,7 @@ public:
 	virtual Color get_color(Context context, const Point &pos)const;
 
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
-	virtual bool accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context,cairo_t *cr,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 
 	virtual synfig::Rect get_full_bounding_rect(synfig::Context context)const;
 	virtual synfig::Rect get_bounding_rect()const;
