@@ -48,28 +48,30 @@ class Outline : public synfig::Layer_Polygon
 {
 	SYNFIG_LAYER_MODULE_EXT
 private:
+	//! Parameter: type list of BLinePoints
+	ValueBase param_bline;
+	//! Parameter: (bool)
+	ValueBase param_round_tip[2];
+	//! Parameter: (bool)
+	ValueBase param_sharp_cusps;
+	//! Parameter: (bool)
+	ValueBase param_loop;
+	//! Parameter: (Real)
+	ValueBase param_width;
+	//! Parameter: (Real)
+	ValueBase param_expand;
+	//! Parameter: (Real)
+	ValueBase param_loopyness;
+	//! Parameter: (bool)
+	ValueBase param_homogeneous_width;
 
-	synfig::ValueBase bline;
-
-	std::vector<synfig::Segment> segment_list;
-	std::vector<synfig::Real> width_list;
-
-	bool round_tip[2];
-
-	bool sharp_cusps;
-
-	bool loop_;
-
-	synfig::Real width;
-
-	synfig::Real expand;
-
-	Real loopyness;
 	bool old_version;
 
 	bool needs_sync;
 
-	bool homogeneous_width;
+
+	std::vector<synfig::Segment> segment_list;
+	std::vector<synfig::Real> width_list;
 
 public:
 
@@ -83,11 +85,10 @@ public:
 	virtual ValueBase get_param(const String & param)const;
 
 	virtual Vocab get_param_vocab()const;
-	virtual void set_time(Context context, Time time)const;
-	virtual void set_time(Context context, Time time, Vector pos)const;
+	virtual void set_time(IndependentContext context, Time time)const;
+	virtual void set_time(IndependentContext context, Time time, Vector pos)const;
 	virtual bool set_version(const String &ver){if(ver=="0.1")old_version=true; return true;}
 	virtual void reset_version(){old_version=false;}
-	virtual bool accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 };
 
 /* === E N D =============================================================== */
