@@ -77,26 +77,39 @@ inline int floor_to_int(const double x) { return static_cast<int>(floor(x)); }
 
 Rectangle::Rectangle():
 	Layer_Composite(1.0,Color::BLEND_COMPOSITE),
-	color(Color::black()),
-	point1(0,0),
-	point2(1,1),
-	expand(0),
-	invert(false)
+	param_color(ValueBase(Color::black())),
+	param_point1(ValueBase(Point(0,0))),
+	param_point2(ValueBase(Point(1,1))),
+	param_expand(ValueBase(Real(0))),
+	param_invert(ValueBase(false))
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 bool
 Rectangle::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT_PLUS(color, { if (color.get_a() == 0) { if (converted_blend_) {
-					set_blend_method(Color::BLEND_ALPHA_OVER);
-					color.set_a(1); } else transparent_color_ = true; } });
-	IMPORT(point1);
-	IMPORT(point2);
-	IMPORT(expand);
-	IMPORT(invert);
+	IMPORT_VALUE_PLUS(param_color,
+					  {
+						  Color color(param_color.get(Color()));
+						  if (color.get_a() == 0)
+						  {
+							  if(converted_blend_)
+							  {
+								  set_blend_method(Color::BLEND_ALPHA_OVER);
+								  color.set_a(1);
+								  param_color.set(color);
+							  }
+							  else
+								  transparent_color_ = true;
+						  }
+					  }
+					  );
+	IMPORT_VALUE(param_point1);
+	IMPORT_VALUE(param_point2);
+	IMPORT_VALUE(param_expand);
+	IMPORT_VALUE(param_invert);
 
 	return Layer_Composite::set_param(param,value);
 }
@@ -104,11 +117,11 @@ Rectangle::set_param(const String & param, const ValueBase &value)
 ValueBase
 Rectangle::get_param(const String &param)const
 {
-	EXPORT(color);
-	EXPORT(point1);
-	EXPORT(point2);
-	EXPORT(expand);
-	EXPORT(invert);
+	EXPORT_VALUE(param_color);
+	EXPORT_VALUE(param_point1);
+	EXPORT_VALUE(param_point2);
+	EXPORT_VALUE(param_expand);
+	EXPORT_VALUE(param_invert);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -152,6 +165,11 @@ Rectangle::get_param_vocab()const
 synfig::Layer::Handle
 Rectangle::hit_check(synfig::Context context, const synfig::Point &pos)const
 {
+	Point point1=param_point1.get(Point());
+	Point point2=param_point2.get(Point());
+	Real expand=param_expand.get(Real());
+	bool invert=param_invert.get(bool());
+	
 	if(is_disabled())
 		return context.hit_check(pos);
 
@@ -189,6 +207,8 @@ Rectangle::hit_check(synfig::Context context, const synfig::Point &pos)const
 bool
 Rectangle::is_solid_color()const
 {
+	Color color=param_color.get(Color());
+	
 	return Layer_Composite::is_solid_color() ||
 		(get_blend_method() == Color::BLEND_COMPOSITE &&
 		 get_amount() == 1.0f &&
@@ -198,6 +218,12 @@ Rectangle::is_solid_color()const
 Color
 Rectangle::get_color(Context context, const Point &pos)const
 {
+	Color color=param_color.get(Color());
+	Point point1=param_point1.get(Point());
+	Point point2=param_point2.get(Point());
+	Real expand=param_expand.get(Real());
+	bool invert=param_invert.get(bool());
+
 	if(is_disabled())
 		return context.get_color(pos);
 
@@ -305,6 +331,12 @@ Rectangle::get_color(Context context, const Point &pos)const
 bool
 Rectangle::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Color color=param_color.get(Color());
+	Point point1=param_point1.get(Point());
+	Point point2=param_point2.get(Point());
+	Real expand=param_expand.get(Real());
+	bool invert=param_invert.get(bool());
+
 	if(is_disabled())
 		return context.accelerated_render(surface,quality,renddesc,cb);
 
@@ -559,35 +591,31 @@ Rectangle::accelerated_render(Context context,Surface *surface,int quality, cons
 	return true;
 }
 
-//////
-
+///////
 bool
-Rectangle::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Rectangle::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Color color=param_color.get(Color());
+	Point point1=param_point1.get(Point());
+	Point point2=param_point2.get(Point());
+	Real expand=param_expand.get(Real());
+	bool invert=param_invert.get(bool());
+
 	if(is_disabled())
-		return context.accelerated_cairorender(surface,quality,renddesc,cb);
+		return context.accelerated_cairorender(cr,quality,renddesc,cb);
 	
 	const float r(color.get_r());
 	const float g(color.get_g());
 	const float b(color.get_b());
 	const float a(color.get_a());
 	
-	const Point	tl(renddesc.get_tl());
-	const Point br(renddesc.get_br());
-	
-	const int	w(renddesc.get_w());
-	const int	h(renddesc.get_h());
-
-	// Width and Height of a pixel
-	const Real pw = (br[0] - tl[0]) / w;
-	const Real ph = (br[1] - tl[1]) / h;
 	Point min(point1), max(point2);
-
+	
 	// if x max, min are swaped then swap the x coordinate
 	if(min[0] > max[0]) swap(min[0],max[0]);
 	// if y max min are swaped then swap the y coordinate
 	if(min[1] > max[1]) swap(min[1],max[1]);
-	// min is the lower left corner and max is the upper right corner 
+	// min is the lower left corner and max is the upper right corner
 	// now we need to expand the edges
 	min[0]-=expand;
 	max[0]+=expand;
@@ -596,161 +624,55 @@ Rectangle::accelerated_cairorender(Context context,cairo_surface_t *surface,int
 	//
 	//synfig::info("min=%f, %f     max=%f, %f", min[0], min[1], max[0], max[1]);
 	//
-	cairo_t* cr=cairo_create(surface);
 	// This is a rectangle with the same dimensions of the rectangle
 	const Rect shape(min, max);
-	// This is a rectangle with the same dimensions of the canvas
-	const Rect dest(tl, br);
-	Rect inter(dest);
-	// inter holds the intersection rectangle of both rectangles.
-	inter&=shape;  	
-
-	Point inter_min(inter.get_min());
-	Point inter_max(inter.get_max());
-	if(invert)
-	{
-		if(is_solid_color())
-		{
-			// The rectangle is inverted and is solid color
-			// Let's do this:
-			// Fill the surface with the color intially
-			cairo_save(cr);
-			cairo_set_source_rgba(cr, r, g, b, a); // a=1.0
-			cairo_paint(cr);
-			cairo_restore(cr);
-			
-			// Check for the case where there is nothing to render
-			if (inter.area()<0.00000001)
-			{
-				cairo_destroy(cr);
-				return true;
-			}
-			// Modify the Render Description to render on the intersection.
-			// this will modify the w and h values in pixels.
-			RendDesc desc(renddesc);
-			desc.set_flags(0);
-			desc.set_tl(Point(inter.get_min()[0], inter.get_max()[1]));
-			desc.set_br(Point(inter.get_max()[0], inter.get_min()[1]));
-			// create a new similar surface with the wxh dimensions
-			cairo_surface_t* subimage=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, desc.get_w(), desc.get_h());
-			// Render what is behind us
-			if(!context.accelerated_cairorender(subimage,quality,desc,cb))
-			{
-				if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
-				{
-					cairo_surface_destroy(subimage);
-					cairo_destroy(cr);
-					return false;
-				}
-			}
-			cairo_save(cr);
-			double width (inter_max[0]-inter_min[0]);
-			double height(inter_max[1]-inter_min[1]);
-			double tx(-tl[0]/pw);
-			double ty(-tl[1]/ph);
-			double sx(1/pw);
-			double sy(1/ph);  
-			cairo_set_source_surface(cr, subimage, (inter_min[0]-tl[0])/pw, (inter_max[1]-tl[1])/ph);
-			cairo_translate(cr, tx , ty);
-			cairo_scale(cr, sx, sy);
-			cairo_rectangle(cr, inter_min[0], inter_min[1], width, height);
-			cairo_clip(cr);
-			cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
-			cairo_paint(cr);
-			cairo_restore(cr);
-			cairo_surface_destroy(subimage);
-			cairo_destroy(cr);
-			return true;
-		}
-		else // not solid color so we need to render what's behind
-		{
-			// Initially render what's behind us
-			if(!context.accelerated_cairorender(surface,quality,renddesc,cb))
-			{
-				if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
-				cairo_destroy(cr);
-				return false;
-			}
-			// Now let's render the rectangle in other surface
-			// Initially I'll fill it completely with the alpha color 
-			// Create a similar image with the same dimensions than the current renddesc
-			cairo_surface_t* subimage=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, renddesc.get_w(), renddesc.get_h());
-			cairo_t* subcr=cairo_create(subimage);
-			cairo_set_source_rgba(subcr, r, g, b, a);
-			cairo_paint(subcr);
-			// now remove the area of the intersection rectangle
-			cairo_save(subcr);
-			double width (inter_max[0]-inter_min[0]);
-			double height(inter_max[1]-inter_min[1]);
-			double tx(-tl[0]/pw);
-			double ty(-tl[1]/ph);
-			double sx(1/pw);
-			double sy(1/ph);  
-			cairo_translate(subcr, tx , ty);
-			cairo_scale(subcr, sx, sy);
-			cairo_rectangle(subcr, inter_min[0], inter_min[1], width, height);
-			cairo_set_operator(subcr, CAIRO_OPERATOR_CLEAR);
-			cairo_fill(subcr);
-			cairo_restore(subcr);
-			// now let's paint the inverted rectangle with the hole on the rendered context 
-			cairo_save(cr);
-			cairo_set_source_surface(cr, subimage, 0, 0);
-			cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
-			cairo_restore(cr);
-			cairo_surface_destroy(subimage);
-			cairo_destroy(subcr);
-			cairo_destroy(cr);
-			return true;			
-		}
-	}
 	
-	// not inverted
-	// optimization - if the whole canvas is covered by this rectangle,
-	// and the rectangle is a solid color, we don't need to render
-	// what's behind us
-	if (is_solid_color() && inter==Rect(tl, br))
-	{
-		cairo_set_source_rgba(cr, r, g, b, a);
-		cairo_paint(cr);
-		cairo_destroy(cr);
-		return true;
-	}
-	// The rectangle intersects the canvas partially
-	// Render what is behind us
-	if(!context.accelerated_cairorender(surface,quality,renddesc,cb))
+	const Point shape_min(shape.get_min());
+	const Point shape_max(shape.get_max());
+	const double width (shape_max[0]-shape_min[0]);
+	const double height(shape_max[1]-shape_min[1]);
+
+	if(!context.accelerated_cairorender(cr,quality,renddesc,cb))
 	{
 		if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
-		cairo_destroy(cr);
 		return false;
 	}
-	
-	// In the case where there is nothing to render...
-	if (inter.area()<0.00000001)
-		return true;
+
 	cairo_save(cr);
 	cairo_set_source_rgba(cr, r, g, b, a);
-	double width (inter_max[0]-inter_min[0]);
-	double height(inter_max[1]-inter_min[1]);
-	double tx(-tl[0]/pw);
-	double ty(-tl[1]/ph);
-	double sx(1/pw);
-	double sy(1/ph);  
-	cairo_translate(cr, tx , ty);
-	cairo_scale(cr, sx, sy);
-	cairo_rectangle(cr, inter_min[0], inter_min[1], width, height);
-	cairo_clip(cr);
+	if(invert)
+	{
+		cairo_push_group(cr);
+		cairo_reset_clip(cr);
+		cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+		cairo_paint(cr);
+		cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+	}
+	cairo_rectangle(cr, shape_min[0], shape_min[1], width, height);
+	if(invert)
+	{
+		cairo_fill(cr);
+		cairo_pop_group_to_source(cr);
+	}
+	else
+		cairo_clip(cr);
+	
 	cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
 	cairo_restore(cr);
-	cairo_destroy(cr);
-	return true;
+
+	return  true;
 }
 
 
-//////
 
 Rect
 Rectangle::get_bounding_rect()const
 {
+	Point point1=param_point1.get(Point());
+	Point point2=param_point2.get(Point());
+	Real expand=param_expand.get(Real());
+	bool invert=param_invert.get(bool());
+
 	if(invert)
 		return Rect::full_plane();
 
@@ -787,6 +709,12 @@ Rectangle::get_bounding_rect()const
 Rect
 Rectangle::get_full_bounding_rect(Context context)const
 {
+	Color color=param_color.get(Color());
+	Point point1=param_point1.get(Point());
+	Point point2=param_point2.get(Point());
+	Real expand=param_expand.get(Real());
+	bool invert=param_invert.get(bool());
+
 	if(invert)
 	{
 		if(is_solid_color() && color.get_a()==0)
