@@ -67,32 +67,32 @@ SYNFIG_LAYER_SET_CVS_ID(ConicalGradient,"$Id$");
 
 ConicalGradient::ConicalGradient():
 	Layer_Composite(1.0,Color::BLEND_COMPOSITE),
-	gradient(Color::black(),Color::white()),
-	center(0,0),
-	angle(Angle::zero()),
-	symmetric(false)
+	param_gradient(ValueBase(Gradient(Color::black(),Color::white()))),
+	param_center(ValueBase(Point(0,0))),
+	param_angle(ValueBase(Angle::zero())),
+	param_symmetric(ValueBase(false))
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 bool
 ConicalGradient::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT(gradient);
-	IMPORT(center);
-	IMPORT(angle);
-	IMPORT(symmetric);
+	IMPORT_VALUE(param_gradient);
+	IMPORT_VALUE(param_center);
+	IMPORT_VALUE(param_angle);
+	IMPORT_VALUE(param_symmetric);
 	return Layer_Composite::set_param(param,value);
 }
 
 ValueBase
 ConicalGradient::get_param(const String &param)const
 {
-	EXPORT(gradient);
-	EXPORT(center);
-	EXPORT(angle);
-	EXPORT(symmetric);
+	EXPORT_VALUE(param_gradient);
+	EXPORT_VALUE(param_center);
+	EXPORT_VALUE(param_angle);
+	EXPORT_VALUE(param_symmetric);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -132,6 +132,11 @@ ConicalGradient::get_param_vocab()const
 inline Color
 ConicalGradient::color_func(const Point &pos, float supersample)const
 {
+	Gradient gradient=param_gradient.get(Gradient());
+	Point center=param_center.get(Point());
+	Angle angle=param_angle.get(Angle());
+	bool symmetric=param_symmetric.get(bool());
+	
 	const Point centered(pos-center);
 	Angle::rot a=Angle::tan(-centered[1],centered[0]).mod();
 	a+=angle;
@@ -196,6 +201,8 @@ ConicalGradient::color_func(const Point &pos, float supersample)const
 float
 ConicalGradient::calc_supersample(const synfig::Point &x, float pw,float ph)const
 {
+	Point center=param_center.get(Point());
+
 	Point adj(x-center);
 	if(abs(adj[0])<abs(pw*0.5) && abs(adj[1])<abs(ph*0.5))
 		return 0.5;
@@ -292,29 +299,17 @@ ConicalGradient::accelerated_render(Context context,Surface *surface,int quality
 
 /////////
 bool
-ConicalGradient::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+ConicalGradient::accelerated_cairorender(Context context,cairo_t *cr,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
-	if(!is_solid_color() && !gradient.size())
-		return context.accelerated_cairorender(surface,quality,renddesc,cb);
+	Gradient gradient=param_gradient.get(Gradient());
+	Point center=param_center.get(Point());
+
+	cairo_save(cr);
 	const Point	tl(renddesc.get_tl());
 	const Point br(renddesc.get_br());
 	const Point tr(Point(tl[1], br[0]));
 	const Point bl(Point(tl[0], br[1]));
-	
-	const int	w(renddesc.get_w());
-	const int	h(renddesc.get_h());
-	
-	// Width and Height of a pixel
-	const Real pw = (br[0] - tl[0]) / w;
-	const Real ph = (br[1] - tl[1]) / h;
-	
-	const double tx((center[0]-tl[0])/pw);
-	const double ty((center[1]-tl[1])/ph);
-	const double sx(1/pw);
-	const double sy(1/ph);
-	
-	cairo_t* cr=cairo_create(surface);
-	cairo_save(cr);
+		
 	cairo_pattern_t* pattern=cairo_pattern_create_mesh();
 	// Calculate the outer radius of the mesh pattern. It has to
 	// cover the whole render desc
@@ -324,7 +319,7 @@ ConicalGradient::accelerated_cairorender(Context context,cairo_surface_t *surfac
 	Real c4=(tr-center).mag_squared();
 	Real radius(max(max(max(c1,c2),c3),c4));
 	radius=sqrt(radius)*1.20;
-	//synfig::info("radius=%f", radius);
+
 	bool cpoints_all_opaque=compile_mesh(pattern, gradient, radius);
 	if(quality>8) cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
 	else if(quality>=4) cairo_set_antialias(cr, CAIRO_ANTIALIAS_GOOD);
@@ -332,25 +327,22 @@ ConicalGradient::accelerated_cairorender(Context context,cairo_surface_t *surfac
 	if(
 	   !
 	   (is_solid_color() ||
-		cpoints_all_opaque && get_blend_method()==Color::BLEND_COMPOSITE && get_amount()==1.0)
+		(cpoints_all_opaque && get_blend_method()==Color::BLEND_COMPOSITE && get_amount()==1.f))
 	   )
 	{
 		// Initially render what's behind us
-		if(!context.accelerated_cairorender(surface,quality,renddesc,cb))
+		if(!context.accelerated_cairorender(cr,quality,renddesc,cb))
 		{
 			if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
-			cairo_destroy(cr);
 			return false;
 		}
 	}
-	cairo_translate(cr, tx , ty);
-	cairo_scale(cr, sx, sy);
+	cairo_translate(cr, center[0], center[1]);
 	cairo_set_source(cr, pattern);
 	cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
 	
 	cairo_pattern_destroy(pattern); // Not needed more
 	cairo_restore(cr);
-	cairo_destroy(cr);
 	return true;
 	
 }
@@ -359,6 +351,9 @@ ConicalGradient::accelerated_cairorender(Context context,cairo_surface_t *surfac
 bool
 ConicalGradient::compile_mesh(cairo_pattern_t* pattern, Gradient mygradient, Real radius)const
 {
+	Angle angle=param_angle.get(Angle());
+	bool symmetric=param_symmetric.get(bool());
+
 	bool cpoints_all_opaque=true;
 	float a1, r1, g1, b1, a2, r2, g2, b2;
 	Gradient::CPoint cp;
