@@ -50,24 +50,31 @@ class CurveGradient : public Layer_Composite, public Layer_NoDeform
 	SYNFIG_LAYER_MODULE_EXT
 
 private:
-	std::vector<synfig::BLinePoint> bline;
+	//! Parameter: (Point)
+	ValueBase param_origin;
+	//! Parameter: (Real)
+	ValueBase param_width;
+	//! Parameter: (std::vector<synfig::BLinePoint>)
+	ValueBase param_bline;
+	//! Parameter: (Gradient)
+	ValueBase param_gradient;
+	//! Parameter: (bool)
+	ValueBase param_loop;
+	//! Parameter: (bool)
+	ValueBase param_zigzag;
+	//! Parameter: (bool)
+	ValueBase param_perpendicular;
+	//! Parameter: (bool)
+	ValueBase param_fast;
 
-	Point origin;
-	Real width;
-	Gradient gradient;
 	Real curve_length_;
-
-	bool loop;
-	bool zigzag;
 	bool bline_loop;
-	bool perpendicular;
-	bool fast;
 
 	void sync();
 
-	synfig::Color color_func(const synfig::Point &x, int quality=10, float supersample=0)const;
+	Color color_func(const Point &x, int quality=10, float supersample=0)const;
 
-	float calc_supersample(const synfig::Point &x, float pw,float ph)const;
+	float calc_supersample(const Point &x, float pw,float ph)const;
 
 public:
 	CurveGradient();
@@ -76,7 +83,8 @@ public:
 	virtual ValueBase get_param(const String &param)const;
 	virtual Color get_color(Context context, const Point &pos)const;
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
-	synfig::Layer::Handle hit_check(synfig::Context context, const synfig::Point &point)const;
+	virtual bool accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	Layer::Handle hit_check(synfig::Context context, const synfig::Point &point)const;
 
 	virtual Vocab get_param_vocab()const;
 };
