@@ -49,13 +49,15 @@ class LinearGradient : public Layer_Composite, public Layer_NoDeform
 	SYNFIG_LAYER_MODULE_EXT
 
 private:
-
-	Point p1,p2;
+	//! Parameter: (Point)
+	ValueBase param_p1,param_p2;
 	Vector diff;
-	Gradient gradient;
-
-	bool loop;
-	bool zigzag;
+	//! Parameter: (Gradient)
+	ValueBase param_gradient;
+	//! Parameter: (bool)
+	ValueBase param_loop;
+	//! Parameter: (bool)
+	ValueBase param_zigzag;
 
 	void sync();
 
@@ -71,7 +73,7 @@ public:
 	virtual ValueBase get_param(const String &param)const;
 	virtual Color get_color(Context context, const Point &pos)const;
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
-	virtual bool accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 
 	synfig::Layer::Handle hit_check(synfig::Context context, const synfig::Point &point)const;
 
