@@ -38,6 +38,7 @@
 #include <synfig/surface.h>
 #include <synfig/value.h>
 #include <synfig/valuenode.h>
+#include <synfig/cairo_renddesc.h>
 
 #include "spiralgradient.h"
 
@@ -66,24 +67,24 @@ SYNFIG_LAYER_SET_CVS_ID(SpiralGradient,"$Id$");
 
 SpiralGradient::SpiralGradient():
 	Layer_Composite(1.0,Color::BLEND_COMPOSITE),
-	gradient(Color::black(),Color::white()),
-	center(0,0),
-	radius(0.5),
-	angle(Angle::zero()),
-	clockwise(false)
+	param_gradient(ValueBase(Gradient(Color::black(),Color::white()))),
+	param_center(ValueBase(Point(0,0))),
+	param_radius(ValueBase(Real(0.5))),
+	param_angle(ValueBase(Angle::zero())),
+	param_clockwise(ValueBase(false))
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 bool
 SpiralGradient::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT(gradient);
-	IMPORT(center);
-	IMPORT(radius);
-	IMPORT(angle);
-	IMPORT(clockwise);
+	IMPORT_VALUE(param_gradient);
+	IMPORT_VALUE(param_center);
+	IMPORT_VALUE(param_radius);
+	IMPORT_VALUE(param_angle);
+	IMPORT_VALUE(param_clockwise);
 
 	return Layer_Composite::set_param(param,value);
 }
@@ -91,11 +92,11 @@ SpiralGradient::set_param(const String & param, const ValueBase &value)
 ValueBase
 SpiralGradient::get_param(const String &param)const
 {
-	EXPORT(gradient);
-	EXPORT(center);
-	EXPORT(radius);
-	EXPORT(angle);
-	EXPORT(clockwise);
+	EXPORT_VALUE(param_gradient);
+	EXPORT_VALUE(param_center);
+	EXPORT_VALUE(param_radius);
+	EXPORT_VALUE(param_angle);
+	EXPORT_VALUE(param_clockwise);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -142,6 +143,13 @@ SpiralGradient::get_param_vocab()const
 inline Color
 SpiralGradient::color_func(const Point &pos, float supersample)const
 {
+
+	Gradient gradient=param_gradient.get(Gradient());
+	Point center=param_center.get(Point());
+	Real radius=param_radius.get(Real());
+	Angle angle=param_angle.get(Angle());
+	bool clockwise=param_clockwise.get(bool());
+	
 	const Point centered(pos-center);
 	Angle a;
 	a=Angle::tan(-centered[1],centered[0]).mod();
@@ -179,6 +187,9 @@ SpiralGradient::color_func(const Point &pos, float supersample)const
 float
 SpiralGradient::calc_supersample(const synfig::Point &x, float pw,float /*ph*/)const
 {
+	Point center=param_center.get(Point());
+	Real radius=param_radius.get(Real());
+
 	return (1.41421*pw/radius+(1.41421*pw/Point(x-center).mag())/(PI*2))*0.5;
 }
 
@@ -251,3 +262,72 @@ SpiralGradient::accelerated_render(Context context,Surface *surface,int quality,
 
 	return true;
 }
+
+////
+bool
+SpiralGradient::accelerated_cairorender(Context context, cairo_t *cr,int quality, const RendDesc &renddesc_, ProgressCallback *cb)const
+{
+	RendDesc	renddesc(renddesc_);
+	
+	// Untransform the render desc
+	if(!cairo_renddesc_untransform(cr, renddesc))
+		return false;
+
+	Point pos;
+	const Real pw(renddesc.get_pw()),ph(renddesc.get_ph());
+	const Point tl(renddesc.get_tl());
+	const int w(renddesc.get_w());
+	const int h(renddesc.get_h());
+	
+	SuperCallback supercb(cb,0,9500,10000);
+	
+	if(get_amount()==1.0 && get_blend_method()==Color::BLEND_STRAIGHT)
+	{
+		cairo_save(cr);
+		cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+		cairo_paint(cr);
+		cairo_restore(cr);
+	}
+	else
+	{
+		if(!context.accelerated_cairorender(cr,quality,renddesc,&supercb))
+			return false;
+		if(get_amount()==0)
+			return true;
+	}
+	
+	
+	int x,y;
+	cairo_surface_t *surface;
+	
+	surface=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, w, h);
+	
+	CairoSurface csurface(surface);
+	if(!csurface.map_cairo_image())
+	{
+		synfig::warning("Spiral Gradient: map cairo surface failed");
+		return false;
+	}
+	for(y=0,pos[1]=tl[1];y<h;y++,pos[1]+=ph)
+		for(x=0,pos[0]=tl[0];x<w;x++,pos[0]+=pw)
+			csurface[y][x]=CairoColor(color_func(pos,calc_supersample(pos,pw,ph))).premult_alpha();
+	csurface.unmap_cairo_image();
+	
+	// paint surface on cr
+	cairo_save(cr);
+	cairo_translate(cr, tl[0], tl[1]);
+	cairo_scale(cr, pw, ph);
+	cairo_set_source_surface(cr, surface, 0, 0);
+	cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
+	cairo_restore(cr);
+	
+	cairo_surface_destroy(surface);	
+	// Mark our progress as finished
+	if(cb && !cb->amount_complete(10000,10000))
+		return false;
+	
+	return true;
+}
+
+
+
