@@ -48,23 +48,32 @@ class Noise : public synfig::Layer_Composite, public synfig::Layer_NoDeform
 	SYNFIG_LAYER_MODULE_EXT
 
 private:
+	//!Parameter: (Gradient)
+	synfig::ValueBase param_gradient;
+	//!Parameter: (RandomNoise)
+	synfig::ValueBase param_random;
+	//!Parameter: (synfig::Vector)
+	synfig::ValueBase param_size;
+	//!Parameter: (RandomNoise::SmoothType)
+	synfig::ValueBase param_smooth;
+	//!Parameter: (int)
+	synfig::ValueBase param_detail;
+	//!Parameter: (synfig::Real)
+	synfig::ValueBase param_speed;
+	//!Parameter: (bool)
+	synfig::ValueBase param_turbulent;
+	//!Parameter: (bool)
+	synfig::ValueBase param_do_alpha;
+	//!Parameter: (bool)
+	synfig::ValueBase param_super_sample;
 
-	synfig::Vector size;
 
-	RandomNoise random;
-	RandomNoise::SmoothType smooth;
-	int detail;
-	bool do_alpha;
-	synfig::Gradient gradient;
-	synfig::Real speed;
-	bool turbulent;
-	bool do_displacement;
-	synfig::Vector displacement;
+//	bool do_displacement;
+//	synfig::Vector displacement;
 
 	//void sync();
 	mutable synfig::Time curr_time;
 
-	bool super_sample;
 
 	synfig::Color color_func(const synfig::Point &x, float supersample,synfig::Context context)const;
 
@@ -79,8 +88,8 @@ public:
 	virtual synfig::CairoColor get_cairocolor(synfig::Context context, const synfig::Point &pos)const;
 	virtual bool accelerated_render(synfig::Context context,synfig::Surface *surface,int quality, const synfig::RendDesc &renddesc, synfig::ProgressCallback *cb)const;
 	synfig::Layer::Handle hit_check(synfig::Context context, const synfig::Point &point)const;
-	virtual void set_time(synfig::Context context, synfig::Time time)const;
-	virtual void set_time(synfig::Context context, synfig::Time time, const synfig::Point &point)const;
+	virtual void set_time(synfig::IndependentContext context, synfig::Time time)const;
+	virtual void set_time(synfig::IndependentContext context, synfig::Time time, const synfig::Point &point)const;
 
 	virtual Vocab get_param_vocab()const;
 };
