@@ -46,6 +46,7 @@
 #include <ETL/bezier>
 #include <ETL/hermite>
 #include <vector>
+#include <time.h>
 
 #include <synfig/valuenode_bline.h>
 
@@ -77,20 +78,30 @@ SYNFIG_LAYER_SET_CVS_ID(Plant,"$Id$");
 
 
 Plant::Plant():
-	origin(0,0),
-	split_angle(Angle::deg(10)),
-	gravity(0,-0.1),
-	velocity(0.3),
-	perp_velocity(0.0),
-	step(0.01),
-	sprouts(10),
-	version(version__),
-	use_width(true)
+	param_bline(ValueBase(std::vector<BLinePoint>())),
+	param_origin(ValueBase(Vector(0,0))),
+	param_gradient(ValueBase(Gradient(Color::black(), Color::white()))),
+	param_split_angle(ValueBase(Angle::deg(10))),
+	param_gravity(ValueBase(Vector(0,-0.1))),
+	param_velocity(ValueBase(Real(0.3))),
+	param_perp_velocity(ValueBase(Real(0.0))),
+	param_size(ValueBase(Real(0.015))),
+	param_size_as_alpha(ValueBase(false)),
+	param_reverse(ValueBase(true)),
+	param_step(ValueBase(Real(0.01))),
+	param_splits(ValueBase(int(5))),
+	param_sprouts(ValueBase(int(10))),
+	param_random_factor(ValueBase(Real(0.2))),
+	param_drag(ValueBase(Real(0.1))),
+	param_use_width(ValueBase(true)),
+	version(version__)
 {
 	bounding_rect=Rect::zero();
-	random_factor=0.2;
+	Random random;
 	random.set_seed(time(NULL));
-
+	param_random.set(random.get_seed());
+	
+	std::vector<BLinePoint> bline;
 	bline.push_back(BLinePoint());
 	bline.push_back(BLinePoint());
 	bline.push_back(BLinePoint());
@@ -103,22 +114,30 @@ Plant::Plant():
 	bline[0].set_width(1.0f);
 	bline[1].set_width(1.0f);
 	bline[2].set_width(1.0f);
+	param_bline.set(bline);
+	
 	bline_loop=true;
 	mass=(0.5);
-	splits=5;
-	drag=0.1;
-	size=0.015;
 	needs_sync_=true;
 	sync();
-	size_as_alpha=false;
-	reverse=true;
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 void
 Plant::branch(int n,int depth,float t, float stunt_growth, synfig::Point position,synfig::Vector vel)const
 {
+	int splits=param_splits.get(int());
+	Real step=param_step.get(Real());
+	Vector gravity=param_gravity.get(Vector());
+	Real drag=param_drag.get(Real());
+	Gradient gradient=param_gradient.get(Gradient());
+	Angle split_angle=param_split_angle.get(Angle());
+	Real random_factor=param_random_factor.get(Real());
+	Random random;
+	random.set_seed(param_random.get(int()));
+	
 	float next_split((1.0-t)/(splits-depth)+t/*+random_factor*random(40+depth,t*splits,0,0)/splits*/);
 	for(;t<next_split;t+=step)
 	{
@@ -152,6 +171,11 @@ Plant::branch(int n,int depth,float t, float stunt_growth, synfig::Point positio
 void
 Plant::calc_bounding_rect()const
 {
+	std::vector<BLinePoint> bline(param_bline.get_list().begin(), param_bline.get_list().end());
+	Real velocity=param_velocity.get(Real());
+	Vector gravity=param_gravity.get(Vector());
+	Real size=param_size.get(Real());
+
 	std::vector<synfig::BLinePoint>::const_iterator iter,next;
 
 	bounding_rect=Rect::zero();
@@ -184,6 +208,18 @@ Plant::calc_bounding_rect()const
 void
 Plant::sync()const
 {
+	std::vector<BLinePoint> bline(param_bline.get_list().begin(), param_bline.get_list().end());
+	Real step_=param_step.get(Real());
+	Gradient gradient=param_gradient.get(Gradient());
+	Real random_factor=param_random_factor.get(Real());
+	Random random;
+	random.set_seed(param_random.get(int()));
+	int sprouts=param_sprouts.get(int());
+	Real velocity=param_velocity.get(Real());
+	Real perp_velocity=param_perp_velocity.get(Real());
+	int splits=param_splits.get(int());
+	bool use_width=param_use_width.get(bool());
+	
 	Mutex::Lock lock(mutex);
 	if (!needs_sync_) return;
 	time_t start_time; time(&start_time);
@@ -202,7 +238,7 @@ Plant::sync()const
 
 	etl::hermite<Vector> curve;
 
-	Real step(abs(this->step));
+	Real step(abs(step_));
 
 	int seg(0);
 
@@ -276,56 +312,55 @@ Plant::sync()const
 bool
 Plant::set_param(const String & param, const ValueBase &value)
 {
-	if(param=="bline" && value.get_type()==ValueBase::TYPE_LIST)
+	IMPORT_VALUE_PLUS(param_bline,
 	{
-		bline=value;
 		bline_loop=value.get_loop();
 		needs_sync_=true;
-		set_param_static(param, value.get_static());
 		return true;
-	}
-	if(param=="seed" && value.same_type_as(int()))
-	{
-		random.set_seed(value.get(int()));
-		needs_sync_=true;
-		set_param_static(param, value.get_static());
-		return true;
-	}
-	IMPORT(origin);
-	IMPORT_PLUS(split_angle,needs_sync_=true);
-	IMPORT_PLUS(gravity,needs_sync_=true);
-	IMPORT_PLUS(gradient,needs_sync_=true);
-	IMPORT_PLUS(velocity,needs_sync_=true);
-	IMPORT_PLUS(perp_velocity,needs_sync_=true);
-	IMPORT_PLUS(step,{
+	});
+	IMPORT_VALUE_PLUS(param_random, needs_sync_=true);
+	IMPORT_VALUE(param_origin);
+	IMPORT_VALUE_PLUS(param_split_angle,needs_sync_=true);
+	IMPORT_VALUE_PLUS(param_gravity,needs_sync_=true);
+	IMPORT_VALUE_PLUS(param_gradient,needs_sync_=true);
+	IMPORT_VALUE_PLUS(param_velocity,needs_sync_=true);
+	IMPORT_VALUE_PLUS(param_perp_velocity,needs_sync_=true);
+	IMPORT_VALUE_PLUS(param_step,{
 			needs_sync_ = true;
+			Real step=param_step.get(Real());
 			if (step <= 0)
 				step=0.01; // user is probably clueless - give a good default
 			else if (step < 0.00001)
 				step=0.00001; // 100K should be enough for anyone
 			else if (step > 1)
 				step=1;
+			param_step.set(step);
 		});
-	IMPORT_PLUS(splits,{
+	IMPORT_VALUE_PLUS(param_splits,{
 			needs_sync_=true;
+			int splits=param_splits.get(int());
 			if (splits < 1)
 				splits = 1;
+			param_splits.set(splits);
 		});
-	IMPORT_PLUS(sprouts,needs_sync_=true);
-	IMPORT_PLUS(random_factor,needs_sync_=true);
-	IMPORT_PLUS(drag,needs_sync_=true);
-	IMPORT(size);
-	IMPORT(size_as_alpha);
-	IMPORT(reverse);
-	IMPORT(use_width);
+	IMPORT_VALUE_PLUS(param_sprouts,needs_sync_=true);
+	IMPORT_VALUE_PLUS(param_random_factor,needs_sync_=true);
+	IMPORT_VALUE_PLUS(param_drag,needs_sync_=true);
+	IMPORT_VALUE(param_size);
+	IMPORT_VALUE(param_size_as_alpha);
+	IMPORT_VALUE(param_reverse);
+	IMPORT_VALUE(param_use_width);
 
-	IMPORT_AS(origin,"offset");
+	if(param=="offset")
+		return set_param("origin", value);
+	if(param=="seed")
+		return set_param("random", value);
 
 	return Layer_Composite::set_param(param,value);
 }
 /*
 void
-Plant::set_time(Context context, Time time)const
+Plant::set_time(IndependentContext context, Time time)const
 {
 	if(needs_sync==true)
 	{
@@ -337,7 +372,7 @@ Plant::set_time(Context context, Time time)const
 }
 
 void
-Plant::set_time(Context context, Time time, Vector pos)const
+Plant::set_time(IndependentContext context, Time time, Vector pos)const
 {
 	if(needs_sync==true)
 	{
@@ -352,27 +387,25 @@ ValueBase
 Plant::get_param(const String& param)const
 {
 	if(param=="seed")
-	{
-		ValueBase ret(random.get_seed());
-		ret.set_static(get_param_static(param));
-		return ret;
-	}
-	EXPORT(bline);
-	EXPORT(origin);
-	EXPORT(split_angle);
-	EXPORT(gravity);
-	EXPORT(velocity);
-	EXPORT(perp_velocity);
-	EXPORT(step);
-	EXPORT(gradient);
-	EXPORT(splits);
-	EXPORT(sprouts);
-	EXPORT(random_factor);
-	EXPORT(drag);
-	EXPORT(size);
-	EXPORT(size_as_alpha);
-	EXPORT(reverse);
-	EXPORT(use_width);
+		return get_param("random");
+
+	EXPORT_VALUE(param_bline);
+	EXPORT_VALUE(param_origin);
+	EXPORT_VALUE(param_split_angle);
+	EXPORT_VALUE(param_gravity);
+	EXPORT_VALUE(param_velocity);
+	EXPORT_VALUE(param_perp_velocity);
+	EXPORT_VALUE(param_step);
+	EXPORT_VALUE(param_gradient);
+	EXPORT_VALUE(param_splits);
+	EXPORT_VALUE(param_sprouts);
+	EXPORT_VALUE(param_random_factor);
+	EXPORT_VALUE(param_drag);
+	EXPORT_VALUE(param_size);
+	EXPORT_VALUE(param_size_as_alpha);
+	EXPORT_VALUE(param_reverse);
+	EXPORT_VALUE(param_use_width);
+	EXPORT_VALUE(param_random);
 
 	EXPORT_NAME();
 
@@ -485,7 +518,7 @@ Plant::set_version(const String &ver)
 	version = ver;
 
 	if (version == "0.1")
-		use_width = false;
+		param_use_width.set(false);
 
 	return true;
 }
@@ -497,46 +530,89 @@ Plant::accelerated_render(Context context,Surface *surface,int quality, const Re
 	if(is_disabled() || !ret)
 		return ret;
 
+	if(needs_sync_==true)
+		sync();
+
 	Surface dest_surface;
 	dest_surface.set_wh(surface->get_w(),surface->get_h());
 	dest_surface.clear();
 
-	const Point	tl(renddesc.get_tl()-origin);
-	const Point br(renddesc.get_br()-origin);
+	// Here is where drawing occurs 
+	draw_particles(&dest_surface, renddesc);
 
-	const int	w(renddesc.get_w());
-	const int	h(renddesc.get_h());
+	Surface::alpha_pen pen(surface->get_pen(0,0),get_amount(),get_blend_method());
+	dest_surface.blit_to(pen);
 
-	const int	surface_width(surface->get_w());
-	const int	surface_height(surface->get_h());
+	return true;
+}
 
-	// Width and Height of a pixel
-	const Real pw = (br[0] - tl[0]) / w;
-	const Real ph = (br[1] - tl[1]) / h;
 
-	if (isinf(pw) || isinf(ph))
-		return true;
+///
+bool
+Plant::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+{
+	
+	bool ret(context.accelerated_cairorender(cr,quality,renddesc,cb));
+	if(is_disabled() || !ret)
+		return ret;
 
 	if(needs_sync_==true)
 		sync();
+	
+	cairo_save(cr);
+	cairo_push_group(cr);
+	// Here is where drawing occurs
+	draw_particles(cr);
+	cairo_pop_group_to_source(cr);
+	// blend the painted particles on the cr
+	cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
+	cairo_restore(cr);
+	
+	return true;
+}
 
+
+void
+Plant::draw_particles(Surface *dest_surface, const RendDesc &renddesc)const
+{
+	Point origin=param_origin.get(Vector());
+	Real size=param_size.get(Real());
+	bool reverse=param_reverse.get(bool());
+	bool size_as_alpha=param_size_as_alpha.get(bool());
+	
+	const Point	tl(renddesc.get_tl()-origin);
+	const Point br(renddesc.get_br()-origin);
+	
+	const int	w(renddesc.get_w());
+	const int	h(renddesc.get_h());
+	
+	const int	surface_width(dest_surface->get_w());
+	const int	surface_height(dest_surface->get_h());
+	
+	// Width and Height of a pixel
+	const Real pw = (br[0] - tl[0]) / w;
+	const Real ph = (br[1] - tl[1]) / h;
+	
+	if (isinf(pw) || isinf(ph))
+		return;
+	
 	if (particle_list.begin() != particle_list.end())
 	{
 		std::vector<Particle>::iterator iter;
 		Particle *particle;
-
+		
 		float radius(size*sqrt(1.0f/(abs(pw)*abs(ph))));
-
+		
 		int x1,y1,x2,y2;
-
+		
 		if (reverse)	iter = particle_list.end();
 		else			iter = particle_list.begin();
-
+		
 		while (true)
 		{
 			if (reverse)	particle = &(*(iter-1));
 			else			particle = &(*iter);
-
+			
 			float scaled_radius(radius);
 			Color color(particle->color);
 			if(size_as_alpha)
@@ -544,12 +620,12 @@ Plant::accelerated_render(Context context,Surface *surface,int quality, const Re
 				scaled_radius*=color.get_a();
 				color.set_a(1);
 			}
-
+			
 			// previously, radius was multiplied by sqrt(step)*12 only if
 			// the radius came out at less than 1 (pixel):
 			//   if (radius<=1.0f) radius*=sqrt(step)*12.0f;
 			// seems a little arbitrary - does it help?
-
+			
 			// calculate the box that this particle will be drawn as
 			float x1f=(particle->point[0]-tl[0])/pw-(scaled_radius*0.5);
 			float x2f=(particle->point[0]-tl[0])/pw+(scaled_radius*0.5);
@@ -559,25 +635,25 @@ Plant::accelerated_render(Context context,Surface *surface,int quality, const Re
 			x2=ceil_to_int(x2f)-1;
 			y1=ceil_to_int(y1f);
 			y2=ceil_to_int(y2f)-1;
-
+			
 			// if the box isn't entirely off the canvas, draw it
 			if(x1<=surface_width && y1<=surface_height && x2>=0 && y2>=0)
 			{
 				float x1e=x1-x1f, x2e=x2f-x2, y1e=y1-y1f, y2e=y2f-y2;
 				// printf("x1e %.4f x2e %.4f y1e %.4f y2e %.4f\n", x1e, x2e, y1e, y2e);
-
+				
 				// adjust the box so it's entirely on the canvas
 				if(x1<=0) { x1=0; x1e=0; }
 				if(y1<=0) { y1=0; y1e=0; }
 				if(x2>=surface_width)  { x2=surface_width;  x2e=0; }
 				if(y2>=surface_height) { y2=surface_height; y2e=0; }
-
+				
 				int w(x2-x1), h(y2-y1);
-
-				Surface::alpha_pen surface_pen(dest_surface.get_pen(x1,y1),1.0f);
+				
+				Surface::alpha_pen surface_pen(dest_surface->get_pen(x1,y1),1.0f);
 				if(w>0 && h>0)
-					dest_surface.fill(color,surface_pen,w,h);
-
+					dest_surface->fill(color,surface_pen,w,h);
+				
 				/* the rectangle doesn't cross any vertical pixel boundaries so we don't
 				 * need to draw any top or bottom edges
 				 */
@@ -664,7 +740,7 @@ Plant::accelerated_render(Context context,Surface *surface,int quality, const Re
 						}
 						else
 							surface_pen.move_to(x1,y2);
-
+						
 						if (y2e!=0)	// maybe draw bottom edge
 						{
 							surface_pen.set_alpha(y2e);
@@ -682,7 +758,7 @@ Plant::accelerated_render(Context context,Surface *surface,int quality, const Re
 						}
 						else
 							surface_pen.move_to(x2,y2-1);
-
+						
 						if (x2e!=0)	// maybe draw right edge
 						{
 							surface_pen.set_alpha(x2e);
@@ -700,7 +776,7 @@ Plant::accelerated_render(Context context,Surface *surface,int quality, const Re
 						}
 						else
 							surface_pen.move_to(x2-1,y1-1);
-
+						
 						if (y1e!=0)	// maybe draw top edge
 						{
 							surface_pen.set_alpha(y1e);
@@ -713,7 +789,7 @@ Plant::accelerated_render(Context context,Surface *surface,int quality, const Re
 					}
 				}
 			}
-
+			
 			if (reverse)
 			{
 				if (--iter == particle_list.begin())
@@ -726,13 +802,81 @@ Plant::accelerated_render(Context context,Surface *surface,int quality, const Re
 			}
 		}
 	}
-
-	Surface::alpha_pen pen(surface->get_pen(0,0),get_amount(),get_blend_method());
-	dest_surface.blit_to(pen);
-
-	return true;
 }
 
+
+///
+void
+Plant::draw_particles(cairo_t *cr)const
+{
+	Point origin=param_origin.get(Vector());
+	Real size=param_size.get(Real());
+	bool reverse=param_reverse.get(bool());
+	bool size_as_alpha=param_size_as_alpha.get(bool());
+
+	if (particle_list.begin() != particle_list.end())
+	{
+		std::vector<Particle>::iterator iter;
+		Particle *particle;
+		
+		float radius(size);
+		
+		if (reverse)	iter = particle_list.end();
+		else			iter = particle_list.begin();
+		
+		while (true)
+		{
+			if (reverse)	particle = &(*(iter-1));
+			else			particle = &(*iter);
+			
+			float scaled_radius(radius);
+			Color color(particle->color);
+			if(size_as_alpha)
+			{
+				scaled_radius*=color.get_a();
+				color.set_a(1);
+			}
+			
+			// calculate the box that this particle will be drawn as
+			const float x1f=particle->point[0]-scaled_radius*0.5;
+			const float x2f=particle->point[0]+scaled_radius*0.5;
+			const float y1f=particle->point[1]-scaled_radius*0.5;
+			const float y2f=particle->point[1]+scaled_radius*0.5;
+			const double width (x2f-x1f);
+			const double height(y2f-y1f);
+			
+			// grab the color components
+			const float r=color.clamped().get_r();
+			const float g=color.clamped().get_g();
+			const float b=color.clamped().get_b();
+			const float a=color.clamped().get_a();
+			
+			cairo_save(cr);
+			
+			cairo_set_source_rgb(cr, r, g, b);
+			cairo_translate(cr, origin[0], origin[1]);
+			cairo_rectangle(cr, x1f, y1f, width, height);
+			cairo_clip(cr);
+			cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+			cairo_paint_with_alpha(cr, a);
+			
+			cairo_restore(cr);
+			
+			if (reverse)
+			{
+				if (--iter == particle_list.begin())
+					break;
+			}
+			else
+			{
+				if (++iter == particle_list.end())
+					break;
+			}
+		}
+	}
+}
+
+
 Rect
 Plant::get_bounding_rect(Context context)const
 {
