@@ -6,6 +6,7 @@
 **
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
+**	Copyright (c) 2012-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -133,22 +134,28 @@ inline Color zero<Color>()
 	return Color::alpha();
 }
 
+template <>
+inline CairoColorAccumulator zero<CairoColorAccumulator>()
+{
+	return CairoColorAccumulator(0);
+}
+
 template <typename T,typename AT,class VP>
-static void GuassianBlur_2x2(etl::surface<T,AT,VP> &surface)
+static void GaussianBlur_2x2(etl::surface<T,AT,VP> &surface)
 {
 	int x,y;
-	T Tmp1,Tmp2,SR0;
+	AT Tmp1,Tmp2,SR0;
 
-	T *SC0=new T[surface.get_w()];
+	AT *SC0=new AT[surface.get_w()];
 
-	memcpy(SC0,surface[0],surface.get_w()*sizeof(T));
+	memcpy(SC0,surface[0],surface.get_w()*sizeof(AT));
 
 	for(y=0;y<surface.get_h();y++)
 	{
 		SR0=surface[y][0];
 		for(x=0;x<surface.get_w();x++)
 		{
-			Tmp1=surface[y][x];
+			Tmp1=(AT)(surface[y][x]);
 			Tmp2=SR0+Tmp1;
 			SR0=Tmp1;
 			surface[y][x]=(SC0[x]+Tmp2)/4;
@@ -159,20 +166,20 @@ static void GuassianBlur_2x2(etl::surface<T,AT,VP> &surface)
 }
 
 template <typename T,typename AT,class VP>
-static void GuassianBlur_3x3(etl::surface<T,AT,VP> &surface)
+static void GaussianBlur_3x3(etl::surface<T,AT,VP> &surface)
 {
 	int x,y,u,v,w,h;
-	T Tmp1,Tmp2,SR0,SR1;
+	AT Tmp1,Tmp2,SR0,SR1;
 
 	w=surface.get_w();
 	h=surface.get_h();
 
-	T *SC0=new T[w+1];
-	T *SC1=new T[w+1];
+	AT *SC0=new AT[w+1];
+	AT *SC1=new AT[w+1];
 
 	// Setup the row buffers
-	for(x=0;x<w;x++)SC0[x]=surface[0][x]*4;
-//	memcpy(SC1,surface[0],w*sizeof(T));
+	for(x=0;x<w;x++)SC0[x]=(AT)(surface[0][x])*4;
+	memset(SC1,0,w*sizeof(AT));
 
 	for(y=0;y<=h;y++)
 	{
@@ -210,20 +217,19 @@ static void GuassianBlur_3x3(etl::surface<T,AT,VP> &surface)
 }
 
 template <typename T,typename AT,class VP>
-inline static void GaussianBlur_5x5_(etl::surface<T,AT,VP> &surface,T *SC0,T *SC1,T *SC2,T *SC3)
+inline static void GaussianBlur_5x5_(etl::surface<T,AT,VP> &surface,AT *SC0,AT *SC1,AT *SC2,AT *SC3)
 {
 	int x,y,u,v,w,h;
-	T Tmp1,Tmp2,SR0,SR1,SR2,SR3;
+	AT Tmp1,Tmp2,SR0,SR1,SR2,SR3;
 
 	w=surface.get_w();
 	h=surface.get_h();
 
 	// Setup the row buffers
-	for(x=0;x<w;x++)SC0[x+2]=surface[0][x]*24;
-//	memset(SC0,0,(w+2)*sizeof(T));
-	memset(SC1,0,(w+2)*sizeof(T));
-	memset(SC2,0,(w+2)*sizeof(T));
-	memset(SC3,0,(w+2)*sizeof(T));
+	for(x=0;x<w;x++)SC0[x+2]=(AT)(surface[0][x])*24;
+	memset(SC1,0,(w+2)*sizeof(AT));
+	memset(SC2,0,(w+2)*sizeof(AT));
+	memset(SC3,0,(w+2)*sizeof(AT));
 
 	for(y=0;y<h+2;y++)
 	{
@@ -233,7 +239,7 @@ inline static void GaussianBlur_5x5_(etl::surface<T,AT,VP> &surface,T *SC0,T *SC
 			v=y;
 
 		SR0=SR1=SR2=SR3=0;
-		SR0=surface[v][0]*1.5;
+		SR0=(AT)(surface[v][0])*1.5;
 		for(x=0;x<w+2;x++)
 		{
 			if(x>=w)
@@ -272,10 +278,10 @@ inline static void GaussianBlur_5x5(etl::surface<T,AT,VP> &surface)
 {
 	int w=surface.get_w();
 
-	T *SC0=new T[w+2];
-	T *SC1=new T[w+2];
-	T *SC2=new T[w+2];
-	T *SC3=new T[w+2];
+	AT *SC0=new AT[w+2];
+	AT *SC1=new AT[w+2];
+	AT *SC2=new AT[w+2];
+	AT *SC3=new AT[w+2];
 
 	GaussianBlur_5x5_(surface,SC0,SC1,SC2,SC3);
 
@@ -286,29 +292,29 @@ inline static void GaussianBlur_5x5(etl::surface<T,AT,VP> &surface)
 }
 
 template <typename T,typename AT,class VP>
-static void GuassianBlur_nxn(etl::surface<T,AT,VP> &surface,int n)
+static void GaussianBlur_nxn(etl::surface<T,AT,VP> &surface,int n)
 {
 	int x,y,u,v,w,h;
 	int half_n=n/2,i;
-	T inv_divisor=pow(2.0,(n-1));
-	T Tmp1,Tmp2;
+	float inv_divisor=pow(2.0,(n-1));
+	AT Tmp1,Tmp2;
 	inv_divisor=1.0/(inv_divisor*inv_divisor);
 
 	w=surface.get_w();
 	h=surface.get_h();
 
-    T SR[n-1];
-	T *SC[n-1];
+    AT SR[n-1];
+	AT *SC[n-1];
 
 	for(i=0;i<n-1;i++)
 	{
-		SC[i]=new T[w+half_n];
+		SC[i]=new AT[w+half_n];
 		if(!SC[i])
 		{
 			throw(runtime_error(strprintf(__FILE__":%d:Malloc failure",__LINE__)));
 			return;
 		}
-		memset(SC[i],0,(w+half_n)*sizeof(T));
+		memset(SC[i],0,(w+half_n)*sizeof(AT));
 	}
 
 	// Setup the first row
@@ -321,7 +327,7 @@ static void GuassianBlur_nxn(etl::surface<T,AT,VP> &surface,int n)
 		else
 			v=y;
 
-		memset(SR,0,(n-1)*sizeof(T));
+		memset(SR,0,(n-1)*sizeof(AT));
 
 //		SR[0]=surface[v][0]*(2.0-1.9/n);
 
@@ -363,7 +369,7 @@ static void GuassianBlur_nxn(etl::surface<T,AT,VP> &surface,int n)
 }
 
 template <typename T,typename AT,class VP>
-static void GuassianBlur_2x1(etl::surface<T,AT,VP> &surface)
+static void GaussianBlur_2x1(etl::surface<T,AT,VP> &surface)
 {
 	int x,y;
 	AT Tmp1,Tmp2,SR0;
@@ -382,7 +388,7 @@ static void GuassianBlur_2x1(etl::surface<T,AT,VP> &surface)
 }
 
 template <typename T,typename AT,class VP>
-static void GuassianBlur_3x1(etl::surface<T,AT,VP> &surface)
+static void GaussianBlur_3x1(etl::surface<T,AT,VP> &surface)
 {
 	int x,y;
 	AT Tmp1,Tmp2,SR0,SR1;
@@ -406,14 +412,14 @@ static void GuassianBlur_3x1(etl::surface<T,AT,VP> &surface)
 }
 
 template <typename T,typename AT,class VP>
-static void GuassianBlur_1x2(etl::surface<T,AT,VP> &surface)
+static void GaussianBlur_1x2(etl::surface<T,AT,VP> &surface)
 {
 	int x,y;
 	AT Tmp1,Tmp2,SR0;
 
 	for(x=0;x<surface.get_w();x++)
 	{
-		SR0 = zero<T>();
+		SR0 = zero<AT>();
 		for(y=0;y<surface.get_h();y++)
 		{
 			Tmp1=surface[y][x];
@@ -425,7 +431,7 @@ static void GuassianBlur_1x2(etl::surface<T,AT,VP> &surface)
 }
 
 template <typename T,typename AT,class VP>
-static void GuassianBlur_1x3(etl::surface<T,AT,VP> &surface)
+static void GaussianBlur_1x3(etl::surface<T,AT,VP> &surface)
 {
 	int x,y;
 	AT Tmp1,Tmp2,SR0,SR1;
@@ -728,14 +734,14 @@ bool Blur::operator()(const Surface &surface,
 			int bh = (int)(abs(ph)*size[1]*GAUSSIAN_ADJUSTMENT+0.5);
 			int max=bw+bh;
 
-			Color *SC0=new class Color[w+2];
-			Color *SC1=new class Color[w+2];
-			Color *SC2=new class Color[w+2];
-			Color *SC3=new class Color[w+2];
-			memset(SC0,0,(w+2)*sizeof(Color));
-			memset(SC0,0,(w+2)*sizeof(Color));
-			memset(SC0,0,(w+2)*sizeof(Color));
-			memset(SC0,0,(w+2)*sizeof(Color));
+			ColorAccumulator *SC0=new ColorAccumulator[w+2];
+			ColorAccumulator *SC1=new ColorAccumulator[w+2];
+			ColorAccumulator *SC2=new ColorAccumulator[w+2];
+			ColorAccumulator *SC3=new ColorAccumulator[w+2];
+			memset(SC0,0,(w+2)*sizeof(ColorAccumulator));
+			memset(SC1,0,(w+2)*sizeof(ColorAccumulator));
+			memset(SC2,0,(w+2)*sizeof(ColorAccumulator));
+			memset(SC3,0,(w+2)*sizeof(ColorAccumulator));
 
 			//synfig::warning("Didn't crash yet b2");
 			//int i = 0;
@@ -758,7 +764,7 @@ bool Blur::operator()(const Surface &surface,
 				else
 				if(bw>=1 && bh>=1)
 				{
-					GuassianBlur_2x2(*gauss_surface);
+					GaussianBlur_2x2(*gauss_surface);
 					bw--,bh--;
 				}
 
@@ -769,13 +775,13 @@ bool Blur::operator()(const Surface &surface,
 				if(!blurcall.amount_complete(max-(bw+bh),max))return false;
 				if(bw>=2)
 				{
-					GuassianBlur_3x1(*gauss_surface);
+					GaussianBlur_3x1(*gauss_surface);
 					bw-=2;
 				}
 				else
 				if(bw>=1)
 				{
-					GuassianBlur_2x1(*gauss_surface);
+					GaussianBlur_2x1(*gauss_surface);
 					bw--;
 				}
 				//synfig::warning("Didn't crash yet bi - %d",i++);
@@ -785,13 +791,13 @@ bool Blur::operator()(const Surface &surface,
 				if(!blurcall.amount_complete(max-(bw+bh),max))return false;
 				if(bh>=2)
 				{
-					GuassianBlur_1x3(*gauss_surface);
+					GaussianBlur_1x3(*gauss_surface);
 					bh-=2;
 				}
 				else
 				if(bh>=1)
 				{
-					GuassianBlur_1x2(*gauss_surface);
+					GaussianBlur_1x2(*gauss_surface);
 					bh--;
 				}
 				//synfig::warning("Didn't crash yet bi - %d",i++);
@@ -846,6 +852,358 @@ bool Blur::operator()(const Surface &surface,
 	return true;
 }
 
+//////
+bool Blur::operator()(cairo_surface_t *surface,
+					  const Vector &resolution,
+					  cairo_surface_t *out) const
+{
+
+	CairoSurface cairosurface(surface);
+	if(!cairosurface.map_cairo_image())
+	{
+		synfig::info("cairosurface map cairo image failed");
+		return false;
+	}
+
+	int w = cairosurface.get_w(),
+	h = cairosurface.get_h();
+	
+	if(w == 0 || h == 0 || resolution[0] == 0 || resolution[1] == 0)
+	{
+		cairosurface.unmap_cairo_image();
+		return false;
+	}
+	
+	const Real	pw = resolution[0]/w,
+	ph = resolution[1]/h;
+	
+	int	halfsizex = (int) (abs(size[0]*.5/pw) + 1),
+	halfsizey = (int) (abs(size[1]*.5/ph) + 1);
+	
+	int x,y;
+	
+	SuperCallback blurcall(cb,0,5000,5000);
+	
+	CairoSurface cairoout(out);
+	if(!cairoout.map_cairo_image())
+	{
+		synfig::info("cairoout map cairo image failed");
+		cairosurface.unmap_cairo_image();
+		return false;
+	}
+
+	switch(type)
+	{
+		case Blur::DISC:	// D I S C ----------------------------------------------------------
+		{
+			int bw = halfsizex;
+			int bh = halfsizey;
+			
+			if(size[0] && size[1] && w*h>2)
+			{
+				int x2,y2;
+				
+				for(y=0;y<h;y++)
+				{
+					for(x=0;x<w;x++)
+					{
+						//accumulate all the pixels in an ellipse of w,h about the current pixel
+						Color color=Color::alpha();
+						int total=0;
+						
+						for(y2=-bh;y2<=bh;y2++)
+						{
+							for(x2=-bw;x2<=bw;x2++)
+							{
+								//get the floating point distance away from the origin pixel in relative coords
+								float tmp_x=(float)x2/bw;
+								float tmp_y=(float)y2/bh;
+								tmp_x*=tmp_x;
+								tmp_y*=tmp_y;
+								
+								//ignore if it's outside of the disc
+								if( tmp_x+tmp_y>1.0)
+									continue;
+								
+								//cap the pixel indices to inside the surface
+								int u= x+x2,
+								v= y+y2;
+								
+								if( u < 0 )					u = 0;
+								if( u >= w ) u = w-1;
+								
+								if( v < 0 ) 				v = 0;
+								if( v >= h ) v = h-1;
+								
+								//accumulate the color, and # of pixels added in
+								color += Color(cairosurface[v][u]);
+								total++;
+							}
+						}
+						
+						//blend the color with the original color
+						cairoout[y][x]=CairoColor(color/total);
+					}
+					if(!blurcall.amount_complete(y,h))
+					{
+						if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Renderer Failure",__LINE__));
+						cairosurface.unmap_cairo_image();
+						cairoout.unmap_cairo_image();
+						return false;
+					}
+				}
+				break;
+			}
+			//if we don't qualify for disc blur just use box blur
+		}
+			
+		case Blur::BOX: // B O X -------------------------------------------------------
+		{
+			cairo_surface_t* temp=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, w, h);
+			CairoSurface cairotemp(temp);
+			if(!cairotemp.map_cairo_image())
+			{
+				synfig::info("cairotemp map cairo image failed");
+				return false;
+			}
+			//horizontal part
+			if(size[0])
+			{
+				int length = halfsizex;
+				length=std::max(1,length);
+				etl::hbox_blur(cairosurface.begin(),cairosurface.end(),length,cairotemp.begin());
+			}
+			else cairotemp.copy(cairosurface);
+			
+			//vertical part
+			if(size[1])
+			{
+				int length = halfsizey;
+				length = std::max(1,length);
+				etl::vbox_blur(cairotemp.begin(),cairotemp.end(),length,cairoout.begin());
+			}
+			else cairoout.copy(cairotemp);
+			cairotemp.unmap_cairo_image();
+			cairo_surface_destroy(temp);
+		}
+			break;
+			
+		case Blur::FASTGAUSSIAN:	// F A S T G A U S S I A N ----------------------------------------------
+		{
+			//fast gaussian is treated as a 3x3 type of thing, except expanded to work with the length
+			
+			/*	1	2	1
+			 2	4	2
+			 1	2	1
+			 */
+			cairo_surface_t* temp=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, w, h);
+			CairoSurface cairotemp(temp);
+			if(!cairotemp.map_cairo_image())
+			{
+				synfig::info("cairotemp map cairo image failed. Fast Gaussian");
+				return false;
+			}
+			
+			//horizontal part
+			if(size[0])
+			{
+				Real length=abs((float)w/(resolution[0]))*size[0]*0.5+1;
+				length=std::max(1.0,length);
+				
+				//two box blurs produces: 1 2 1
+				etl::hbox_blur(cairosurface.begin(),w,h,(int)(length*3/4),cairotemp.begin());
+				etl::hbox_blur(cairotemp.begin(),w,h,(int)(length*3/4),cairoout.begin());
+			}
+			else cairoout.copy(cairosurface);
+			
+			// Interchange result with temp
+			cairotemp.copy(cairoout);
+			//vertical part
+			if(size[1])
+			{
+				Real length=abs((float)h/(resolution[1]))*size[1]*0.5+1;
+				length=std::max(1.0,length);
+				
+				//two box blurs produces: 1 2 1 on the horizontal 1 2 1
+				etl::vbox_blur(cairoout.begin(),w,h,(int)(length*3/4),cairotemp.begin());
+				etl::vbox_blur(cairotemp.begin(),w,h,(int)(length*3/4),cairoout.begin());
+			}
+			else cairoout.copy(cairotemp);
+			cairotemp.unmap_cairo_image();
+			cairo_surface_destroy(temp);			
+		}
+			break;
+			
+		case Blur::CROSS: // C R O S S  -------------------------------------------------------
+		{
+			//horizontal part
+			cairo_surface_t* temp=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, w, h);
+			CairoSurface cairotemp(temp);
+			if(!cairotemp.map_cairo_image())
+			{
+				synfig::info("cairotemp map cairo image failed. Cross");
+				cairo_surface_destroy(temp);
+				return false;
+			}
+			
+			if(size[0])
+			{
+				int length = halfsizex;
+				length = std::max(1,length);
+				
+				etl::hbox_blur(cairosurface.begin(),cairosurface.end(),length,cairotemp.begin());
+			}
+			else cairotemp.copy(cairosurface);
+			
+			//vertical part
+			cairo_surface_t* temp2=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, w, h);
+			CairoSurface cairotemp2(temp2);
+			if(!cairotemp2.map_cairo_image())
+			{
+				synfig::info("cairotemp2 map cairo image failed. Cross");
+				cairotemp.unmap_cairo_image();
+				cairo_surface_destroy(temp);
+				cairo_surface_destroy(temp2);
+				return false;
+			}
+			
+			if(size[1])
+			{
+				int length = halfsizey;
+				length = std::max(1,length);
+				
+				etl::vbox_blur(cairosurface.begin(),cairosurface.end(),length,cairotemp2.begin());
+			}
+			else cairotemp2.copy(cairosurface);
+			
+			//blend the two together
+			int x,y;
+			
+			for(y=0;y<h;y++)
+			{
+				for(x=0;x<w;x++)
+				{
+					cairoout[y][x] = cairotemp[y][x]*0.5+cairotemp2[y][x]*0.5;
+				}
+			}
+			cairotemp.unmap_cairo_image();
+			cairo_surface_destroy(temp);
+			cairotemp2.unmap_cairo_image();
+			cairo_surface_destroy(temp2);
+			break;
+		}
+			
+		case Blur::GAUSSIAN:	// G A U S S I A N ----------------------------------------------
+		{
+#ifndef	GAUSSIAN_ADJUSTMENT
+#define GAUSSIAN_ADJUSTMENT		(0.05)
+#endif
+			
+			Real	pw = (Real)w/(resolution[0]);
+			Real 	ph = (Real)h/(resolution[1]);
+			
+			CairoSurface *gauss_surface;
+			cairoout.copy(cairosurface);
+			gauss_surface = &cairoout;
+            /* Squaring the pw and ph values
+			 is necessary to insure consistent
+			 results when rendered to different
+			 resolutions.
+			 Unfortunately, this automatically
+			 squares our rendertime.
+			 There has got to be a faster way...
+			 */
+			pw=pw*pw;
+			ph=ph*ph;
+			
+			int bw = (int)(abs(pw)*size[0]*GAUSSIAN_ADJUSTMENT+0.5);
+			int bh = (int)(abs(ph)*size[1]*GAUSSIAN_ADJUSTMENT+0.5);
+			int max=bw+bh;
+			
+			CairoColorAccumulator *SC0=new class CairoColorAccumulator[w+2];
+			CairoColorAccumulator *SC1=new class CairoColorAccumulator[w+2];
+			CairoColorAccumulator *SC2=new class CairoColorAccumulator[w+2];
+			CairoColorAccumulator *SC3=new class CairoColorAccumulator[w+2];
+			memset(SC0,0,(w+2)*sizeof(CairoColorAccumulator));
+			memset(SC1,0,(w+2)*sizeof(CairoColorAccumulator));
+			memset(SC2,0,(w+2)*sizeof(CairoColorAccumulator));
+			memset(SC3,0,(w+2)*sizeof(CairoColorAccumulator));
+						
+			while(bw&&bh)
+			{
+				if(!blurcall.amount_complete(max-(bw+bh),max))return false;
+				
+				if(bw>=4 && bh>=4)
+				{
+					etl::gaussian_blur_5x5_(gauss_surface->begin(),gauss_surface->get_w(),gauss_surface->get_h(),SC0,SC1,SC2,SC3);
+					bw-=4,bh-=4;
+				}
+				else
+					if(bw>=2 && bh>=2)
+					{
+						etl::gaussian_blur_3x3(gauss_surface->begin(),gauss_surface->end());
+						bw-=2,bh-=2;
+					}
+					else
+						if(bw>=1 && bh>=1)
+						{
+							GaussianBlur_2x2(*gauss_surface);
+							bw--,bh--;
+						}				
+			}
+			while(bw)
+			{
+				if(!blurcall.amount_complete(max-(bw+bh),max))return false;
+				if(bw>=2)
+				{
+					GaussianBlur_3x1(*gauss_surface);
+					bw-=2;
+				}
+				else
+					if(bw>=1)
+					{
+						GaussianBlur_2x1(*gauss_surface);
+						bw--;
+					}
+			}
+			while(bh)
+			{
+				if(!blurcall.amount_complete(max-(bw+bh),max))return false;
+				if(bh>=2)
+				{
+					GaussianBlur_1x3(*gauss_surface);
+					bh-=2;
+				}
+				else
+					if(bh>=1)
+					{
+						GaussianBlur_1x2(*gauss_surface);
+						bh--;
+					}
+			}
+			
+			delete [] SC0;
+			delete [] SC1;
+			delete [] SC2;
+			delete [] SC3;
+		}
+			break;
+			
+		default:
+			break;
+	}
+
+	//we are FRIGGGIN done....
+	blurcall.amount_complete(100,100);
+	
+	cairosurface.unmap_cairo_image();
+	cairoout.unmap_cairo_image();
+	
+	return true;
+}
+
+//////
+
 bool Blur::operator()(const etl::surface<float> &surface,
 					  const synfig::Vector &resolution,
 					  etl::surface<float> &out) const
@@ -1132,7 +1490,7 @@ bool Blur::operator()(const etl::surface<float> &surface,
 				else
 				if(bw>=1 && bh>=1)
 				{
-					GuassianBlur_2x2(*gauss_surface);
+					GaussianBlur_2x2(*gauss_surface);
 					bw--,bh--;
 				}
 			}
@@ -1142,13 +1500,13 @@ bool Blur::operator()(const etl::surface<float> &surface,
 				if(!blurcall.amount_complete(max-(bw+bh),max))return false;
 				if(bw>=2)
 				{
-					GuassianBlur_3x1(*gauss_surface);
+					GaussianBlur_3x1(*gauss_surface);
 					bw-=2;
 				}
 				else
 				if(bw>=1)
 				{
-					GuassianBlur_2x1(*gauss_surface);
+					GaussianBlur_2x1(*gauss_surface);
 					bw--;
 				}
 			}
@@ -1158,13 +1516,13 @@ bool Blur::operator()(const etl::surface<float> &surface,
 				if(!blurcall.amount_complete(max-(bw+bh),max))return false;
 				if(bh>=2)
 				{
-					GuassianBlur_1x3(*gauss_surface);
+					GaussianBlur_1x3(*gauss_surface);
 					bh-=2;
 				}
 				else
 				if(bh>=1)
 				{
-					GuassianBlur_1x2(*gauss_surface);
+					GaussianBlur_1x2(*gauss_surface);
 					bh--;
 				}
 			}
