@@ -154,6 +154,9 @@ extern void warning(const String &str);
 extern void info(const char *format,...);
 extern void info(const String &str);
 
+//! Returns absolute path to the binary
+extern String get_binary_path(const String &fallback_path);
+
 }; // END of namespace synfig
 
 /* === E N D =============================================================== */
