@@ -39,6 +39,7 @@
 #include "node.h"
 #include "time.h"
 #include "guid.h"
+#include "interpolation.h"
 #include "target.h" // for RenderMethod. TODO: put RenderMethod apart
 
 #include "cairo.h"
@@ -80,44 +81,34 @@
 		return new class();																\
 	}
 
-//! Imports a parameter 'x' and perform an action based usually based on
-//! some condition 'y'
-#define IMPORT_PLUS(x,y)																\
-	if (param==#x && value.same_type_as(x))												\
-	{																					\
-		value.put(&x);																	\
-		set_param_static(#x,value.get_static());										\
-		{																				\
-			y;																			\
-		}																				\
-		return true;																	\
-	}
-
-//! Imports a parameter 'y' if it has the same type than 'x'
-#define IMPORT_AS(x,y)																	\
-	if (param==y && value.same_type_as(x))												\
-	{																					\
-		value.put(&x);																	\
-		set_param_static(y,value.get_static());										\
-		return true;																	\
-	}
-
 //! Imports a parameter if it is of the same type as param
-#define IMPORT(x)																		\
-	IMPORT_AS(x,#x)
+#define IMPORT_VALUE(x)                                                         \
+	if (#x=="param_"+param && x.get_type()==value.get_type())                   \
+	{                                                                           \
+		x.copy(value);                                                                \
+		return true;                                                            \
+	}
 
-//! Exports a parameter 'x' if param is same type as given 'y'
-#define EXPORT_AS(x,y)																	\
-	if (param==y)																		\
-	{																					\
-		synfig::ValueBase ret(x);														\
-		ret.set_static(get_param_static(y));											\
-		return ret;																		\
+//! Imports a parameter 'x' and perform an action usually based on
+//! some condition 'y'
+#define IMPORT_VALUE_PLUS(x,y)                                                  \
+	if (#x=="param_"+param && x.get_type()==value.get_type())                   \
+	{                                                                           \
+		x=value;                                                                \
+		{                                                                       \
+			y;                                                                  \
+		}                                                                       \
+		return true;                                                            \
 	}
 
 //! Exports a parameter if it is the same type as value
-#define EXPORT(x)																		\
-	EXPORT_AS(x,#x)
+#define EXPORT_VALUE(x)                                                         \
+	if (#x=="param_"+param)                                                     \
+	{                                                                           \
+		synfig::ValueBase ret;					\
+		ret.copy(x);							\
+		return ret;							\
+	}
 
 //! Exports the name or the local name of the layer
 #define EXPORT_NAME()																	\
@@ -134,19 +125,32 @@
 //! This is used as the category for layer book entries which represent aliases of layers.
 //! It prevents these layers showing up in the menu.
 #define CATEGORY_DO_NOT_USE "Do Not Use"
-/*
-//! x=variable name, y=static bool value
-#define SET_STATIC(x,y)																	\
-	if(param==#x && x ## _static != y)													\
-	{																					\
-		x ## _static = y;																\
-		return true;																	\
-	}
 
-#define GET_STATIC(x)																	\
-	if(param==#x)																		\
-		return x ## _static;															\
-*/
+//! Sets the interpolation defaults for the layer
+#define SET_INTERPOLATION_DEFAULTS()                                           \
+{                                                                              \
+	Vocab vocab(get_param_vocab());                                            \
+	Vocab::const_iterator viter;                                               \
+	for(viter=vocab.begin();viter!=vocab.end();viter++)                        \
+	{                                                                          \
+		ValueBase v=get_param(viter->get_name());                              \
+		v.set_interpolation(viter->get_interpolation());                       \
+		set_param(viter->get_name(), v);                                       \
+	}                                                                          \
+}                                                                              \
+
+//! Sets the static defaults for the layer
+#define SET_STATIC_DEFAULTS()                                                  \
+{                                                                              \
+	Vocab vocab(get_param_vocab());                                            \
+	Vocab::const_iterator viter;                                               \
+	for(viter=vocab.begin();viter!=vocab.end();viter++)                        \
+	{                                                                          \
+		ValueBase v=get_param(viter->get_name());                              \
+		v.set_static(viter->get_static());                                     \
+		set_param(viter->get_name(), v);                                       \
+	}                                                                          \
+}                                                                              \
 
 /* === T Y P E D E F S ===================================================== */
 
@@ -167,6 +171,7 @@ class Surface;
 class CairoSurface;
 class RendDesc;
 class ProgressCallback;
+class IndependentContext;
 class Context;
 class Color;
 class CairoColor;
@@ -182,6 +187,7 @@ class GUID;
 class Layer : public Node
 {
 	friend class ValueNode;
+	friend class IndependentContext;
 	friend class Context;
 
 	/*
@@ -267,6 +273,12 @@ private:
 	*/
 	bool active_;
 
+	/*! When \c true, layer will skipped while final rendering
+	**	but will still present onto work view.
+	**	\see set_exclude_from_rendering(), get_exclude_from_rendering()
+	*/
+	bool exclude_from_rendering_;
+
 	//! Handle to the canvas to which this layer belongs
 	etl::loose_handle<Canvas> canvas_;
 
@@ -277,11 +289,7 @@ private:
 	String description_;
 
 	//! The depth parameter of the layer in the layer stack
-	float z_depth;
-
-	//! True if zdepth is not affected when in animation mode
-	typedef std::map<String, bool> Sparams;
-	Sparams static_params;
+	ValueBase param_z_depth;
 
 	//! \writeme
 	mutable Time dirty_time_;
@@ -390,18 +398,26 @@ public:
 	//! Returns that status of the 'active' flag
 	bool active()const { return active_; }
 
+	//! Sets the 'exclude_from_rendering' flag for the Layer
+	/*! When set, layer will skipped while final rendering
+	**	but will still present onto work view. */
+	void set_exclude_from_rendering(bool x);
+
+	//! Returns that status of the 'exclude_from_rendering' flag
+	bool get_exclude_from_rendering()const { return exclude_from_rendering_; }
+
 	//! Returns the position of the layer in the canvas.
 	/*! Returns negative on error */
 	int get_depth()const;
 
 	//! Gets the non animated z depth of the layer
-	float get_z_depth()const { return z_depth; }
+	float get_z_depth()const { return param_z_depth.get(Real()); }
 
 	//! Gets the z depth of the layer at a time t
 	float get_z_depth(const synfig::Time& t)const;
 
 	//! Sets the z depth of the layer (non animated)
-	void set_z_depth(float x) { z_depth=x; }
+	void set_z_depth(float x) { param_z_depth=ValueBase(Real(x)); }
 
 	//! Sets the Canvas that this Layer is a part of
 	void set_canvas(etl::loose_handle<Canvas> canvas);
@@ -412,6 +428,9 @@ public:
 	//! Returns the description of the layer
 	const String& get_description()const { return description_; }
 
+
+	String get_string()const;
+
 	//! Sets the description of the layer
 	void set_description(const String& x);
 
@@ -470,10 +489,6 @@ public:
 	*/
 	virtual bool set_param(const String &param, const ValueBase &value);
 
-	virtual bool set_param_static(const String &param, const bool x);
-	virtual bool get_param_static(const String &param) const;
-	virtual void fill_static(Vocab vocab);
-
 	//!	Sets a list of parameters
 	virtual bool set_param_list(const ParamList &);
 
@@ -492,7 +507,7 @@ public:
 	**	\param time			writeme
 	**	\see Context::set_time()
 	*/
-	virtual void set_time(Context context, Time time)const;
+	virtual void set_time(IndependentContext context, Time time)const;
 
 	//! Sets the \a time for the selected Layer and those under it for a specific \a point
 	/*!	\param context		Context iterator referring to next Layer.
@@ -500,7 +515,7 @@ public:
 	**	\param point		writeme
 	**	\see Context::set_time()
 	**	\todo \a point should be of the type <tt>const Point \&</tt> */
-	virtual void set_time(Context context, Time time, const Point &point)const;
+	virtual void set_time(IndependentContext context, Time time, const Point &point)const;
 
 	//! Gets the blend color of the Layer in the context at \a pos
 	/*!	\param context		Context iterator referring to next Layer.
@@ -523,7 +538,7 @@ public:
 	**	\see Context::accelerated_render()
 	*/
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
-	virtual bool accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context, cairo_t* cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 
 	//! Checks to see if a part of the layer is directly under \a point
 	/*!	\param context		Context iterator referring to next Layer.
@@ -535,7 +550,7 @@ public:
 	virtual Handle hit_check(Context context, const Point &point)const;
 
 	//! Duplicates the Layer
-	virtual Handle clone(const GUID& deriv_guid=GUID())const;
+	virtual Handle clone(etl::loose_handle<Canvas> canvas, const GUID& deriv_guid=GUID())const;
 
 	//! Returns true if the layer needs to be able to examine its context.
 	/*! context to render itself, other than for simple blending.  For
