@@ -64,31 +64,30 @@ using namespace etl;
 synfig::Layer_Bitmap::Layer_Bitmap():
     Layer_Composite	(1.0,Color::BLEND_COMPOSITE),
 	method			(SOFTWARE),
-	tl				(-0.5,0.5),
-	br				(0.5,-0.5),
-	c				(1),
+	param_tl				(Point(-0.5,0.5)),
+	param_br				(Point(0.5,-0.5)),
+	param_c				(int(1)),
+	param_gamma_adjust	(Real(1.0)),
 	surface			(128,128),
-	trimmed			(false),
-	gamma_adjust	(1.0)
+	trimmed			(false)
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
-	set_param_static("c", true);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 bool
 synfig::Layer_Bitmap::set_param(const String & param, const ValueBase & value)
 {
-	IMPORT(tl);
-	IMPORT(br);
-	IMPORT(c);
-	if(param=="gamma_adjust"&& value.get_type()==ValueBase::TYPE_REAL)
-	{
-		set_param_static(param, value.get_static());
-		gamma_adjust=1.0/value.get(Real());
-		//gamma_adjust.set_gamma(1.0/value.get(Real()));
-		return true;
-	}
+	IMPORT_VALUE(param_tl);
+	IMPORT_VALUE(param_br);
+	IMPORT_VALUE(param_c);
+	IMPORT_VALUE_PLUS(param_gamma_adjust,
+		if(param=="gamma_adjust"&& value.get_type()==ValueBase::TYPE_REAL)
+		{
+			param_gamma_adjust.set(Real(1.0/value.get(Real())));
+			return true;
+		}
+		);
 
 	return Layer_Composite::set_param(param,value);
 }
@@ -96,13 +95,13 @@ synfig::Layer_Bitmap::set_param(const String & param, const ValueBase & value)
 ValueBase
 synfig::Layer_Bitmap::get_param(const String & param)const
 {
-	EXPORT(tl);
-	EXPORT(br);
-	EXPORT(c);
+	EXPORT_VALUE(param_tl);
+	EXPORT_VALUE(param_br);
+	EXPORT_VALUE(param_c);
 	if(param=="gamma_adjust")
 	{
-		ValueBase ret(1.0/gamma_adjust);
-		ret.set_static(get_param_static(param));
+		ValueBase ret=param_gamma_adjust;
+		ret.set(1.0/param_gamma_adjust.get(Real()));
 		return ret;
 	}
 
@@ -121,8 +120,6 @@ synfig::Layer_Bitmap::get_param(const String & param)const
 				ret2=int(csurface.get_w());
 				break;
 		}
-		ret1.set_static(get_param_static(param));
-		ret2.set_static(get_param_static(param));
 		if (trimmed) return ret1;
 		return ret2;
 	}
@@ -141,8 +138,6 @@ synfig::Layer_Bitmap::get_param(const String & param)const
 				ret2=int(csurface.get_h());
 				break;
 		}
-		ret1.set_static(get_param_static(param));
-		ret2.set_static(get_param_static(param));
 		if (trimmed) return ret1;
 		return ret2;
 	}
@@ -150,6 +145,7 @@ synfig::Layer_Bitmap::get_param(const String & param)const
 	return Layer_Composite::get_param(param);
 }
 
+
 Layer::Vocab
 Layer_Bitmap::get_param_vocab()const
 {
@@ -173,6 +169,7 @@ Layer_Bitmap::get_param_vocab()const
 		.add_enum_value(1,"linear",_("Linear"))
 		.add_enum_value(2,"cosine",_("Cosine"))
 		.add_enum_value(3,"cubic",_("Cubic"))
+		.set_static(true)
 	);
 
 	ret.push_back(ParamDesc("gamma_adjust")
@@ -185,6 +182,8 @@ Layer_Bitmap::get_param_vocab()const
 synfig::Layer::Handle
 Layer_Bitmap::hit_check(synfig::Context context, const synfig::Point &pos)const
 {
+	Point tl(param_tl.get(Point()));
+	Point br(param_br.get(Point()));
 	Point surface_pos;
 	surface_pos=pos-tl;
 
@@ -212,6 +211,7 @@ inline
 const Color&
 synfig::Layer_Bitmap::filter(Color& x)const
 {
+	Real gamma_adjust(param_gamma_adjust.get(Real()));
 	if(gamma_adjust!=1.0)
 	{
 		x.set_r(powf((float)x.get_r(),gamma_adjust));
@@ -225,6 +225,7 @@ inline
 const CairoColor&
 synfig::Layer_Bitmap::filter(CairoColor& x)const
 {
+	Real gamma_adjust(param_gamma_adjust.get(Real()));
 	if(gamma_adjust!=1.0)
 	{
 		x.set_r(powf((float)(x.get_r()/CairoColor::range),gamma_adjust)*CairoColor::range);
@@ -238,6 +239,10 @@ synfig::Layer_Bitmap::filter(CairoColor& x)const
 Color
 synfig::Layer_Bitmap::get_color(Context context, const Point &pos)const
 {
+	Point tl(param_tl.get(Point()));
+	Point br(param_br.get(Point()));
+	int c(param_c.get(int()));
+
 	Point surface_pos;
 
 	if(!get_amount())
@@ -311,6 +316,10 @@ synfig::Layer_Bitmap::get_color(Context context, const Point &pos)const
 CairoColor
 synfig::Layer_Bitmap::get_cairocolor(Context context, const Point &pos)const
 {
+	Point tl(param_tl.get(Point()));
+	Point br(param_br.get(Point()));
+	int c(param_c.get(int()));
+
 	Point surface_pos;
 	
 	if(!get_amount())
@@ -383,6 +392,11 @@ synfig::Layer_Bitmap::get_cairocolor(Context context, const Point &pos)const
 bool
 Layer_Bitmap::accelerated_render(Context context,Surface *out_surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)  const
 {
+	Point tl(param_tl.get(Point()));
+	Point br(param_br.get(Point()));
+	int c(param_c.get(int()));
+	Real gamma_adjust(param_gamma_adjust.get(Real()));
+
 	int interp=c;
 	if(quality>=10)
 		interp=0;
@@ -650,11 +664,16 @@ Layer_Bitmap::accelerated_render(Context context,Surface *out_surface,int qualit
 	return true;
 }
 
+/////
 /////
 
 bool
-Layer_Bitmap::accelerated_cairorender(Context context, cairo_surface_t *out_surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)  const
+Layer_Bitmap::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)  const
 {
+	Point tl(param_tl.get(Point()));
+	Point br(param_br.get(Point()));
+	int c(param_c.get(int()));
+	Real gamma_adjust(param_gamma_adjust.get(Real()));
 	
 	int interp=c;
 	if(quality>=10)
@@ -666,25 +685,25 @@ Layer_Bitmap::accelerated_cairorender(Context context, cairo_surface_t *out_surf
 	if(!csurface.is_mapped())
 	{
 		// Render what is behind us
-		return context.accelerated_cairorender(out_surface,quality,renddesc,cb);
+		return context.accelerated_cairorender(cr,quality,renddesc,cb);
 	}
 	
 	cairo_surface_t* cs=csurface.get_cairo_image_surface();
-
+	
 	if(cairo_surface_status(cs) || cairo_surface_get_type(cs)!=CAIRO_SURFACE_TYPE_IMAGE)
 	{
 		// Render what is behind us
-		return context.accelerated_cairorender(out_surface,quality,renddesc,cb);
+		return context.accelerated_cairorender(cr,quality,renddesc,cb);
 	}
 	
 	SuperCallback subcb(cb,1,10000,10001+renddesc.get_h());
-
+	
 	Point	obr	= renddesc.get_br();
 	Point   otl = renddesc.get_tl();
-
+	
 	int		outw = renddesc.get_w();
 	int		outh = renddesc.get_h();
-
+	
 	int		inw = cairo_image_surface_get_width(cs);
 	int		inh = cairo_image_surface_get_height(cs);
 	
@@ -699,11 +718,11 @@ Layer_Bitmap::accelerated_cairorender(Context context, cairo_surface_t *out_surf
 		{
 			if(cb && !cb->amount_complete(0,100)) return false;
 			{
-				cairo_t* cr=cairo_create(out_surface); // create a cairo context with the out surface
+				cairo_save(cr);
 				cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE); // set operator to ignore destiny
 				cairo_set_source_surface(cr, cs, 0, 0); // set the source our cairosurface
 				cairo_paint(cr); // paint on the destiny
-				cairo_destroy(cr); // finaly destroy the cairo context pointer
+				cairo_restore(cr);
 			}
 			if(cb && !cb->amount_complete(100,100)) return false;
 			return true;
@@ -712,17 +731,17 @@ Layer_Bitmap::accelerated_cairorender(Context context, cairo_surface_t *out_surf
 	else // It is not the trivial case
 	{
 		// Render what is behind us...
-		if(!context.accelerated_cairorender(out_surface,quality,renddesc,&subcb))
+		if(!context.accelerated_cairorender(cr,quality,renddesc,&subcb))
 			return false;
 	}
 	
 	if(cb && !cb->amount_complete(10000,10001+renddesc.get_h())) return false;
 	
-
+	
 	// Calculate the width and height in pixels of the bitmap in the output surface
 	float wp=(br[0]-tl[0])/renddesc.get_pw();
 	float hp=(br[1]-tl[1])/renddesc.get_ph();
-	// So we need to scale the bitmap by wp/inw in horizontal and hp/inh in vertical. 
+	// So we need to scale the bitmap by wp/inw in horizontal and hp/inh in vertical.
 	float scalex=wp/inw;
 	float scaley=hp/inh;
 	// Now let's calculate the displacement of the image in the output surface.
@@ -731,30 +750,35 @@ Layer_Bitmap::accelerated_cairorender(Context context, cairo_surface_t *out_surf
 	cairo_filter_t filter;
 	switch(c)
 	{
-	case 3:	// Cubic
-		filter=CAIRO_FILTER_BEST;
-		break;
-	case 2:	// Cosine
-		filter=CAIRO_FILTER_GOOD;
-		break;
-	case 1:	// Linear
-		filter=CAIRO_FILTER_FAST;
-		break;
-	case 0:	// Nearest Neighbor
-	default:
-		filter=CAIRO_FILTER_NEAREST;
-		break;
+		case 3:	// Cubic
+			filter=CAIRO_FILTER_BEST;
+			break;
+		case 2:	// Cosine
+			filter=CAIRO_FILTER_GOOD;
+			break;
+		case 1:	// Linear
+			filter=CAIRO_FILTER_FAST;
+			break;
+		case 0:	// Nearest Neighbor
+		default:
+			filter=CAIRO_FILTER_NEAREST;
+			break;
 	}
 	// TODO: filter the image with gamma_adjust!!
-	cairo_t* cr=cairo_create(out_surface); // create a cairo context with the out surface
+	cairo_save(cr);
+	// Need to scale down to user coordinates before pass to cr
+	cairo_translate(cr, renddesc.get_tl()[0], renddesc.get_tl()[1]);
+	cairo_scale(cr, renddesc.get_pw(), renddesc.get_ph());
+	// Apply the bitmap scale and tanslate
 	cairo_translate(cr, disp[0]/renddesc.get_pw(), disp[1]/renddesc.get_ph());
 	cairo_scale(cr, scalex, scaley);
-	cairo_set_source_surface(cr, cs, 0,0);
+	// set the surface, filter, and paint
 	cairo_pattern_set_filter(cairo_get_source(cr), filter);
-	
+	cairo_set_source_surface(cr, cs, 0,0);
 	cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
+	// we don't need cs anymore
 	cairo_surface_destroy(cs);
-	cairo_destroy(cr);
+	cairo_restore(cr);
 	
 	return true;
 }
@@ -765,6 +789,9 @@ Layer_Bitmap::accelerated_cairorender(Context context, cairo_surface_t *out_surf
 Rect
 Layer_Bitmap::get_bounding_rect()const
 {
+	Point tl(param_tl.get(Point()));
+	Point br(param_br.get(Point()));
+
 	return Rect(tl,br);
 }
 
