@@ -65,12 +65,15 @@ Layer_Duplicate::Layer_Duplicate():
 {
 	LinkableValueNode* index_value_node = ValueNode_Duplicate::create(Real(3));
 	connect_dynamic_param("index", index_value_node);
+
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 Layer::Handle
-Layer_Duplicate::clone(const GUID& deriv_guid)const
+Layer_Duplicate::clone(Canvas::LooseHandle canvas, const GUID& deriv_guid)const
 {
-	Layer::Handle ret = (Layer::Handle)Layer_Composite::clone(deriv_guid);
+	Layer::Handle ret = (Layer::Handle)Layer_Composite::clone(canvas, deriv_guid);
 
 	const DynamicParamList &dpl = dynamic_param_list();
 	DynamicParamList::const_iterator iter = dpl.find("index");
@@ -79,7 +82,7 @@ Layer_Duplicate::clone(const GUID& deriv_guid)const
 	// it's not good to have two references to the same index valuenode,
 	// or nested duplications cause an infinite loop
 	if (iter != dpl.end())
-		ret->connect_dynamic_param(iter->first,iter->second->clone(deriv_guid));
+		ret->connect_dynamic_param(iter->first,iter->second->clone(canvas, deriv_guid));
 
 	return ret;
 }
@@ -87,14 +90,14 @@ Layer_Duplicate::clone(const GUID& deriv_guid)const
 bool
 Layer_Duplicate::set_param(const String &param, const ValueBase &value)
 {
-	IMPORT(index);
+	IMPORT_VALUE(param_index);
 	return Layer_Composite::set_param(param,value);
 }
 
 ValueBase
 Layer_Duplicate::get_param(const String &param)const
 {
- 	EXPORT(index);
+ 	EXPORT_VALUE(param_index);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -103,14 +106,14 @@ Layer_Duplicate::get_param(const String &param)const
 }
 
 void
-Layer_Duplicate::set_time(Context context, Time time)const
+Layer_Duplicate::set_time(IndependentContext context, Time time)const
 {
 	context.set_time(time);
 	time_cur=time;
 }
 
 void
-Layer_Duplicate::set_time(Context context, Time time, const Point &pos)const
+Layer_Duplicate::set_time(IndependentContext context, Time time, const Point &pos)const
 {
 	context.set_time(time,pos);
 	time_cur=time;
@@ -208,3 +211,53 @@ Layer_Duplicate::accelerated_render(Context context,Surface *surface,int quality
 
 	return true;
 }
+
+/////
+bool
+Layer_Duplicate::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+{
+	if(quality == 10)
+		return context.accelerated_cairorender(cr,quality,renddesc,cb);
+	
+	if(context->empty())
+	{
+		cairo_save(cr);
+		cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+		cairo_paint(cr);
+		cairo_restore(cr);
+		return true;
+	}
+	
+	SuperCallback subimagecb;
+	
+	int i = 0;
+	
+	handle<ValueNode_Duplicate> duplicate_param = get_duplicate_param();
+	if (!duplicate_param) return context.accelerated_cairorender(cr,quality,renddesc,cb);
+	
+	Color::BlendMethod blend_method(get_blend_method());
+	int steps = duplicate_param->count_steps(time_cur);
+	
+	Mutex::Lock lock(mutex);
+	duplicate_param->reset_index(time_cur);
+	cairo_save(cr);
+	do
+	{
+		subimagecb=SuperCallback(cb,i*(5000/steps),(i+1)*(5000/steps),5000);
+		// \todo can we force a re-evaluation of all the variables without changing the time twice?
+		context.set_time(time_cur+1);
+		context.set_time(time_cur);
+		cairo_push_group(cr);
+		if(!context.accelerated_cairorender(cr,quality,renddesc,&subimagecb))
+		{
+			cairo_pop_group(cr);
+			return false;
+		}
+		cairo_pop_group_to_source(cr);;
+		// \todo have a checkbox allowing use of 'behind' to reverse the order?
+		cairo_paint_with_alpha_operator(cr, get_amount(), i ? blend_method : Color::BLEND_COMPOSITE);
+		i++;
+	} while (duplicate_param->step(time_cur));
+	cairo_restore(cr);
+	return true;
+}
