@@ -49,11 +49,11 @@ class Layer_MotionBlur : public synfig::Layer_Composite
 	};
 
 private:
-	Time aperture;
-	Real subsamples_factor;
-	SubsamplingType subsampling_type;
-	Real subsample_start;
-	Real subsample_end;
+	ValueBase param_aperture;
+	ValueBase param_subsamples_factor;
+	ValueBase param_subsampling_type;
+	ValueBase param_subsample_start;
+	ValueBase param_subsample_end;
 	mutable Time time_cur;
 
 public:
@@ -61,9 +61,10 @@ public:
 	virtual bool set_param(const String & param, const synfig::ValueBase &value);
 	virtual ValueBase get_param(const String & param)const;
 	virtual Color get_color(Context context, const Point &pos)const;
-	virtual void set_time(Context context, Time time)const;
-	virtual void set_time(Context context, Time time, const Point &point)const;
+	virtual void set_time(IndependentContext context, Time time)const;
+	virtual void set_time(IndependentContext context, Time time, const Point &point)const;
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 	virtual Vocab get_param_vocab()const;
 	virtual bool reads_context()const { return true; }
 }; // END of class Layer_MotionBlur
