@@ -41,6 +41,8 @@
 #include "value.h"
 #include "valuenode.h"
 #include "canvas.h"
+#include "cairo_renddesc.h"
+
 
 #endif
 
@@ -83,20 +85,21 @@ SYNFIG_LAYER_SET_CVS_ID(Layer_PasteCanvas,"$Id$");
 /* === M E T H O D S ======================================================= */
 
 Layer_PasteCanvas::Layer_PasteCanvas():
-	origin(0,0),
-	focus(0,0),
+	param_origin      (Vector(0,0)),
+	param_focus       (Vector(0,0)),
+	param_zoom        (Real(0)),
+	param_time_offset (Time(0)),
 	depth(0),
-	zoom(0),
-	time_offset(0),
 	extra_reference(false)
 {
-	children_lock=false;
+	param_children_lock=ValueBase(bool(false));
+	param_outline_grow=ValueBase(Real(0));
+	param_curr_time=ValueBase(Time::begin());
+
 	muck_with_time_=true;
-	curr_time=Time::begin();
-	outline_grow=0.0;
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
-	set_param_static("children_lock", true);
+
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 Layer_PasteCanvas::~Layer_PasteCanvas()
@@ -140,6 +143,8 @@ Layer_PasteCanvas::get_param_vocab()const
 	ret.push_back(ParamDesc("zoom")
 		.set_local_name(_("Zoom"))
 		.set_description(_("Size of canvas"))
+		.set_origin("origin")
+		.set_exponential()
 	);
 
 	ret.push_back(ParamDesc("time_offset")
@@ -150,6 +155,7 @@ Layer_PasteCanvas::get_param_vocab()const
 	ret.push_back(ParamDesc("children_lock")
 		.set_local_name(_("Children Lock"))
 		.set_description(_("When checked prevents to select the children using the mouse click"))
+		.set_static(true)
 	);
 
 	ret.push_back(ParamDesc("focus")
@@ -188,14 +194,13 @@ Layer_PasteCanvas::get_param_vocab()const
 bool
 Layer_PasteCanvas::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT(origin);
-	IMPORT(focus);
+	IMPORT_VALUE(param_origin);
+	IMPORT_VALUE(param_focus);
 
 	// IMPORT(canvas);
 	if(param=="canvas" && value.same_type_as(Canvas::Handle()))
 	{
 		set_sub_canvas(value.get(Canvas::Handle()));
-		set_param_static(param, value.get_static());
 		return true;
 	}
 	//! \todo this introduces bug 1844764 if enabled; it was introduced in r954.
@@ -217,13 +222,13 @@ Layer_PasteCanvas::set_param(const String & param, const ValueBase &value)
 		return true;
 	}
 #else
-	IMPORT(time_offset);
+	IMPORT_VALUE(param_time_offset);
 #endif
 
-	IMPORT(children_lock);
-	IMPORT(zoom);
-	IMPORT(curr_time);
-	IMPORT(outline_grow);
+	IMPORT_VALUE(param_children_lock);
+	IMPORT_VALUE(param_zoom);
+	IMPORT_VALUE(param_outline_grow);
+	IMPORT_VALUE(param_curr_time);
 	return Layer_Composite::set_param(param,value);
 }
 
@@ -251,8 +256,6 @@ Layer_PasteCanvas::set_sub_canvas(etl::handle<synfig::Canvas> x)
 			)
 		);
 	*/
-	if(canvas)
-		bounds = ((canvas->get_context().get_full_bounding_rect() - focus) * exp(zoom) + origin + focus);
 
 	if(canvas && muck_with_time_)
 		add_child(canvas.get());
@@ -280,9 +283,9 @@ Layer_PasteCanvas::update_renddesc()
 	if(!get_canvas() || !canvas || !canvas->is_inline()) return;
 
 	canvas->rend_desc()=get_canvas()->rend_desc();
-	for (Context context = canvas->get_context(); !context->empty(); context++)
+	for (IndependentContext iter = canvas->get_independent_context(); !iter->empty(); iter++)
 	{
-		etl::handle<Layer_PasteCanvas> paste = etl::handle<Layer_PasteCanvas>::cast_dynamic(*context);
+		etl::handle<Layer_PasteCanvas> paste = etl::handle<Layer_PasteCanvas>::cast_dynamic(*iter);
 		if (paste) paste->update_renddesc();
 	}
 }
@@ -300,14 +303,18 @@ Layer_PasteCanvas::on_canvas_set()
 ValueBase
 Layer_PasteCanvas::get_param(const String& param)const
 {
-	EXPORT(origin);
-	EXPORT(focus);
-	EXPORT(canvas);
-	EXPORT(zoom);
-	EXPORT(time_offset);
-	EXPORT(children_lock);
-	EXPORT(curr_time);
-	EXPORT(outline_grow);
+	EXPORT_VALUE(param_origin);
+	EXPORT_VALUE(param_focus);
+	if (param=="canvas")
+	{
+		synfig::ValueBase ret(canvas);
+		return ret;
+	}
+	EXPORT_VALUE(param_zoom);
+	EXPORT_VALUE(param_time_offset);
+	EXPORT_VALUE(param_children_lock);
+	EXPORT_VALUE(param_curr_time);
+	EXPORT_VALUE(param_outline_grow);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -316,20 +323,16 @@ Layer_PasteCanvas::get_param(const String& param)const
 }
 
 void
-Layer_PasteCanvas::set_time(Context context, Time time)const
+Layer_PasteCanvas::set_time(IndependentContext context, Time time)const
 {
+	Time time_offset=param_time_offset.get(Time());
+
 	if(depth==MAX_DEPTH)return;depth_counter counter(depth);
-	curr_time=time;
+	param_curr_time.set(time);
 
 	context.set_time(time);
 	if(canvas)
-	{
 		canvas->set_time(time+time_offset);
-
-		bounds=(canvas->get_context().get_full_bounding_rect()-focus)*exp(zoom)+origin+focus;
-	}
-	else
-		bounds=Rect::zero();
 }
 
 synfig::Layer::Handle
@@ -337,14 +340,19 @@ Layer_PasteCanvas::hit_check(synfig::Context context, const synfig::Point &pos)c
 {
 	if(depth==MAX_DEPTH)return 0;depth_counter counter(depth);
 
+	Vector origin=param_origin.get(Vector());
+	Vector focus=param_focus.get(Vector());
+	Real zoom=param_zoom.get(Real());
+	bool children_lock=param_children_lock.get(bool(true));
+	
 	if (canvas) {
 		Point target_pos=(pos-focus-origin)/exp(zoom)+focus;
 
-		if(canvas && get_amount() && canvas->get_context().get_color(target_pos).get_a()>=0.25)
+		if(canvas && get_amount() && canvas->get_context(context).get_color(target_pos).get_a()>=0.25)
 		{
 			if(!children_lock)
 			{
-				return canvas->get_context().hit_check(target_pos);
+				return canvas->get_context(context).hit_check(target_pos);
 			}
 			return const_cast<Layer_PasteCanvas*>(this);
 		}
@@ -355,6 +363,10 @@ Layer_PasteCanvas::hit_check(synfig::Context context, const synfig::Point &pos)c
 Color
 Layer_PasteCanvas::get_color(Context context, const Point &pos)const
 {
+	Vector origin=param_origin.get(Vector());
+	Vector focus=param_focus.get(Vector());
+	Real zoom=param_zoom.get(Real());
+	
 	if(!canvas || !get_amount())
 		return context.get_color(pos);
 
@@ -362,13 +374,39 @@ Layer_PasteCanvas::get_color(Context context, const Point &pos)const
 
 	Point target_pos=(pos-focus-origin)/exp(zoom)+focus;
 
-	return Color::blend(canvas->get_context().get_color(target_pos),context.get_color(pos),get_amount(),get_blend_method());
+	return Color::blend(canvas->get_context(context).get_color(target_pos),context.get_color(pos),get_amount(),get_blend_method());
 }
 
+Rect
+Layer_PasteCanvas::get_bounding_rect_context_dependent(const ContextParams &context_params)const
+{
+	Vector origin=param_origin.get(Vector());
+	Vector focus=param_focus.get(Vector());
+	Real zoom=param_zoom.get(Real());
+	return canvas
+		 ? (canvas->get_context(context_params).get_full_bounding_rect()-focus)*exp(zoom)+origin+focus
+		 : Rect::zero();
+}
+
+Rect
+Layer_PasteCanvas::get_full_bounding_rect(Context context)const
+{
+	if(is_disabled() || Color::is_onto(get_blend_method()))
+		return context.get_full_bounding_rect();
+
+	return context.get_full_bounding_rect()|get_bounding_rect_context_dependent(context.get_params());
+}
 
 bool
 Layer_PasteCanvas::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Vector origin=param_origin.get(Vector());
+	Vector focus=param_focus.get(Vector());
+	Real zoom=param_zoom.get(Real());
+	Real outline_grow=param_outline_grow.get(Real());
+	Time time_offset=param_time_offset.get(Time());
+	Time curr_time=param_curr_time.get(Time());
+	
 	if(cb && !cb->amount_complete(0,10000)) return false;
 
 	if(depth==MAX_DEPTH)
@@ -405,9 +443,9 @@ Layer_PasteCanvas::accelerated_render(Context context,Surface *surface,int quali
 
 	if(muck_with_time_ && curr_time!=Time::begin() /*&& canvas->get_time()!=curr_time+time_offset*/)
 		canvas->set_time(curr_time+time_offset);
-
+		
 	Color::BlendMethod blend_method(get_blend_method());
-	const Rect full_bounding_rect(canvas->get_context().get_full_bounding_rect());
+	const Rect full_bounding_rect(canvas->get_context(context).get_full_bounding_rect());
 
 	bool blend_using_straight = false; // use 'straight' just for the central blit
 
@@ -543,7 +581,7 @@ Layer_PasteCanvas::accelerated_render(Context context,Surface *surface,int quali
 
 	// render the canvas to be pasted onto pastesurface
 	Surface pastesurface;
-	if(!canvas->get_context().accelerated_render(&pastesurface,quality,desc,&stagetwo))
+	if(!canvas->get_context(context).accelerated_render(&pastesurface,quality,desc,&stagetwo))
 		return false;
 
 #ifdef SYNFIG_CLIP_PASTECANVAS
@@ -564,142 +602,96 @@ Layer_PasteCanvas::accelerated_render(Context context,Surface *surface,int quali
 ///////
 
 bool
-Layer_PasteCanvas::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Layer_PasteCanvas::accelerated_cairorender(Context context,cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Vector origin=param_origin.get(Vector());
+	Vector focus=param_focus.get(Vector());
+	Real zoom=param_zoom.get(Real());
+	Real outline_grow=param_outline_grow.get(Real());
+	Time time_offset=param_time_offset.get(Time());
+	Time curr_time=param_curr_time.get(Time());
+
 	if(cb && !cb->amount_complete(0,10000)) return false;
 	
 	if(depth==MAX_DEPTH)
 		// if we are at the extent of our depth,
 		// then we should just return whatever is under us.
-		return context.accelerated_cairorender(surface,quality,renddesc,cb);
+		return context.accelerated_cairorender(cr,quality,renddesc,cb);
 	
 	depth_counter counter(depth);
 	
 	if(!canvas || !get_amount())
-		return context.accelerated_cairorender(surface,quality,renddesc,cb);
+		return context.accelerated_cairorender(cr,quality,renddesc,cb);
 	
 	SuperCallback stageone(cb,0,4500,10000);
 	SuperCallback stagetwo(cb,4500,9000,10000);
 	SuperCallback stagethree(cb,9000,9999,10000);
-	
-	RendDesc desc(renddesc);
-	Vector::value_type zoomfactor=1.0/exp(zoom);
-	desc.clear_flags();
-	desc.set_tl((desc.get_tl()-focus-origin)*zoomfactor+focus);
-	desc.set_br((desc.get_br()-focus-origin)*zoomfactor+focus);
-	desc.set_flags(RendDesc::PX_ASPECT);
-	
-	if (is_solid_color() || context->empty())
-	{
-		// clear the surface
-		cairo_t* cr=cairo_create(surface);
-		cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
-		cairo_paint(cr);
-		cairo_destroy(cr);
-	}
-	else if (!context.accelerated_cairorender(surface,quality,renddesc,&stageone))
-		return false;
+		
 	
 	Real grow_value(get_parent_canvas_grow_value());
 	canvas->set_grow_value(outline_grow+grow_value);
 	
 	if(muck_with_time_ && curr_time!=Time::begin() /*&& canvas->get_time()!=curr_time+time_offset*/)
 		canvas->set_time(curr_time+time_offset);
-	
-	Color::BlendMethod blend_method(get_blend_method());
-	const Rect full_bounding_rect(canvas->get_context().get_full_bounding_rect());
-	
-	bool blend_using_straight = false; // use 'straight' just for the central blit
-	
-	// sometimes the user changes the parameters while we're
-	// rendering, causing our pasted canvas' bounding box to shrink
-	// and no longer overlap with our tile.  if that has happened,
-	// let's just stop now - we'll be refreshing soon anyway
-	//! \todo shouldn't a mutex ensure this isn't needed?
-	// http://synfig.org/images/d/d2/Bbox-change.sifz is an example
-	// that shows this happening - open the encapsulation, select the
-	// 'shade', and toggle the 'invert' parameter quickly.
-	// Occasionally you'll see:
-	//   error: Context::accelerated_render(): Layer "shade" threw a bad_alloc exception!
-	// where the shade layer tries to allocate itself a canvas of
-	// negative proportions, due to changing bounding boxes.
-	if (!etl::intersect(desc.get_rect(), full_bounding_rect))
-	{
-		warning("%s:%d bounding box shrank while rendering?", __FILE__, __LINE__);
-		return true;
-	}
-	
-	// we have rendered what's under us, if necessary
-	if(context->empty())
-	{
-		// if there's nothing under us, and we're blending 'onto', then we've finished
-		if (Color::is_onto(blend_method)) return true;
-		
-		// there's nothing under us, so using straight blending is
-		// faster than and equivalent to using composite, but we don't
-		// want to blank the surrounding areas.
-		if (blend_method==Color::BLEND_COMPOSITE) blend_using_straight = true;
-	}
 
-	Rect area(desc.get_rect() & full_bounding_rect);
+	bool ret;
+	RendDesc workdesc(renddesc);
 	
-	Point min(area.get_min());
-	Point max(area.get_max());
-	
-	if (desc.get_tl()[0] > desc.get_br()[0]) swap(min[0], max[0]);
-	if (desc.get_tl()[1] > desc.get_br()[1]) swap(min[1], max[1]);
-	
-	const int x(floor_to_int((min[0] - desc.get_tl()[0]) / desc.get_pw()));
-	const int y(floor_to_int((min[1] - desc.get_tl()[1]) / desc.get_ph()));
-	const int w( ceil_to_int((max[0] - desc.get_tl()[0]) / desc.get_pw()) - x);
-	const int h( ceil_to_int((max[1] - desc.get_tl()[1]) / desc.get_ph()) - y);
-		
-	desc.set_subwindow(x,y,w,h);
-	
-	if(desc.get_w()==0 || desc.get_h()==0)
-	{
-		if(cb && !cb->amount_complete(10000,10000)) return false;
-		return true;
-	}
-
-	if (!etl::intersect(context.get_full_bounding_rect(),(full_bounding_rect-focus)*exp(zoom)+origin+focus))
-	{
-		// if there's no intersection between the context and our
-		// surface, and we're rendering 'onto', then we're done
-		if (Color::is_onto(blend_method) && !Color::is_straight(blend_method))
-			return true;
-	}
-	
-	// render the canvas to be pasted onto pastesurface
-	cairo_surface_t* pastesurface=cairo_surface_create_similar_image(surface, CAIRO_FORMAT_ARGB32, desc.get_w(), desc.get_h());
-	if(!canvas->get_context().accelerated_cairorender(pastesurface,quality,desc,&stagetwo))
+	// Render the background
+	ret=context.accelerated_cairorender(cr, quality, renddesc, &stagethree);
+	if(!ret)
 		return false;
 
-	cairo_t *cr = cairo_create(surface);
-	cairo_set_source_surface(cr, pastesurface, x, y);
-	cairo_surface_destroy(pastesurface);
-	cairo_paint_with_alpha_operator(cr, get_amount(), blend_using_straight ? Color::BLEND_STRAIGHT : blend_method);
-	cairo_destroy(cr);
+
+	// render the canvas to be pasted onto pastesurface
+	cairo_surface_t* pastesurface=cairo_surface_create_similar_image(cairo_get_target(cr), CAIRO_FORMAT_ARGB32, workdesc.get_w(), workdesc.get_h());
+	cairo_t* subcr=cairo_create(pastesurface);
+	// apply the transformations form the current context
+	cairo_matrix_t matrix;
+	cairo_get_matrix(cr, &matrix);
+	// apply the transformations form the (paste canvas) group layer
+	cairo_set_matrix(subcr, &matrix);
+	cairo_translate(subcr, origin[0], origin[1]);
+	cairo_translate(subcr, focus[0], focus[1]);
+	cairo_scale(subcr, exp(zoom), exp(zoom));
+	cairo_translate(subcr, -focus[0], -focus[1]);
+	// Effectively render the canvas content
+	ret=canvas->get_context(context).accelerated_cairorender(subcr, quality, workdesc, &stagetwo);
+	// we are done apply the result to the source
+	cairo_destroy(subcr);
 	
+	if(!ret)
+		return false;
+	// Let's paint the result with its alpha
+	cairo_save(cr);
+
+	cairo_status_t status;
+	status=cairo_matrix_invert(&matrix);
+	if(status) // doh! the matrix can't be inverted!
+	{
+		synfig::error("Can't invert current Cairo matrix!");
+		return false;
+	}
+	// apply the inverse of the transformation of the current context to
+	// compensate the pending transformations form cr to be applied.
+	cairo_transform(cr, &matrix);
+	cairo_set_source_surface(cr, pastesurface, 0, 0);
+	cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
+
+	cairo_restore(cr);
+	cairo_surface_destroy(pastesurface);
+
 	if(cb && !cb->amount_complete(10000,10000)) return false;
 	
 	return true;
 }
-
-
 ///////
 
 
-
-
-Rect
-Layer_PasteCanvas::get_bounding_rect()const
-{
-	return bounds;
-}
-
 void Layer_PasteCanvas::get_times_vfunc(Node::time_set &set) const
 {
+	Time time_offset=param_time_offset.get(Time());
+
 	Node::time_set tset;
 	if(canvas) tset = canvas->get_times();
 
@@ -719,24 +711,11 @@ void Layer_PasteCanvas::get_times_vfunc(Node::time_set &set) const
 }
 
 
-bool
-Layer_PasteCanvas::set_param_static(const String &param, const bool x)
-{
-	return Layer_Composite::set_param_static(param, x);
-}
-
-
-bool
-Layer_PasteCanvas::get_param_static(const String &param) const
-{
-	return Layer_Composite::get_param_static(param);
-}
-
 void
 Layer_PasteCanvas::set_render_method(Context context, RenderMethod x)
 {
 	if(canvas) // if there is a canvas pass down to it
-		canvas->get_context().set_render_method(x);
+		canvas->get_context(context).set_render_method(x);
 
 	// in any case pass it down
 	context.set_render_method(x);
