@@ -51,8 +51,8 @@ class Layer_Polygon : public Layer_Shape
 
 private:
 
-	//exported data
-	std::vector< Point > 	vector_list;
+	//! Parameter: (std::vector<Point>)
+	ValueBase param_vector_list;
 
 protected:
 
@@ -88,8 +88,6 @@ public:
 	virtual ValueBase get_param(const String & param)const;
 
 	virtual Vocab get_param_vocab()const;
-	virtual bool accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
-
 private:
 	class 		PolySpan;
 	bool render_polyspan(Surface *surface,PolySpan &polyspan)const;
