@@ -42,6 +42,8 @@
 #include "valuenode.h"
 #include "float.h"
 #include "blur.h"
+#include "cairo_renddesc.h"
+
 
 #include "curve_helper.h"
 
@@ -1122,18 +1124,18 @@ public:
 /* === M E T H O D S ======================================================= */
 
 Layer_Shape::Layer_Shape(const Real &a, const Color::BlendMethod m):
-	Layer_Composite	(a,m),
-	edge_table		(new Intersector),
-	color			(Color::black()),
-	origin			(0,0),
-	invert			(false),
-	antialias		(true),
-	blurtype		(Blur::FASTGAUSSIAN),
-	feather			(0),
-	winding_style	(WINDING_NON_ZERO),
-	bytestream		(0),
-	lastbyteop		(Primitive::NONE),
-	lastoppos		(-1)
+	Layer_Composite      (a,m),
+	edge_table	         (new Intersector),
+	param_color          (Color::black()),
+	param_origin         (Vector(0,0)),
+	param_invert         (bool(false)),
+	param_antialias      (bool(true)),
+	param_blurtype       (int(Blur::FASTGAUSSIAN)),
+	param_feather        (Real(0.0)),
+	param_winding_style	 (WINDING_NON_ZERO),
+	bytestream           (0),
+	lastbyteop           (Primitive::NONE),
+	lastoppos            (-1)
 {
 }
 
@@ -1152,31 +1154,57 @@ Layer_Shape::clear()
 bool
 Layer_Shape::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT_PLUS(color, { if (color.get_a() == 0) { if (converted_blend_) {
-					set_blend_method(Color::BLEND_ALPHA_OVER);
-					color.set_a(1); } else transparent_color_ = true; } });
-	IMPORT(origin);
-	IMPORT(invert);
-	IMPORT(antialias);
-	IMPORT_PLUS(feather, if(feather<0)feather=0;);
-	IMPORT(blurtype);
-	IMPORT(winding_style);
+	IMPORT_VALUE_PLUS(param_color,
+	{
+		Color color=param_color.get(Color());
+		if (color.get_a() == 0)
+		{
+			if (converted_blend_)
+			{
+				set_blend_method(Color::BLEND_ALPHA_OVER);
+				color.set_a(1);
+			}
+			else
+			transparent_color_ = true;
+		}
+		param_color.set(color);
+	}
+	);
+	IMPORT_VALUE(param_origin);
+	IMPORT_VALUE(param_invert);
+	IMPORT_VALUE(param_antialias);
+	IMPORT_VALUE_PLUS(param_feather,
+	{
+		Real feather=param_feather.get(Real());
+		if(feather<0)
+		{
+			feather=0;
+			param_feather.set(feather);
+		}
+	}
+	);
 
-	IMPORT_AS(origin,"offset");
+	IMPORT_VALUE(param_blurtype);
+	IMPORT_VALUE(param_winding_style);
 
+	if(param=="offset" && param_origin.get_type() == value.get_type())
+	{
+		param_origin=value;
+		return true;
+	}
 	return Layer_Composite::set_param(param,value);
 }
 
 ValueBase
 Layer_Shape::get_param(const String &param)const
 {
-	EXPORT(color);
-	EXPORT(origin);
-	EXPORT(invert);
-	EXPORT(antialias);
-	EXPORT(feather);
-	EXPORT(blurtype);
-	EXPORT(winding_style);
+	EXPORT_VALUE(param_color);
+	EXPORT_VALUE(param_origin);
+	EXPORT_VALUE(param_invert);
+	EXPORT_VALUE(param_antialias);
+	EXPORT_VALUE(param_feather);
+	EXPORT_VALUE(param_blurtype);
+	EXPORT_VALUE(param_winding_style);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -1230,6 +1258,10 @@ Layer_Shape::get_param_vocab()const
 synfig::Layer::Handle
 Layer_Shape::hit_check(synfig::Context context, const synfig::Point &p)const
 {
+	Color color=param_color.get(Color());
+	Point origin=param_origin.get(Point());
+	bool invert =param_invert.get(bool(true));
+	
 	Point pos(p-origin);
 
 	int intercepts = edge_table->intersect(pos[0],pos[1]);
@@ -1272,6 +1304,12 @@ Layer_Shape::hit_check(synfig::Context context, const synfig::Point &p)const
 Color
 Layer_Shape::get_color(Context context, const Point &p)const
 {
+	Color color=param_color.get(Color());
+	Point origin=param_origin.get(Point());
+	bool invert =param_invert.get(bool(true));
+	int blurtype=param_blurtype.get(int());
+	Real feather=param_feather.get(Real());
+
 	Point pp = p;
 
 	if(feather)
@@ -2160,6 +2198,11 @@ void Layer_Shape::curve_to_smooth(Real x2, Real y2, Real x, Real y)		//x1,y1 der
 bool Layer_Shape::render_polyspan(Surface *surface, PolySpan &polyspan,
 								Color::BlendMethod got_blend_method, Color::value_type got_amount) const
 {
+	Color color=param_color.get(Color());
+	bool invert =param_invert.get(bool(true));
+	bool antialias =param_antialias.get(bool(true));
+	WindingStyle winding_style=param_winding_style.get(WINDING_NON_ZERO);
+
 	Surface::alpha_pen p(surface->begin(),got_amount,got_blend_method);
 	PolySpan::cover_array::iterator cur_mark = polyspan.covers.begin();
 	PolySpan::cover_array::iterator end_mark = polyspan.covers.end();
@@ -2285,6 +2328,10 @@ bool Layer_Shape::render_polyspan(Surface *surface, PolySpan &polyspan,
 
 bool Layer_Shape::render_polyspan(etl::surface<float> *surface, PolySpan &polyspan) const
 {
+	bool invert =param_invert.get(bool(true));
+	bool antialias =param_antialias.get(bool(true));
+	WindingStyle winding_style=param_winding_style.get(WINDING_NON_ZERO);
+
 	etl::surface<float>::pen p(surface->begin());
 	PolySpan::cover_array::iterator cur_mark = polyspan.covers.begin();
 	PolySpan::cover_array::iterator end_mark = polyspan.covers.end();
@@ -2460,6 +2507,12 @@ bool Layer_Shape::render_polyspan(etl::surface<float> *surface, PolySpan &polysp
 bool
 Layer_Shape::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Color color=param_color.get(Color());
+	Point origin=param_origin.get(Point());
+	bool invert =param_invert.get(bool(true));
+	int blurtype=param_blurtype.get(int());
+	Real feather=param_feather.get(Real());
+
 	const unsigned int w = renddesc.get_w();
 	const unsigned int h = renddesc.get_h();
 
@@ -2620,6 +2673,183 @@ Layer_Shape::accelerated_render(Context context,Surface *surface,int quality, co
 	}
 
 }
+
+////
+bool
+Layer_Shape::accelerated_cairorender(Context context,cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+{
+	Color color=param_color.get(Color());
+	Point origin=param_origin.get(Point());
+	bool invert =param_invert.get(bool(true));
+	bool antialias =param_antialias.get(bool(true));
+	int blurtype=param_blurtype.get(int());
+	Real feather=param_feather.get(Real());
+	WindingStyle winding_style=param_winding_style.get(WINDING_NON_ZERO);
+
+	// Grab the rgba values
+	const float r(color.get_r());
+	const float g(color.get_g());
+	const float b(color.get_b());
+	const float a(color.get_a());
+	
+	// First render the context
+	if(!is_solid_color())
+		if(!context.accelerated_cairorender(cr,quality,renddesc,cb))
+		{
+			if(cb)
+				cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
+			return false;
+		}
+	
+	if(feather && quality != 10)
+	{
+		cairo_surface_t* subimage;
+		RendDesc	workdesc(renddesc);
+		
+		// Untransform the render desc
+		if(!cairo_renddesc_untransform(cr, workdesc))
+			return false;
+		
+		int halfsizex(0), halfsizey(0);
+		
+		int w=workdesc.get_w(), h=workdesc.get_h();
+		const double wpw=(workdesc.get_br()[0]-workdesc.get_tl()[0])/w;
+		const double wph=(workdesc.get_br()[1]-workdesc.get_tl()[1])/h;
+		//the expanded size = 1/2 the size in each direction rounded up
+		halfsizex = (int) (abs(feather*.5/wpw) + 3),
+		halfsizey = (int) (abs(feather*.5/wph) + 3);
+		
+		//expand by 1/2 size in each direction on either side
+		switch(blurtype)
+		{
+			case Blur::DISC:
+			case Blur::BOX:
+			case Blur::CROSS:
+			{
+				workdesc.set_subwindow(-max(1,halfsizex),-max(1,halfsizey),w+2*max(1,halfsizex),h+2*max(1,halfsizey));
+				break;
+			}
+			case Blur::FASTGAUSSIAN:
+			{
+				if(quality < 4)
+				{
+					halfsizex*=2;
+					halfsizey*=2;
+				}
+				workdesc.set_subwindow(-max(1,halfsizex),-max(1,halfsizey),w+2*max(1,halfsizex),h+2*max(1,halfsizey));
+				break;
+			}
+			case Blur::GAUSSIAN:
+			{
+#define GAUSSIAN_ADJUSTMENT		(0.05)
+				Real	pw = (Real)workdesc.get_w()/(workdesc.get_br()[0]-workdesc.get_tl()[0]);
+				Real 	ph = (Real)workdesc.get_h()/(workdesc.get_br()[1]-workdesc.get_tl()[1]);
+				
+				pw=pw*pw;
+				ph=ph*ph;
+				
+				halfsizex = (int)(abs(pw)*feather*GAUSSIAN_ADJUSTMENT+0.5);
+				halfsizey = (int)(abs(ph)*feather*GAUSSIAN_ADJUSTMENT+0.5);
+				
+				halfsizex = (halfsizex + 1)/2;
+				halfsizey = (halfsizey + 1)/2;
+				workdesc.set_subwindow( -halfsizex, -halfsizey, w+2*halfsizex, h+2*halfsizey );
+				break;
+#undef GAUSSIAN_ADJUSTMENT
+			}
+		}
+		
+		// New expanded workdesc values
+		const int ww=workdesc.get_w();
+		const int wh=workdesc.get_h();
+		const double wtlx=workdesc.get_tl()[0];
+		const double wtly=workdesc.get_tl()[1];
+		subimage=cairo_surface_create_similar(cairo_get_target(cr), CAIRO_CONTENT_COLOR_ALPHA, ww, wh);
+		cairo_t* subcr=cairo_create(subimage);
+		cairo_scale(subcr, 1/wpw, 1/wph);
+		cairo_translate(subcr, -wtlx, -wtly);
+		cairo_translate(subcr, origin[0], origin[1]);
+		switch(winding_style)
+		{
+			case WINDING_NON_ZERO:
+				cairo_set_fill_rule(subcr, CAIRO_FILL_RULE_WINDING);
+				break;
+			default:
+				cairo_set_fill_rule(subcr, CAIRO_FILL_RULE_EVEN_ODD);
+				break;
+		}
+		
+		cairo_set_source_rgba(subcr, r, g, b, a);
+		if(invert)
+			cairo_paint(subcr);
+		// Draw the shape
+		if(!antialias)
+			cairo_set_antialias(subcr, CAIRO_ANTIALIAS_NONE);
+		if(invert)
+			cairo_set_operator(subcr, CAIRO_OPERATOR_CLEAR);
+		else
+			cairo_set_operator(subcr, CAIRO_OPERATOR_OVER);
+		
+		Layer_Shape::shape_to_cairo(subcr);
+		cairo_clip(subcr);
+		cairo_paint(subcr);
+		
+		if(!feather_cairo_surface(subimage, workdesc, quality))
+		{
+			cairo_surface_destroy(subimage);
+			cairo_destroy(subcr);
+			return false;
+		}
+		cairo_destroy(subcr);
+		
+		cairo_save(cr);
+		cairo_translate(cr, wtlx, wtly);
+		cairo_scale(cr, wpw, wph);
+		cairo_set_source_surface(cr, subimage, 0, 0);
+		cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
+		cairo_restore(cr);
+		cairo_surface_destroy(subimage);
+		return true;
+	}
+	cairo_save(cr);
+	cairo_translate(cr, origin[0], origin[1]);
+	cairo_set_source_rgba(cr, r, g, b, a);
+	switch(winding_style)
+	{
+		case WINDING_NON_ZERO:
+			cairo_set_fill_rule(cr, CAIRO_FILL_RULE_WINDING);
+			break;
+		default:
+			cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
+			break;
+	}
+	if(!antialias)
+		cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
+	if(invert)
+	{
+		cairo_push_group(cr);
+		cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+		cairo_set_source_rgba(cr, r, g, b, a);
+		cairo_paint(cr);
+		Layer_Shape::shape_to_cairo(cr);
+		cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+		cairo_clip(cr);
+		cairo_paint(cr);
+		cairo_pop_group_to_source(cr);
+		cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
+	}
+	else
+	{
+		Layer_Shape::shape_to_cairo(cr);
+		cairo_clip(cr);
+		cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
+	}
+	cairo_restore(cr);
+	return true;
+}
+////
+
+
 //
 bool
 Layer_Shape::shape_to_cairo(cairo_t *cr)const
@@ -2811,10 +3041,57 @@ Layer_Shape::shape_to_cairo(cairo_t *cr)const
 }
 //
 
+bool
+Layer_Shape::feather_cairo_surface(cairo_surface_t* surface, RendDesc renddesc, int quality)const
+{
+	Color color=param_color.get(Color());
+	int blurtype=param_blurtype.get(int());
+	Real feather=param_feather.get(Real());
+
+	if(feather && quality!=10)
+	{
+		etl::surface<float>	shapesurface;
+		shapesurface.set_wh(renddesc.get_w(),renddesc.get_h());
+		shapesurface.clear();
+		
+		CairoSurface cairosurface(surface);
+		if(!cairosurface.map_cairo_image())
+		{
+			synfig::info("map cairo image failed");
+			return false;
+		}
+		// Extract the alpha values:
+		int x, y;
+		int h(renddesc.get_h()), w(renddesc.get_w());
+		float div=1.0/((float)(CairoColor::ceil));
+		for(y=0; y<h; y++)
+			for(x=0;x<w;x++)
+				shapesurface[y][x]=cairosurface[y][x].get_a()*div;
+		// Blue the alpha values
+		Blur(feather,feather,blurtype)(shapesurface, renddesc.get_br()-renddesc.get_tl(), shapesurface);
+		// repaint the cairosurface with the result
+		Color ccolor(color);
+		for(y=0; y<h; y++)
+			for(x=0;x<w;x++)
+			{
+				float a=shapesurface[y][x];
+				ccolor.set_a(a);
+				ccolor.clamped();
+				cairosurface[y][x]=CairoColor(ccolor).premult_alpha();
+			}
+		
+		cairosurface.unmap_cairo_image();
+	}
+	return true;
+}
+
+
 bool
 Layer_Shape::render_shape(Surface *surface,bool useblend,int /*quality*/,
 							const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Point origin=param_origin.get(Point());
+
 	int tmp(0);
 
 	SuperCallback	progress(cb,0,renddesc.get_h(),renddesc.get_h());
@@ -3059,6 +3336,7 @@ bool
 Layer_Shape::render_shape(etl::surface<float> *surface,int /*quality*/,
 							const RendDesc &renddesc, ProgressCallback */*cb*/)const
 {
+	Point origin=param_origin.get(Point());
 	// If our amount is set to zero, no need to render anything
 	if(!get_amount())
 		return true;
@@ -3285,6 +3563,10 @@ Layer_Shape::render_shape(etl::surface<float> *surface,int /*quality*/,
 Rect
 Layer_Shape::get_bounding_rect()const
 {
+	Point origin=param_origin.get(Point());
+	bool invert =param_invert.get(bool(true));
+	Real feather=param_feather.get(Real());
+
 	if(invert)
 		return Rect::full_plane();
 
