@@ -66,18 +66,30 @@ SYNFIG_LAYER_SET_CVS_ID(Layer_SolidColor,"$Id$");
 
 Layer_SolidColor::Layer_SolidColor():
 	Layer_Composite(1.0,Color::BLEND_COMPOSITE),
-	color(Color::black())
+	param_color(ValueBase(Color::black()))
 {
-	Layer::Vocab voc(get_param_vocab());
-	Layer::fill_static(voc);
+	SET_INTERPOLATION_DEFAULTS();
+	SET_STATIC_DEFAULTS();
 }
 
 bool
 Layer_SolidColor::set_param(const String & param, const ValueBase &value)
 {
-	IMPORT_PLUS(color, { if (color.get_a() == 0) { if (converted_blend_) {
+	IMPORT_VALUE_PLUS(param_color,
+		{
+			Color color=param_color.get(Color());
+			if (color.get_a() == 0)
+			{
+				if (converted_blend_)
+				{
 					set_blend_method(Color::BLEND_ALPHA_OVER);
-					color.set_a(1); } else transparent_color_ = true; } });
+					color.set_a(1);
+					param_color.set(color);
+				}
+				else transparent_color_ = true;
+			}
+		}
+		);
 
 	return Layer_Composite::set_param(param,value);
 }
@@ -85,7 +97,7 @@ Layer_SolidColor::set_param(const String & param, const ValueBase &value)
 ValueBase
 Layer_SolidColor::get_param(const String &param)const
 {
-	EXPORT(color);
+	EXPORT_VALUE(param_color);
 
 	EXPORT_NAME();
 	EXPORT_VERSION();
@@ -109,6 +121,7 @@ Layer_SolidColor::get_param_vocab()const
 synfig::Layer::Handle
 Layer_SolidColor::hit_check(synfig::Context context, const synfig::Point &point)const
 {
+	Color color=param_color.get(Color());
 	if(get_blend_method()==Color::BLEND_STRAIGHT && get_amount()>=0.5)
 		return const_cast<Layer_SolidColor*>(this);
 	else
@@ -123,6 +136,7 @@ Layer_SolidColor::hit_check(synfig::Context context, const synfig::Point &point)
 Color
 Layer_SolidColor::get_color(Context context, const Point &pos)const
 {
+	Color color=param_color.get(Color());
 	if(get_amount()==1.0 && get_blend_method()==Color::BLEND_STRAIGHT)
 		return color;
 	else
@@ -132,6 +146,7 @@ Layer_SolidColor::get_color(Context context, const Point &pos)const
 bool
 Layer_SolidColor::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
+	Color color=param_color.get(Color());
 	if(get_amount()==1.0 && get_blend_method()==Color::BLEND_STRAIGHT)
 	{
 		// Mark our progress as starting
@@ -175,44 +190,44 @@ Layer_SolidColor::accelerated_render(Context context,Surface *surface,int qualit
 //////
 
 bool
-Layer_SolidColor::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+Layer_SolidColor::accelerated_cairorender(Context context, cairo_t *cr, int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
-	cairo_t* cr=cairo_create(surface);
-	float	r(color.get_r()),
-			g(color.get_g()),
-			b(color.get_b()),
-			a(color.get_a());
-
-	if(get_amount()==1.0 && get_blend_method()==Color::BLEND_STRAIGHT
+	Color color=param_color.get(Color());
+	float r(color.get_r()),
+	      g(color.get_g()),
+		  b(color.get_b()),
+		  a(color.get_a());
+	
+	if((get_amount()==1.f && get_blend_method()==Color::BLEND_STRAIGHT)
 	   ||
-	   get_amount()==1.0 && color.get_a()==1.0 && get_blend_method()==Color::BLEND_COMPOSITE
+	   (get_amount()==1.f && color.get_a()==1.f && get_blend_method()==Color::BLEND_COMPOSITE)
 	   )
 	{
 		// Mark our progress as starting
 		if(cb && !cb->amount_complete(0,1000))
 			return false;
+		cairo_save(cr);
 		cairo_set_source_rgba(cr, r, g, b, a);
 		cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
 		cairo_paint(cr);
-		cairo_destroy(cr);
+		cairo_restore(cr);
 		// Mark our progress as finished
 		if(cb && !cb->amount_complete(1000,1000))
-			return false;		
+			return false;
 		return true;
 	}
 	
 	SuperCallback supercb(cb,0,9500,10000);
 	
-	if(!context.accelerated_cairorender(surface,quality,renddesc,&supercb))
-	{
-		cairo_destroy(cr);
+	if(!context.accelerated_cairorender(cr,quality,renddesc,&supercb))
 		return false;
-	}
-
+	
+	cairo_save(cr);
+	cairo_reset_clip(cr);
 	cairo_set_source_rgba(cr, r, g, b, a);
 	cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
-	cairo_destroy(cr);
-	
+	cairo_restore(cr);
+
 	// Mark our progress as finished
 	if(cb && !cb->amount_complete(10000,10000))
 		return false;
@@ -220,4 +235,4 @@ Layer_SolidColor::accelerated_cairorender(Context context,cairo_surface_t *surfa
 	return true;
 }
 
-//////
\ No newline at end of file
+//////
