@@ -145,6 +145,12 @@
 //!Register an Importer class in the book of importers by one file extension string
 #define IMPORTER_EXT(x,y) synfig::Importer::book()[synfig::String(y)]=x::create;
 
+//! Register an CairoImporter class in the book of importers by the default extension
+#define CAIROIMPORTER(x) synfig::CairoImporter::book()[synfig::String(x::ext__)]=x::create;
+
+//!Register an Importer class in the book of importers by one file extension string
+#define CAIROIMPORTER_EXT(x,y) synfig::CairoImporter::book()[synfig::String(y)]=x::create;
+
 //! Marks the end of the importers in the module's inventory
 #define END_IMPORTERS }
 
