@@ -30,6 +30,7 @@
 #include "string.h"
 #include "real.h"
 #include "color.h"
+#include "interpolation.h"
 #include <list>
 
 /* === M A C R O S ========================================================= */
@@ -78,11 +79,14 @@ private:
    	String connect_;
    	String box_;
 	Real scalar_;			//! Scalar value for visual editing
+	bool exponential_;		//! Allow range from -inf to inf for visual editing
    	bool critical_;
 	bool hidden_;
 	bool invisible_duck_;
 	bool is_distance_;
 	bool animation_only_;
+	bool static_;
+	Interpolation interpolation_;
 
 	std::list<EnumData> enum_list_;
 
@@ -96,18 +100,18 @@ public:
 		name_			(a),
 		local_name_		(a),
 		scalar_			(1.0),
+		exponential_		(false),
 		critical_		(true),
 		hidden_			(false),
 		invisible_duck_	(false),
 		is_distance_	(false),
-		animation_only_	(false)
+		animation_only_	(false),
+		static_			(false),
+		interpolation_	(INTERPOLATION_UNDEFINED)
 	{ }
 
    	ParamDesc(const ValueBase&, const String &a);
 
-   	ParamDesc(synfig::Color::BlendMethod, const String &a);
-
-
 	/*
  --	** -- M E M B E R   F U N C T I O N S -------------------------------------
 	*/
@@ -156,6 +160,10 @@ public:
    	/*! This value determines how the value is to be presented
    	**	to the user when editing visually. */
    	ParamDesc &set_scalar(const Real &n) { scalar_=n; return *this; }
+	
+	//! Sets if the parameter value should be exposed for visual editing using the exponential function
+   	/*!	Such representation allows to set the Real values in the range from \c -inf to \c inf . */
+	ParamDesc &set_exponential(bool x=true) { exponential_=x; return *this; }
 
    	//!	Marks the parameter as not necessary for saving or copying
    	ParamDesc &not_critical() { critical_=false; return *this; }
@@ -202,14 +210,22 @@ public:
 
    	//! Returns the scalar value for the parameter. Used for visual editing.
    	const Real &get_scalar()const { return scalar_; }
+	
+	//! Tells if the value should be exposed for editing using the exponential function
+   	/*!	Such representation allows to set the Real values in the range from \c -inf to \c inf . */
+	bool get_exponential()const {return exponential_; }
 
    	//! Returns \c true if the layer is critical, \c false otherwise.
    	bool get_critical()const { return critical_; }
 
    	//! Returns \c true if the layer is hidden, \c false otherwise.
    	bool get_hidden()const { return hidden_; }
+	
+	bool get_static()const { return static_; }
+	ParamDesc &set_static(bool s) { static_=s; return *this; }
 
-
+	Interpolation get_interpolation()const { return interpolation_; }
+	ParamDesc &set_interpolation(Interpolation i) { interpolation_=i; return *this; }
 
    	ParamDesc& set_is_distance(bool x=true) { is_distance_=x; return *this;}
    	bool get_is_distance()const { return is_distance_; }
