@@ -40,9 +40,13 @@
 #include "valuenode.h"
 #include "valuenode_animated.h"
 #include "valuenode_const.h"
+#include "valuenode_linear.h"
+#include "valuenode_staticlist.h"
 #include "valuenode_dynamiclist.h"
 #include "valuenode_reference.h"
+#include "valuenode_subtract.h"
 #include "valuenode_bline.h"
+#include "valuenode_bone.h"
 #include "valuenode_wplist.h"
 #include "valuenode_dilist.h"
 #include "dashitem.h"
@@ -85,6 +89,8 @@ int valuenode_too_new_count;
 
 xmlpp::Element* encode_canvas(xmlpp::Element* root,Canvas::ConstHandle canvas);
 xmlpp::Element* encode_value_node(xmlpp::Element* root,ValueNode::ConstHandle value_node,Canvas::ConstHandle canvas);
+xmlpp::Element* encode_value_node_bone(xmlpp::Element* root,ValueNode::ConstHandle value_node,Canvas::ConstHandle canvas);
+xmlpp::Element* encode_value_node_bone_id(xmlpp::Element* root,ValueNode::ConstHandle value_node,Canvas::ConstHandle canvas);
 
 xmlpp::Element* encode_keyframe(xmlpp::Element* root,const Keyframe &kf, float fps)
 {
@@ -96,6 +102,37 @@ xmlpp::Element* encode_keyframe(xmlpp::Element* root,const Keyframe &kf, float f
 	return root;
 }
 
+xmlpp::Element* encode_interpolation(xmlpp::Element* root,Interpolation value,String attribute)
+{
+	if (value!=INTERPOLATION_UNDEFINED)
+	{
+		switch(value)
+		{
+		case INTERPOLATION_HALT:
+			root->set_attribute(attribute,"halt");
+			break;
+		case INTERPOLATION_LINEAR:
+			root->set_attribute(attribute,"linear");
+			break;
+		case INTERPOLATION_MANUAL:
+			root->set_attribute(attribute,"manual");
+			break;
+		case INTERPOLATION_CONSTANT:
+			root->set_attribute(attribute,"constant");
+			break;
+		case INTERPOLATION_TCB:
+			root->set_attribute(attribute,"auto");
+			break;
+		case INTERPOLATION_CLAMPED:
+			root->set_attribute(attribute,"clamped");
+			break;
+		default:
+			error("Unknown waypoint type for \""+attribute+"\" attribute");
+		}
+	}
+	return root;
+}
+
 xmlpp::Element* encode_static(xmlpp::Element* root,bool s)
 {
 	if(s)
@@ -104,82 +141,73 @@ xmlpp::Element* encode_static(xmlpp::Element* root,bool s)
 }
 
 
-xmlpp::Element* encode_real(xmlpp::Element* root,Real v,bool s=false)
+xmlpp::Element* encode_real(xmlpp::Element* root,Real v)
 {
 	root->set_name("real");
 	root->set_attribute("value",strprintf(VECTOR_VALUE_TYPE_FORMAT,v));
-	encode_static(root, s);
 	return root;
 }
 
-xmlpp::Element* encode_time(xmlpp::Element* root,Time t,bool s=false)
+xmlpp::Element* encode_time(xmlpp::Element* root,Time t)
 {
 	root->set_name("time");
 	root->set_attribute("value",t.get_string());
-	encode_static(root, s);
 	return root;
 }
 
-xmlpp::Element* encode_integer(xmlpp::Element* root,int i,bool s=false)
+xmlpp::Element* encode_integer(xmlpp::Element* root,int i)
 {
 	root->set_name("integer");
 	root->set_attribute("value",strprintf("%i",i));
-	encode_static(root, s);
 	return root;
 }
 
-xmlpp::Element* encode_bool(xmlpp::Element* root, bool b,bool s=false)
+xmlpp::Element* encode_bool(xmlpp::Element* root, bool b)
 {
 	root->set_name("bool");
 	root->set_attribute("value",b?"true":"false");
-	encode_static(root, s);
 	return root;
 }
 
-xmlpp::Element* encode_string(xmlpp::Element* root,const String &str,bool s=false)
+xmlpp::Element* encode_string(xmlpp::Element* root,const String &str)
 {
 	root->set_name("string");
 	root->set_child_text(str);
-	encode_static(root, s);
 	return root;
 }
 
-xmlpp::Element* encode_vector(xmlpp::Element* root,Vector vect,bool s=false)
+xmlpp::Element* encode_vector(xmlpp::Element* root,Vector vect)
 {
 	root->set_name("vector");
 	root->add_child("x")->set_child_text(strprintf(VECTOR_VALUE_TYPE_FORMAT,(float)vect[0]));
 	root->add_child("y")->set_child_text(strprintf(VECTOR_VALUE_TYPE_FORMAT,(float)vect[1]));
-	encode_static(root, s);
 	return root;
 }
 
-xmlpp::Element* encode_color(xmlpp::Element* root,Color color,bool s=false)
+xmlpp::Element* encode_color(xmlpp::Element* root,Color color)
 {
 	root->set_name("color");
 	root->add_child("r")->set_child_text(strprintf(COLOR_VALUE_TYPE_FORMAT,(float)color.get_r()));
 	root->add_child("g")->set_child_text(strprintf(COLOR_VALUE_TYPE_FORMAT,(float)color.get_g()));
 	root->add_child("b")->set_child_text(strprintf(COLOR_VALUE_TYPE_FORMAT,(float)color.get_b()));
 	root->add_child("a")->set_child_text(strprintf(COLOR_VALUE_TYPE_FORMAT,(float)color.get_a()));
-	encode_static(root, s);
 	return root;
 }
 
-xmlpp::Element* encode_angle(xmlpp::Element* root,Angle theta,bool s=false)
+xmlpp::Element* encode_angle(xmlpp::Element* root,Angle theta)
 {
 	root->set_name("angle");
 	root->set_attribute("value",strprintf("%f",(float)Angle::deg(theta).get()));
-	encode_static(root, s);
 	return root;
 }
 
-xmlpp::Element* encode_segment(xmlpp::Element* root,Segment seg,bool s=false)
+xmlpp::Element* encode_segment(xmlpp::Element* root,Segment seg)
 {
 	root->set_name("segment");
 	encode_vector(root->add_child("p1")->add_child("vector"),seg.p1);
 	encode_vector(root->add_child("t1")->add_child("vector"),seg.t1);
 	encode_vector(root->add_child("p2")->add_child("vector"),seg.p2);
 	encode_vector(root->add_child("t2")->add_child("vector"),seg.t2);
-	encode_static(root, s);
 	return root;
 }
 
@@ -218,10 +246,9 @@ xmlpp::Element* encode_dash_item(xmlpp::Element* root, DashItem dash_item)
 	return root;
 }
 
-xmlpp::Element* encode_gradient(xmlpp::Element* root,Gradient x,bool s=false)
+xmlpp::Element* encode_gradient(xmlpp::Element* root,Gradient x)
 {
 	root->set_name("gradient");
-	encode_static(root, s);
 	Gradient::const_iterator iter;
 	x.sort();
 	for(iter=x.begin();iter!=x.end();iter++)
@@ -250,26 +277,54 @@ xmlpp::Element* encode_list(xmlpp::Element* root,std::list<ValueBase> list, Canv
 
 xmlpp::Element* encode_value(xmlpp::Element* root,const ValueBase &data,Canvas::ConstHandle canvas)
 {
+	if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode_value (type %s)\n", __FILE__, __LINE__, ValueBase::type_name(data.get_type()).c_str());
 	switch(data.get_type())
 	{
 	case ValueBase::TYPE_REAL:
-		return encode_real(root,data.get(Real()), data.get_static());
+		encode_real(root,data.get(Real()));
+		encode_static(root, data.get_static());
+		encode_interpolation(root, data.get_interpolation(), "interpolation");
+		return root;
 	case ValueBase::TYPE_TIME:
-		return encode_time(root,data.get(Time()), data.get_static());
+		encode_time(root,data.get(Time()));
+		encode_static(root, data.get_static());
+		encode_interpolation(root, data.get_interpolation(), "interpolation");
+		return root;
 	case ValueBase::TYPE_INTEGER:
-		return encode_integer(root,data.get(int()), data.get_static());
+		encode_integer(root,data.get(int()));
+		encode_static(root, data.get_static());
+		encode_interpolation(root, data.get_interpolation(), "interpolation");
+		return root;
 	case ValueBase::TYPE_COLOR:
-		return encode_color(root,data.get(Color()), data.get_static());
+		encode_color(root,data.get(Color()));
+		encode_static(root, data.get_static());
+		encode_interpolation(root, data.get_interpolation(), "interpolation");
+		return root;
 	case ValueBase::TYPE_VECTOR:
-		return encode_vector(root,data.get(Vector()), data.get_static());
+		encode_vector(root,data.get(Vector()));
+		encode_static(root, data.get_static());
+		encode_interpolation(root, data.get_interpolation(), "interpolation");
+		return root;
 	case ValueBase::TYPE_ANGLE:
-		return encode_angle(root,data.get(Angle()), data.get_static());
+		encode_angle(root,data.get(Angle()));
+		encode_static(root, data.get_static());
+		encode_interpolation(root, data.get_interpolation(), "interpolation");
+		return root;
 	case ValueBase::TYPE_BOOL:
-		return encode_bool(root,data.get(bool()), data.get_static());
+		encode_bool(root,data.get(bool()));
+		encode_static(root, data.get_static());
+		encode_interpolation(root, data.get_interpolation(), "interpolation");
+		return root;
 	case ValueBase::TYPE_STRING:
-		return encode_string(root,data.get(String()), data.get_static());
+		encode_string(root,data.get(String()));
+		encode_static(root, data.get_static());
+		encode_interpolation(root, data.get_interpolation(), "interpolation");
+		return root;
 	case ValueBase::TYPE_SEGMENT:
-		return encode_segment(root,data.get(Segment()), data.get_static());
+		encode_segment(root,data.get(Segment()));
+		encode_static(root, data.get_static());
+		encode_interpolation(root, data.get_interpolation(), "interpolation");
+		return root;
 	case ValueBase::TYPE_BLINEPOINT:
 		return encode_bline_point(root,data.get(BLinePoint()));
 	case ValueBase::TYPE_WIDTHPOINT:
@@ -277,11 +332,25 @@ xmlpp::Element* encode_value(xmlpp::Element* root,const ValueBase &data,Canvas::
 	case ValueBase::TYPE_DASHITEM:
 		return encode_dash_item(root,data.get(DashItem()));
 	case ValueBase::TYPE_GRADIENT:
-		return encode_gradient(root,data.get(Gradient()), data.get_static());
+		encode_gradient(root,data.get(Gradient()));
+		encode_static(root, data.get_static());
+		encode_interpolation(root, data.get_interpolation(), "interpolation");
+		return root;
 	case ValueBase::TYPE_LIST:
 		return encode_list(root,data,canvas);
 	case ValueBase::TYPE_CANVAS:
 		return encode_canvas(root,data.get(Canvas::Handle()).get());
+		encode_static(root, data.get_static());
+	case ValueBase::TYPE_VALUENODE_BONE:
+		if (!canvas)
+		{
+			printf("%s:%d ------------------------------------------------------------------------\n", __FILE__, __LINE__);
+			printf("%s:%d zero canvas - please fix - report\n", __FILE__, __LINE__);
+			printf("%s:%d ------------------------------------------------------------------------\n", __FILE__, __LINE__);
+		}
+		root = encode_value_node_bone_id(root,data.get(ValueNode_Bone::Handle()).get(),canvas);
+		root->set_name("bone_valuenode");
+		return root;
 	case ValueBase::TYPE_NIL:
 		synfig::error("Encountered NIL ValueBase");
 		root->set_name("nil");
@@ -302,7 +371,9 @@ xmlpp::Element* encode_animated(xmlpp::Element* root,ValueNode_Animated::ConstHa
 
 	const ValueNode_Animated::WaypointList &waypoint_list=value_node->waypoint_list();
 	ValueNode_Animated::WaypointList::const_iterator iter;
-
+	
+	encode_interpolation(root, value_node->get_interpolation(), "interpolation");
+	
 	for(iter=waypoint_list.begin();iter!=waypoint_list.end();++iter)
 	{
 		xmlpp::Element *waypoint_node=root->add_child("waypoint");
@@ -312,7 +383,8 @@ xmlpp::Element* encode_animated(xmlpp::Element* root,ValueNode_Animated::ConstHa
 			waypoint_node->set_attribute("use",iter->get_value_node()->get_relative_id(canvas));
 		else {
 			ValueNode::ConstHandle value_node = iter->get_value_node();
-			if(ValueNode_Const::ConstHandle::cast_dynamic(value_node)) {
+			if(ValueNode_Const::ConstHandle::cast_dynamic(value_node))
+			{
 				const ValueBase data = ValueNode_Const::ConstHandle::cast_dynamic(value_node)->get_value();
 				if (data.get_type() == ValueBase::TYPE_CANVAS)
 					waypoint_node->set_attribute("use",data.get(Canvas::Handle()).get()->get_relative_id(canvas));
@@ -322,54 +394,16 @@ xmlpp::Element* encode_animated(xmlpp::Element* root,ValueNode_Animated::ConstHa
 			else
 				encode_value_node(waypoint_node->add_child("value_node"),iter->get_value_node(),canvas);
 		}
-
-		switch(iter->get_before())
-		{
-		case INTERPOLATION_HALT:
-			waypoint_node->set_attribute("before","halt");
-			break;
-		case INTERPOLATION_LINEAR:
-			waypoint_node->set_attribute("before","linear");
-			break;
-		case INTERPOLATION_MANUAL:
-			waypoint_node->set_attribute("before","manual");
-			break;
-		case INTERPOLATION_CONSTANT:
-			waypoint_node->set_attribute("before","constant");
-			break;
-		case INTERPOLATION_TCB:
-			waypoint_node->set_attribute("before","auto");
-			break;
-		case INTERPOLATION_CLAMPED:
-			waypoint_node->set_attribute("before","clamped");
-			break;
-		default:
+		
+		if (iter->get_before()!=INTERPOLATION_UNDEFINED)
+			encode_interpolation(waypoint_node,iter->get_before(),"before");
+		else
 			error("Unknown waypoint type for \"before\" attribute");
-		}
 
-		switch(iter->get_after())
-		{
-		case INTERPOLATION_HALT:
-			waypoint_node->set_attribute("after","halt");
-			break;
-		case INTERPOLATION_LINEAR:
-			waypoint_node->set_attribute("after","linear");
-			break;
-		case INTERPOLATION_MANUAL:
-			waypoint_node->set_attribute("after","manual");
-			break;
-		case INTERPOLATION_CONSTANT:
-			waypoint_node->set_attribute("after","constant");
-			break;
-		case INTERPOLATION_TCB:
-			waypoint_node->set_attribute("after","auto");
-			break;
-		case INTERPOLATION_CLAMPED:
-			waypoint_node->set_attribute("after","clamped");
-			break;
-		default:
+		if (iter->get_after()!=INTERPOLATION_UNDEFINED)
+			encode_interpolation(waypoint_node,iter->get_after(),"after");
+		else
 			error("Unknown waypoint type for \"after\" attribute");
-		}
 
 		if(iter->get_tension()!=0.0)
 			waypoint_node->set_attribute("tension",strprintf("%f",iter->get_tension()));
@@ -385,6 +419,73 @@ xmlpp::Element* encode_animated(xmlpp::Element* root,ValueNode_Animated::ConstHa
 	return root;
 }
 
+
+xmlpp::Element* encode_subtract(xmlpp::Element* root,ValueNode_Subtract::ConstHandle value_node,Canvas::ConstHandle canvas=0)
+{
+	assert(value_node);
+	root->set_name("subtract");
+
+	ValueNode::ConstHandle lhs=value_node->get_lhs();
+	ValueNode::ConstHandle rhs=value_node->get_rhs();
+	ValueNode::ConstHandle scalar=value_node->get_scalar();
+
+	assert(lhs);
+	assert(rhs);
+
+	root->set_attribute("type",ValueBase::type_name(value_node->get_type()));
+
+	if(lhs==rhs)
+		warning("LHS is equal to RHS, this <subtract> will always be zero!");
+
+	//if(value_node->get_scalar()!=1)
+	//	root->set_attribute("scalar",strprintf(VECTOR_VALUE_TYPE_FORMAT,value_node->get_scalar()));
+
+	if(!scalar->get_id().empty())
+		root->set_attribute("scalar",scalar->get_relative_id(canvas));
+	else
+		encode_value_node(root->add_child("scalar")->add_child("value_node"),scalar,canvas);
+
+	if(!lhs->get_id().empty())
+		root->set_attribute("lhs",lhs->get_relative_id(canvas));
+	else
+		encode_value_node(root->add_child("lhs")->add_child("value_node"),lhs,canvas);
+
+	if(!rhs->get_id().empty())
+		root->set_attribute("rhs",rhs->get_relative_id(canvas));
+	else
+		encode_value_node(root->add_child("rhs")->add_child("value_node"),rhs,canvas);
+
+	return root;
+}
+
+xmlpp::Element* encode_static_list(xmlpp::Element* root,ValueNode_StaticList::ConstHandle value_node,Canvas::ConstHandle canvas=0)
+{
+	if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode_static_list %s\n", __FILE__, __LINE__, value_node->get_string().c_str());
+	assert(value_node);
+
+	root->set_name(value_node->get_name());
+
+	root->set_attribute("type",ValueBase::type_name(value_node->get_contained_type()));
+
+	vector<ValueNode::RHandle>::const_iterator iter;
+
+	for(iter=value_node->list.begin();iter!=value_node->list.end();++iter)
+	{
+		xmlpp::Element	*entry_node=root->add_child("entry");
+		assert(*iter);
+		if(!(*iter)->get_id().empty())
+			entry_node->set_attribute("use",(*iter)->get_relative_id(canvas));
+		else
+		{
+			if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode entry %s\n", __FILE__, __LINE__, (*iter)->get_string().c_str());
+			encode_value_node(entry_node->add_child("value_node"),*iter,canvas);
+		}
+	}
+
+	if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode_static_list %s done\n", __FILE__, __LINE__, value_node->get_string().c_str());
+	return root;
+}
+
 xmlpp::Element* encode_dynamic_list(xmlpp::Element* root,ValueNode_DynamicList::ConstHandle value_node,Canvas::ConstHandle canvas=0)
 {
 	assert(value_node);
@@ -484,6 +585,7 @@ xmlpp::Element* encode_dynamic_list(xmlpp::Element* root,ValueNode_DynamicList::
 // Generic linkable data node entry
 xmlpp::Element* encode_linkable_value_node(xmlpp::Element* root,LinkableValueNode::ConstHandle value_node,Canvas::ConstHandle canvas=0)
 {
+	if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode_linkable_value_node %s\n", __FILE__, __LINE__, value_node->get_string().c_str());
 	assert(value_node);
 
 	String name(value_node->get_name());
@@ -510,29 +612,58 @@ xmlpp::Element* encode_linkable_value_node(xmlpp::Element* root,LinkableValueNod
 	synfig::ParamVocab::iterator iter(child_vocab.begin());
 	for(i=0;i<value_node->link_count();i++, iter++)
 	{
+		// printf("saving link %d : %s\n", i, value_node->link_local_name(i).c_str());
 		ValueNode::ConstHandle link=value_node->get_link(i).constant();
 		if(!link)
 			throw runtime_error("Bad link");
 		if(link->is_exported())
 			root->set_attribute(value_node->link_name(i),link->get_relative_id(canvas));
 		else if(iter->get_critical())
+		{
+			if (name == "bone" && value_node->link_name(i) == "parent")
+			{
+				if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d saving bone's parent\n", __FILE__, __LINE__);
+			}
 			encode_value_node(root->add_child(value_node->link_name(i))->add_child("value_node"),link,canvas);
+		}
 	}
 
+	if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode_linkable_value_node %s done\n", __FILE__, __LINE__, value_node->get_string().c_str());
 	return root;
 }
 
 xmlpp::Element* encode_value_node(xmlpp::Element* root,ValueNode::ConstHandle value_node,Canvas::ConstHandle canvas)
 {
 	assert(value_node);
+	if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode_value_node %s %s\n", __FILE__, __LINE__, value_node->get_string().c_str(), value_node->get_guid().get_string().c_str());
 
+	if(value_node->rcount()>1)
+		root->set_attribute("guid",(value_node->get_guid()^canvas->get_root()->get_guid()).get_string());
+
+	if(ValueNode_Bone::ConstHandle::cast_dynamic(value_node))
+	{
+		if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d shortcutting for valuenode_bone\n", __FILE__, __LINE__);
+		encode_value_node_bone_id(root,ValueNode_Bone::ConstHandle::cast_dynamic(value_node),canvas);
+	}
+	else
 	if(ValueNode_Animated::ConstHandle::cast_dynamic(value_node))
 		encode_animated(root,ValueNode_Animated::ConstHandle::cast_dynamic(value_node),canvas);
 	else
+	if(ValueNode_Subtract::ConstHandle::cast_dynamic(value_node))
+		encode_subtract(root,ValueNode_Subtract::ConstHandle::cast_dynamic(value_node),canvas);
+	else
+	if(ValueNode_StaticList::ConstHandle::cast_dynamic(value_node))
+		encode_static_list(root,ValueNode_StaticList::ConstHandle::cast_dynamic(value_node),canvas);
+	else
 	if(ValueNode_DynamicList::ConstHandle::cast_dynamic(value_node))
+	{
 		encode_dynamic_list(root,ValueNode_DynamicList::ConstHandle::cast_dynamic(value_node),canvas);
+	}
+	// if it's a ValueNode_Const
 	else if(ValueNode_Const::ConstHandle::cast_dynamic(value_node))
 	{
+		if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d got ValueNode_Const encoding value\n", __FILE__, __LINE__);
+		// encode its get_value()
 		encode_value(root,ValueNode_Const::ConstHandle::cast_dynamic(value_node)->get_value(),canvas);
 	}
 	else
@@ -549,9 +680,66 @@ xmlpp::Element* encode_value_node(xmlpp::Element* root,ValueNode::ConstHandle va
 	if(!value_node->get_id().empty())
 		root->set_attribute("id",value_node->get_id());
 
-	if(value_node->rcount()>1)
+//	if(ValueNode_Bone::ConstHandle::cast_dynamic(value_node))
+//		root->set_attribute("guid",value_node->get_guid().get_string());
+
+	if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode_value_node %s done\n", __FILE__, __LINE__, value_node->get_string().c_str());
+	return root;
+}
+
+xmlpp::Element* encode_value_node_bone(xmlpp::Element* root,ValueNode::ConstHandle value_node,Canvas::ConstHandle canvas)
+{
+	assert(value_node);
+	if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode_value_node_bone %s %s\n", __FILE__, __LINE__, value_node->get_string().c_str(), value_node->get_guid().get_string().c_str());
+
+	if(ValueNode_Bone::ConstHandle::cast_dynamic(value_node))
+		encode_linkable_value_node(root,LinkableValueNode::ConstHandle::cast_dynamic(value_node),canvas);
+	else
+	{
+		error(_("Unknown ValueNode Type (%s), cannot create an XML representation"),value_node->get_local_name().c_str());
+		assert(0);
+		root->set_name("nil");
+	}
+
+	assert(root);
+
+	if(!value_node->get_id().empty())
+		root->set_attribute("id",value_node->get_id());
+
+	if(ValueNode_Bone::ConstHandle::cast_dynamic(value_node))
 		root->set_attribute("guid",(value_node->get_guid()^canvas->get_root()->get_guid()).get_string());
 
+	if(value_node->rcount()>1)
+	{
+		// ~/notes/synfig/crash-when-saving.txt is an example of the execution reaching this line
+		printf("%s:%d xxx value_node->rcount() = %d\n", __FILE__, __LINE__, value_node->rcount());
+		root->set_attribute("guid",(value_node->get_guid()^canvas->get_root()->get_guid()).get_string());
+	}
+
+	if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode_value_node %s done\n", __FILE__, __LINE__, value_node->get_string().c_str());
+	return root;
+}
+
+xmlpp::Element* encode_value_node_bone_id(xmlpp::Element* root,ValueNode::ConstHandle value_node,Canvas::ConstHandle canvas)
+{
+	root->set_name("bone");
+	root->set_attribute("type",ValueBase::type_name(ValueBase::TYPE_BONE));
+	if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode_value_node_bone_id %s %s\n", __FILE__, __LINE__, value_node->get_string().c_str(), value_node->get_guid().get_string().c_str());
+	if(!value_node->get_id().empty())
+		root->set_attribute("id",value_node->get_id());
+
+	if(ValueNode_Bone::ConstHandle::cast_dynamic(value_node))
+	{
+		if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d bone guid case 1 guid %s\n", __FILE__, __LINE__, value_node->get_guid().get_string().c_str());
+		root->set_attribute("guid",(value_node->get_guid()^canvas->get_root()->get_guid()).get_string());
+	}
+
+	if(value_node->rcount()>1)
+	{
+		printf("%s:%d this happens too\n", __FILE__, __LINE__);
+		root->set_attribute("guid",(value_node->get_guid()^canvas->get_root()->get_guid()).get_string());
+	}
+
 	return root;
 }
 
@@ -561,6 +749,7 @@ xmlpp::Element* encode_layer(xmlpp::Element* root,Layer::ConstHandle layer)
 
 	root->set_attribute("type",layer->get_name());
 	root->set_attribute("active",layer->active()?"true":"false");
+	root->set_attribute("exclude_from_rendering",layer->get_exclude_from_rendering()?"true":"false");
 
 	if(!layer->get_version().empty())
 		root->set_attribute("version",layer->get_version());
@@ -712,12 +901,32 @@ xmlpp::Element* encode_canvas(xmlpp::Element* root,Canvas::ConstHandle canvas)
 			encode_keyframe(root->add_child("keyframe"),*iter,canvas->rend_desc().get_frame_rate());
 	}
 
+	// Output the <bones> section
+	if((!canvas->is_inline() && !ValueNode_Bone::get_bone_map(canvas).empty()))
+	{
+		xmlpp::Element *node=root->add_child("bones");
+
+		encode_value_node_bone(node->add_child("value_node"),ValueNode_Bone::get_root_bone(),canvas);
+
+		ValueNode_Bone::BoneList bone_list(ValueNode_Bone::get_ordered_bones(canvas));
+		for(ValueNode_Bone::BoneList::iterator iter=bone_list.begin();iter!=bone_list.end();++iter)
+		{
+			ValueNode_Bone::Handle bone(*iter);
+			encode_value_node_bone(node->add_child("value_node"),bone,canvas);
+		}
+	}
+
 	// Output the <defs> section
-	//! Check where the parentheses should really go - around the && or the ||?
+
+	//! \todo check where the parentheses should really go - around the && or the ||?
+	// I guess it should be the other way - but then, why would an inline canvas have either exported valuenode or child canvases?  it shouldn't, right?
+	// (not an inline canvas)   (has some exported valuenodes)         (has some child canvases)
+
 	//! If children is not empty (there are exported canvases in the current canvas)
 	//! they must be listed in the defs section regardless the result of check the
 	//! Value Node list (exported value nodes in the canvas) and if the canvas is
 	//! in line or not. Inline canvases cannot have exported canvases inside.
+
 	if((!canvas->is_inline() && !canvas->value_node_list().empty()) || !canvas->children().empty())
 	{
 		xmlpp::Element *node=root->add_child("defs");
@@ -729,7 +938,7 @@ xmlpp::Element* encode_canvas(xmlpp::Element* root,Canvas::ConstHandle canvas)
 			if(handle<ValueNode_Const>::cast_dynamic(*iter))
 			{
 				ValueNode_Const::Handle value_node(ValueNode_Const::Handle::cast_dynamic(*iter));
-				reinterpret_cast<xmlpp::Element*>(encode_value(node->add_child("value"),value_node->get_value()))->set_attribute("id",value_node->get_id());
+				reinterpret_cast<xmlpp::Element*>(encode_value(node->add_child("value"),value_node->get_value(),canvas))->set_attribute("id",value_node->get_id());
 				continue;
 			}
 			encode_value_node(node->add_child("value_node"),*iter,canvas);
