@@ -285,6 +285,8 @@ synfig::Target_Tile::render(ProgressCallback *cb)
 	frame_start=desc.get_frame_start();
 	frame_end=desc.get_frame_end();
 
+	ContextParams context_params(desc.get_render_excluded_contexts());
+
 	// Calculate the number of frames
 	total_frames=frame_end-frame_start+1;
 	if(total_frames<=0)total_frames=1;
@@ -309,12 +311,12 @@ synfig::Target_Tile::render(ProgressCallback *cb)
 					return false;
 				Context context;
 				// pass the Render Method to the context
-				context=canvas->get_context();
+				context=canvas->get_context(context_params);
 				context.set_render_method(SOFTWARE);
 
 				// Set the time that we wish to render
 				//if(!get_avoid_time_sync() || canvas->get_time()!=t)
-				// Why the above line is commented here and not in TargetScaline?
+				// Why the above line was commented here and not in TargetScaline?
 					canvas->set_time(t);
 
 	#ifdef SYNFIG_OPTIMIZE_LAYER_TREE
@@ -323,13 +325,13 @@ synfig::Target_Tile::render(ProgressCallback *cb)
 				{
 					op_canvas = Canvas::create();
 					op_canvas->set_file_name(canvas->get_file_name());
-					optimize_layers(canvas->get_time(), canvas->get_context(), op_canvas);
-					context=op_canvas->get_context();
+					optimize_layers(canvas->get_time(), canvas->get_context(context_params), op_canvas);
+					context=op_canvas->get_context(context_params);
 				}
 				else
-					context=canvas->get_context();
+					context=canvas->get_context(context_params);
 	#else
-				context=canvas->get_context();
+				context=canvas->get_context(context_params);
 	#endif
 
 	/*
@@ -339,13 +341,13 @@ synfig::Target_Tile::render(ProgressCallback *cb)
 				op_canvas->set_file_name(canvas->get_file_name());
 				// Set the time that we wish to render
 				canvas->set_time(t);
-				optimize_layers(canvas->get_time(), canvas->get_context(), op_canvas);
+				optimize_layers(canvas->get_time(), canvas->get_context(context_params), op_canvas);
 				context=op_canvas->get_context();
 				#else
 				Context context;
 				// Set the time that we wish to render
 				canvas->set_time(t);
-				context=canvas->get_context();
+				context=canvas->get_context(context_params);
 				#endif
 	*/
 
@@ -363,7 +365,7 @@ synfig::Target_Tile::render(ProgressCallback *cb)
 				return false;
 
 			// Set the time that we wish to render
-//			if(!get_avoid_time_sync() || canvas->get_time()!=t)
+			//if(!get_avoid_time_sync() || canvas->get_time()!=t)
 				canvas->set_time(t);
 
 			//synfig::info("2time_set_to %s",t.get_string().c_str());
@@ -376,13 +378,13 @@ synfig::Target_Tile::render(ProgressCallback *cb)
 			{
 				op_canvas = Canvas::create();
 				op_canvas->set_file_name(canvas->get_file_name());
-				optimize_layers(canvas->get_time(), canvas->get_context(), op_canvas);
-				context=op_canvas->get_context();
+				optimize_layers(canvas->get_time(), canvas->get_context(context_params), op_canvas);
+				context=op_canvas->get_context(context_params);
 			}
 			else
-				context=canvas->get_context();
+				context=canvas->get_context(context_params);
 #else
-			context=canvas->get_context();
+			context=canvas->get_context(context_params);
 #endif
 
 			if(!render_frame_(context, cb))
