@@ -38,9 +38,11 @@
 #include <ETL/handle>
 #include "general.h"
 #include "blinepoint.h"
+#include "bone.h"
 #include "widthpoint.h"
 #include "dashitem.h"
 #include "exception.h"
+#include "interpolation.h"
 
 #ifdef USE_HALF_TYPE
 #include <OpenEXR/half.h>
@@ -66,6 +68,10 @@ class BLinePoint;
 class WidthPoint;
 class DashItem;
 class Color;
+class Bone;
+class ValueNode_Bone;
+class Matrix;
+class BoneWeightPair;
 
 /*!	\class ValueBase
 **	\brief Base class for the Values of Synfig
@@ -98,6 +104,8 @@ public:
 		TYPE_COLOR,			//!< Color (Real, Real, Real, Real)
 		TYPE_SEGMENT,		//!< Segment Point and Vector
 		TYPE_BLINEPOINT,	//!< BLinePoint Origin (Point) 2xTangents (Vector) Width (Real), Origin (Real) Split Tangent (Boolean)
+		TYPE_MATRIX,		//!< Matrix
+		TYPE_BONE_WEIGHT_PAIR,	//!< pair<Bone,Real>
 		TYPE_WIDTHPOINT,	//!< WidthPoint Position (Real), Width (Real), 2xSide Type (int enum)
 		TYPE_DASHITEM,		//!< DashItem Offset (Real distance), Length (Real distance), 2xSide Type (int enum)
 
@@ -107,6 +115,8 @@ public:
 		TYPE_CANVAS,		//!< Canvas
 		TYPE_STRING,		//!< String
 		TYPE_GRADIENT,		//!< Color Gradient
+		TYPE_BONE,			//!< Bone
+		TYPE_VALUENODE_BONE,//!< ValueNode_Bone
 
 		TYPE_END			//!< Not a valid type, used for sanity checks
 	};
@@ -132,6 +142,8 @@ protected:
 	bool loop_;
 	//! For Values of Constant Value Nodes
 	bool static_;
+	//! Parameter interpolation
+	Interpolation interpolation_;
 
 	/*
  --	** -- C O N S T R U C T O R S -----------------------------------
@@ -145,7 +157,8 @@ public:
 	//! Template constructor for any type
 	template <typename T>
 	ValueBase(const T &x, bool loop_=false, bool static_=false):
-		type(TYPE_NIL),data(0),ref_count(0),loop_(loop_), static_(static_)
+		type(TYPE_NIL),data(0),ref_count(0),loop_(loop_), static_(static_),
+		interpolation_(INTERPOLATION_UNDEFINED)
 		{ set(x); }
 
 	//! Copy constructor. The data is not copied, just the type.
@@ -199,6 +212,18 @@ public:
 	//! Sets the static option.
 	void set_static(bool x) { static_=x; }
 
+	//! Gets the interpolation.
+	Interpolation get_interpolation()const { return interpolation_; }
+
+	//! Sets the interpolation.
+	void set_interpolation(Interpolation x) { interpolation_=x; }
+	
+	//! Create independent copy from existing ValueBase object
+	void copy(const ValueBase& x);
+
+	//! Copies properties (static, interpolation, etc) from other ValueBase object.
+	void copy_properties_of(const ValueBase& x);
+
 	//! True if the Value is not valid or is type LIST and is empty
 	bool empty()const;
 
@@ -245,6 +270,12 @@ public:
 	template <typename T>
 	const T &get(const T& x __attribute__ ((unused)))const
 	{
+#ifdef _DEBUG
+		if (!is_valid())
+			printf("%s:%d !is_valid()\n", __FILE__, __LINE__);
+		else if (!same_type_as(x))
+			printf("%s:%d !'%s'.same_type_as('%s')\n", __FILE__, __LINE__, type_name(type).c_str(), type_name(get_type(x)).c_str());
+#endif
 		assert(is_valid() && same_type_as(x));
 		return *static_cast<const T*>(data);
 	}
@@ -339,10 +370,15 @@ public:
 	static Type get_type(const Color&) { return TYPE_COLOR; }
 	static Type get_type(const Segment&) { return TYPE_SEGMENT; }
 	static Type get_type(const BLinePoint&) { return TYPE_BLINEPOINT; }
+	static Type get_type(const Matrix&) {return TYPE_MATRIX;}
+	static Type get_type(const BoneWeightPair&) {return TYPE_BONE_WEIGHT_PAIR;}
 	static Type get_type(const WidthPoint&) { return TYPE_WIDTHPOINT; }
 	static Type get_type(const DashItem&) { return TYPE_DASHITEM; }
 	static Type get_type(const String&) { return TYPE_STRING; }
 	static Type get_type(const Gradient&) { return TYPE_GRADIENT; }
+	static Type get_type(const Bone&) { return TYPE_BONE; }
+	static Type get_type(const etl::handle<ValueNode_Bone>&) { return TYPE_VALUENODE_BONE; }
+	static Type get_type(const etl::loose_handle<ValueNode_Bone>&) { return TYPE_VALUENODE_BONE; }
 	static Type get_type(Canvas*) { return TYPE_CANVAS; }
 	static Type get_type(const etl::handle<Canvas>&)
 		{ return TYPE_CANVAS; }
@@ -370,12 +406,14 @@ public:
 
 	operator const Vector&()const {  return get(Vector()); }
 	operator const BLinePoint&()const {  return get(BLinePoint()); }
+	operator const Matrix&()const { return get(Matrix()); }
 	operator const WidthPoint&()const {  return get(WidthPoint()); }
 	operator const DashItem&()const {  return get(DashItem()); }
 	//operator const int&()const {  return get(int()); }
 	//operator const String&()const {  return get(String()); }
 	//operator const char *()const {  return get(String()).c_str(); }
 	operator const Segment&()const { return get(Segment()); }
+	operator const Bone&()const { return get(Bone()); }
 
 
 	/*
@@ -433,6 +471,10 @@ private:
 
 		type=newtype;
 		ref_count.reset();
+
+//		if (type == TYPE_BONE && &x == 0)
+//			data = 0;
+//		else
 		data=new T(x);
 	}
 
