@@ -7,6 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2007, 2008 Chris Moore
+**  Copyright (c) 2011 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -56,13 +57,10 @@ public:
 	virtual ValueBase operator()(Time t)const;
 	virtual bool set_link_vfunc(int i,ValueNode::Handle x);
 	virtual ValueNode::LooseHandle get_link_vfunc(int i)const;
-	virtual int link_count()const;
-	virtual String link_local_name(int i)const;
-	virtual String link_name(int i)const;
-	virtual int get_link_index_from_name(const String &name)const;
 	virtual String get_name()const;
 	virtual String get_local_name()const;
 	static bool check_type(ValueBase::Type type);
+	virtual Vocab get_children_vocab_vfunc()const;
 }; // END of class ValueNode_Add
 
 }; // END of namespace synfig
