@@ -45,7 +45,7 @@ private:
 	ValueBase value;
 
 	ValueNode_Const();
-	ValueNode_Const(const ValueBase &x);
+	ValueNode_Const(const ValueBase &x, etl::loose_handle<Canvas> canvas = 0);
 
 public:
 
@@ -58,13 +58,19 @@ public:
 
 	bool get_static()const {return get_value().get_static();}
 	void set_static(bool x) { get_value().set_static(x); }
+	virtual Interpolation get_interpolation()const {return get_value().get_interpolation();}
+	virtual void set_interpolation(Interpolation x) { get_value().set_interpolation(x); }
 	virtual String get_name()const;
 	virtual String get_local_name()const;
 
-	virtual ValueNode* clone(const GUID& deriv_guid=GUID())const;
-
+	virtual ValueNode* clone(etl::loose_handle<Canvas> canvas, const GUID& deriv_guid=GUID())const;
+#ifdef _DEBUG
+	String get_string()const;
+#endif	// _DEBUG
 public:
-	static ValueNode_Const* create(const ValueBase &x=ValueBase());
+	// create a new ValueNode_Const object with the given value.
+	// Unless the given value is a Bone, in which case make a ValueNode_Bone.
+	static ValueNode* create(const ValueBase &x=ValueBase(), etl::loose_handle<Canvas> canvas = 0);
 
 protected:
 	virtual void get_times_vfunc(Node::time_set &set) const;
