@@ -7,6 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2008 Chris Moore
+**  Copyright (c) 2011 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -53,6 +54,8 @@ using namespace synfig;
 ValueNode_DotProduct::ValueNode_DotProduct(const ValueBase &value):
 	LinkableValueNode(value.get_type())
 {
+	Vocab ret(get_children_vocab());
+	set_children_vocab(ret);
 	switch(value.get_type())
 	{
 	case ValueBase::TYPE_REAL:
@@ -66,8 +69,6 @@ ValueNode_DotProduct::ValueNode_DotProduct(const ValueBase &value):
 	default:
 		throw Exception::BadType(ValueBase::type_local_name(value.get_type()));
 	}
-
-	DCAST_HACK_ENABLE();
 }
 
 LinkableValueNode*
@@ -149,47 +150,6 @@ ValueNode_DotProduct::get_link_vfunc(int i)const
 	return 0;
 }
 
-int
-ValueNode_DotProduct::link_count()const
-{
-	return 2;
-}
-
-String
-ValueNode_DotProduct::link_name(int i)const
-{
-	assert(i>=0 && i<link_count());
-
-	switch(i)
-	{
-		case 0: return "lhs";
-		case 1: return "rhs";
-	}
-	return String();
-}
-
-String
-ValueNode_DotProduct::link_local_name(int i)const
-{
-	assert(i>=0 && i<link_count());
-
-	switch(i)
-	{
-		case 0: return _("LHS");
-		case 1: return _("RHS");
-	}
-	return String();
-}
-
-int
-ValueNode_DotProduct::get_link_index_from_name(const String &name)const
-{
-	if (name=="lhs") return 0;
-	if (name=="rhs") return 1;
-
-	throw Exception::BadLinkName(name);
-}
-
 bool
 ValueNode_DotProduct::check_type(ValueBase::Type type)
 {
@@ -197,3 +157,24 @@ ValueNode_DotProduct::check_type(ValueBase::Type type)
 		type==ValueBase::TYPE_ANGLE ||
 		type==ValueBase::TYPE_REAL;
 }
+
+LinkableValueNode::Vocab
+ValueNode_DotProduct::get_children_vocab_vfunc()const
+{
+	if(children_vocab.size())
+		return children_vocab;
+
+	LinkableValueNode::Vocab ret;
+
+	ret.push_back(ParamDesc(ValueBase(),"lhs")
+		.set_local_name(_("LHS"))
+		.set_description(_("The left side of the dot product"))
+	);
+
+	ret.push_back(ParamDesc(ValueBase(),"rhs")
+		.set_local_name(_("RHS"))
+		.set_description(_("The right side of the dot product"))
+	);
+
+	return ret;
+}
