@@ -240,6 +240,7 @@ ValueNode_DynamicList::create_list_entry(int index, Time time, Real origin)
 		break;
 	}
 
+	ret.value_node->set_parent_canvas(get_parent_canvas());
 
 	return ret;
 }
@@ -576,8 +577,10 @@ ValueNode_DynamicList::erase(const ValueNode::Handle &value_node_)
 				// 	get_parent_canvas()->signal_value_node_child_removed()(this,value_node);
 				// else if(get_root_canvas() && get_parent_canvas())
 				//	get_root_canvas()->signal_value_node_child_removed()(this,value_node);
-				if(get_root_canvas())
-					get_root_canvas()->signal_value_node_child_removed()(this,value_node);
+				if(get_non_inline_ancestor_canvas())
+					get_non_inline_ancestor_canvas()->invoke_signal_value_node_child_removed(this,value_node);
+				else
+					printf("%s:%d == can't get non_inline_ancestor_canvas - parent canvas = %lx\n", __FILE__, __LINE__, uintptr_t(get_parent_canvas().get()));
 			}
 			break;
 		}
@@ -585,17 +588,20 @@ ValueNode_DynamicList::erase(const ValueNode::Handle &value_node_)
 }
 
 
-ValueNode_DynamicList::ValueNode_DynamicList(ValueBase::Type container_type):
+ValueNode_DynamicList::ValueNode_DynamicList(ValueBase::Type container_type, Canvas::LooseHandle canvas):
 	LinkableValueNode(ValueBase::TYPE_LIST),
 	container_type	(container_type),
 	loop_(false)
 {
+	if (getenv("SYNFIG_DEBUG_SET_PARENT_CANVAS"))
+		printf("%s:%d set parent canvas for dynamic_list %lx to %lx\n", __FILE__, __LINE__, uintptr_t(this), uintptr_t(canvas.get()));
+	set_parent_canvas(canvas);
 }
 
 ValueNode_DynamicList::Handle
-ValueNode_DynamicList::create(ValueBase::Type id)
+ValueNode_DynamicList::create(ValueBase::Type id, Canvas::LooseHandle canvas)
 {
-	return new ValueNode_DynamicList(id);
+	return new ValueNode_DynamicList(id, canvas);
 }
 
 ValueNode_DynamicList::~ValueNode_DynamicList()
@@ -704,7 +710,7 @@ ValueNode_DynamicList::link_local_name(int i)const
 }
 
 ValueNode*
-ValueNode_DynamicList::clone(const GUID& deriv_guid)const
+ValueNode_DynamicList::clone(Canvas::LooseHandle canvas, const GUID& deriv_guid)const
 {
 	{ ValueNode* x(find_value_node(get_guid()^deriv_guid).get()); if(x)return x; }
 
@@ -722,12 +728,13 @@ ValueNode_DynamicList::clone(const GUID& deriv_guid)const
 			ListEntry list_entry(*iter);
 			//list_entry.value_node=find_value_node(iter->value_node->get_guid()^deriv_guid).get();
 			//if(!list_entry.value_node)
-			list_entry.value_node=iter->value_node->clone(deriv_guid);
+			list_entry.value_node=iter->value_node->clone(canvas, deriv_guid);
 			ret->add(list_entry);
 			//ret->list.back().value_node=iter->value_node.clone();
 		}
 	}
 	ret->set_loop(get_loop());
+	ret->set_parent_canvas(canvas);
 	return ret;
 }
 
