@@ -7,7 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2007, 2008 Chris Moore
-**  Copyright (c) 2008 Carlos López
+**  Copyright (c) 2008, 2011 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -54,6 +54,8 @@ using namespace synfig;
 ValueNode_Logarithm::ValueNode_Logarithm(const ValueBase &x):
 	LinkableValueNode(x.get_type())
 {
+	Vocab ret(get_children_vocab());
+	set_children_vocab(ret);
 	Real value(x.get(Real()));
 	Real infinity(999999.0);
 	Real epsilon(0.000001);
@@ -108,44 +110,6 @@ ValueNode_Logarithm::get_link_vfunc(int i)const
 	return 0;
 }
 
-int
-ValueNode_Logarithm::link_count()const
-{
-	return 3;
-}
-
-String
-ValueNode_Logarithm::link_local_name(int i)const
-{
-	assert(i>=0 && i<link_count());
-
-	if(i==0) return _("Link");
-	if(i==1) return _("Epsilon");
-	if(i==2) return _("Infinite");
-	return String();
-}
-
-String
-ValueNode_Logarithm::link_name(int i)const
-{
-	assert(i>=0 && i<link_count());
-
-	if(i==0) return "link";
-	if(i==1) return "epsilon";
-	if(i==2) return "infinite";
-	return String();
-}
-
-int
-ValueNode_Logarithm::get_link_index_from_name(const String &name)const
-{
-	if(name=="link")     return 0;
-	if(name=="epsilon")  return 1;
-	if(name=="infinite") return 2;
-
-	throw Exception::BadLinkName(name);
-}
-
 ValueBase
 ValueNode_Logarithm::operator()(Time t)const
 {
@@ -183,4 +147,29 @@ ValueNode_Logarithm::check_type(ValueBase::Type type)
 	return type==ValueBase::TYPE_REAL;
 }
 
+LinkableValueNode::Vocab
+ValueNode_Logarithm::get_children_vocab_vfunc()const
+{
+	if(children_vocab.size())
+		return children_vocab;
+
+	LinkableValueNode::Vocab ret;
+
+	ret.push_back(ParamDesc(ValueBase(),"link")
+		.set_local_name(_("Link"))
+		.set_description(_("Value node used to calculate the Neperian logarithm"))
+	);
+
+		ret.push_back(ParamDesc(ValueBase(),"epsilon")
+		.set_local_name(_("Epsilon"))
+		.set_description(_("Value used to compare 'link' with zero "))
+	);
+
+		ret.push_back(ParamDesc(ValueBase(),"infinite")
+		.set_local_name(_("Infinite"))
+		.set_description(_("Returned value when result tends to infinite"))
+	);
+
+	return ret;
+}
 
