@@ -7,6 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2008 Chris Moore
+**  Copyright (c) 2011 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -60,12 +61,6 @@ public:
 
 	virtual ValueNode::LooseHandle get_link_vfunc(int i)const;
 
-	virtual int link_count()const;
-
-	virtual String link_local_name(int i)const;
-	virtual String link_name(int i)const;
-	virtual int get_link_index_from_name(const String &name)const;
-
 	virtual ValueBase operator()(Time t)const;
 
 	virtual String get_name()const;
@@ -80,6 +75,7 @@ public:
 	using synfig::LinkableValueNode::set_link_vfunc;
 	static bool check_type(ValueBase::Type type);
 	static ValueNode_Stripes* create(const ValueBase &x=ValueBase::TYPE_GRADIENT);
+	virtual Vocab get_children_vocab_vfunc()const;
 }; // END of class ValueNode_Stripes
 
 }; // END of namespace synfig
